@@ -64,9 +64,6 @@ private:
 class ItemEncryptedScriptable : public ItemScriptable
 {
     Q_OBJECT
-public:
-    explicit ItemEncryptedScriptable(QObject *parent) : ItemScriptable(parent) {}
-
 public slots:
     bool isEncrypted();
     QByteArray encrypt();
@@ -79,6 +76,7 @@ public slots:
     void decryptItems();
 
     void copyEncryptedItems();
+    void pasteEncryptedItems();
 
     QString generateTestKeys();
     bool isGpgInstalled();
@@ -99,7 +97,7 @@ public:
 
     ~ItemEncryptedLoader();
 
-    ItemWidget *create(const QModelIndex &index, QWidget *parent, bool) const override;
+    ItemWidget *create(const QVariantMap &data, QWidget *parent, bool) const override;
 
     QString id() const override { return "itemencrypted"; }
     QString name() const override { return tr("Encryption"); }
@@ -127,19 +125,17 @@ public:
 
     const QObject *signaler() const override { return this; }
 
-    ItemScriptable *scriptableObject(QObject *parent) override;
+    ItemScriptable *scriptableObject() override;
 
-    QList<Command> commands() const override;
+    QVector<Command> commands() const override;
 
 signals:
     void error(const QString &);
-    void addCommands(const QList<Command> &commands);
 
 private slots:
     void setPassword();
     void terminateGpgProcess();
     void onGpgProcessFinished(int exitCode, QProcess::ExitStatus exitStatus);
-    void addCommands();
 
 private:
     enum GpgProcessStatus {
