@@ -92,7 +92,7 @@ public:
     QString name() const override { return tr("Notes"); }
     QString author() const override { return QString(); }
     QString description() const override { return tr("Display notes for items."); }
-    QVariant icon() const override { return QVariant(IconEditSign); }
+    QVariant icon() const override { return QVariant(IconPenSquare); }
 
     QStringList formatsToSave() const override;
 
@@ -102,7 +102,7 @@ public:
 
     QWidget *createSettingsWidget(QWidget *parent) override;
 
-    ItemWidget *transform(ItemWidget *itemWidget, const QModelIndex &index) override;
+    ItemWidget *transform(ItemWidget *itemWidget, const QVariantMap &data) override;
 
     bool matches(const QModelIndex &index, const QRegExp &re) const override;
 
