@@ -18,7 +18,6 @@
 */
 
 #include "itempinned.h"
-#include "ui_itempinnedsettings.h"
 
 #include "common/command.h"
 #include "common/contenttype.h"
@@ -48,10 +47,8 @@ bool isPinned(const QModelIndex &index)
 Command dummyPinCommand()
 {
     Command c;
-    c.icon = QString(QChar(IconThumbTack));
+    c.icon = QString(QChar(IconThumbtack));
     c.inMenu = true;
-    c.shortcuts = QStringList()
-            << ItemPinnedLoader::tr("Ctrl+Shift+P", "Shortcut to pin and unpin items");
     return c;
 }
 
@@ -159,22 +156,30 @@ bool ItemPinnedScriptable::isPinned()
 void ItemPinnedScriptable::pin()
 {
     const auto args = currentArguments();
-    for (const auto &arg : args) {
-        bool ok;
-        const int row = arg.toInt(&ok);
-        if (ok)
-            call("change", QVariantList() << row << mimePinned << QString());
+    if (args.isEmpty()) {
+        pinData();
+    } else {
+        for (const auto &arg : args) {
+            bool ok;
+            const int row = arg.toInt(&ok);
+            if (ok)
+                call("change", QVariantList() << row << mimePinned << QString());
+        }
     }
 }
 
 void ItemPinnedScriptable::unpin()
 {
     const auto args = currentArguments();
-    for (const auto &arg : args) {
-        bool ok;
-        const int row = arg.toInt(&ok);
-        if (ok)
-            call("change", QVariantList() << row << mimePinned << QVariant());
+    if (args.isEmpty()) {
+        unpinData();
+    } else {
+        for (const auto &arg : args) {
+            bool ok;
+            const int row = arg.toInt(&ok);
+            if (ok)
+                call("change", QVariantList() << row << mimePinned << QVariant());
+        }
     }
 }
 
@@ -369,19 +374,12 @@ QVariantMap ItemPinnedLoader::applySettings()
 
 QWidget *ItemPinnedLoader::createSettingsWidget(QWidget *parent)
 {
-    ui.reset(new Ui::ItemPinnedSettings);
-    QWidget *w = new QWidget(parent);
-    ui->setupUi(w);
-
-    connect( ui->pushButtonAddCommands, SIGNAL(clicked()),
-             this, SLOT(addCommands()) );
-
-    return w;
+    return new QWidget(parent);
 }
 
-ItemWidget *ItemPinnedLoader::transform(ItemWidget *itemWidget, const QModelIndex &index)
+ItemWidget *ItemPinnedLoader::transform(ItemWidget *itemWidget, const QVariantMap &data)
 {
-    return isPinned(index) ? new ItemPinned(itemWidget) : nullptr;
+    return data.contains(mimePinned) ? new ItemPinned(itemWidget) : nullptr;
 }
 
 ItemSaverPtr ItemPinnedLoader::transformSaver(const ItemSaverPtr &saver, QAbstractItemModel *model)
@@ -400,14 +398,14 @@ QObject *ItemPinnedLoader::tests(const TestInterfacePtr &test) const
 #endif
 }
 
-ItemScriptable *ItemPinnedLoader::scriptableObject(QObject *parent)
+ItemScriptable *ItemPinnedLoader::scriptableObject()
 {
-    return new ItemPinnedScriptable(parent);
+    return new ItemPinnedScriptable();
 }
 
-QList<Command> ItemPinnedLoader::commands() const
+QVector<Command> ItemPinnedLoader::commands() const
 {
-    QList<Command> commands;
+    QVector<Command> commands;
 
     Command c;
 
@@ -415,21 +413,16 @@ QList<Command> ItemPinnedLoader::commands() const
     c.name = tr("Pin");
     c.input = "!OUTPUT";
     c.output = mimePinned;
-    c.cmd = "copyq: plugins.itempinned.pinData()";
+    c.cmd = "copyq: plugins.itempinned.pin()";
     commands.append(c);
 
     c = dummyPinCommand();
     c.name = tr("Unpin");
     c.input = mimePinned;
-    c.cmd = "copyq: plugins.itempinned.unpinData()";
+    c.cmd = "copyq: plugins.itempinned.unpin()";
     commands.append(c);
 
     return commands;
 }
 
-void ItemPinnedLoader::addCommands()
-{
-    emit addCommands(commands());
-}
-
 Q_EXPORT_PLUGIN2(itempinned, ItemPinnedLoader)
