@@ -27,10 +27,6 @@
 
 #include <memory>
 
-namespace Ui {
-class ItemPinnedSettings;
-}
-
 class ItemPinned : public QWidget, public ItemWidget
 {
     Q_OBJECT
@@ -63,9 +59,6 @@ private:
 class ItemPinnedScriptable : public ItemScriptable
 {
     Q_OBJECT
-public:
-    explicit ItemPinnedScriptable(QObject *parent) : ItemScriptable(parent) {}
-
 public slots:
     bool isPinned();
 
@@ -127,7 +120,7 @@ public:
     QString description() const override {
         return tr("Pin items to lock them in current row and avoid deletion (unless unpinned).");
     }
-    QVariant icon() const override { return QVariant(IconThumbTack); }
+    QVariant icon() const override { return QVariant(IconThumbtack); }
 
     QStringList formatsToSave() const override;
 
@@ -137,7 +130,7 @@ public:
 
     QWidget *createSettingsWidget(QWidget *parent) override;
 
-    ItemWidget *transform(ItemWidget *itemWidget, const QModelIndex &index) override;
+    ItemWidget *transform(ItemWidget *itemWidget, const QVariantMap &data) override;
 
     ItemSaverPtr transformSaver(const ItemSaverPtr &saver, QAbstractItemModel *model) override;
 
@@ -145,19 +138,12 @@ public:
 
     const QObject *signaler() const override { return this; }
 
-    ItemScriptable *scriptableObject(QObject *parent) override;
+    ItemScriptable *scriptableObject() override;
 
-    QList<Command> commands() const override;
-
-signals:
-    void addCommands(const QList<Command> &commands);
-
-private slots:
-    void addCommands();
+    QVector<Command> commands() const override;
 
 private:
     QVariantMap m_settings;
-    std::unique_ptr<Ui::ItemPinnedSettings> ui;
     ItemLoaderPtr m_transformedLoader;
 };
 
