@@ -204,15 +204,15 @@ int iconFromBaseNameExtensionHelper(const QString &baseName)
     if (i != -1)  {
         const QString ext = baseName.mid(i + 1);
         if ( hasVideoExtension(ext) )
-            return IconPlayCircle;
+            return IconFileVideo;
         if ( hasAudioExtension(ext) )
-            return IconVolumeUp;
+            return IconFileAudio;
         if ( hasImageExtension(ext) )
-            return IconCamera;
+            return IconFileImage;
         if ( hasArchiveExtension(ext) )
-            return IconFileText;
+            return IconFileArchive;
         if ( hasTextExtension(ext) )
-            return IconFileText;
+            return IconFileAlt;
     }
 
     return -1;
@@ -227,7 +227,7 @@ int iconFromMimeHelper(const QString &format)
     if ( format.startsWith("image/") )
         return IconCamera;
     if ( format.startsWith("text/") )
-        return IconFileText;
+        return IconFileAlt;
     return -1;
 }
 
@@ -253,10 +253,8 @@ QString iconFromMime(const QString &format)
     return iconFromId(iconFromMimeHelper(format));
 }
 
-QString iconForItem(const QModelIndex &index, const QList<FileFormat> &formatSettings)
+QString iconForItem(const QVariantMap &dataMap, const QString &baseName, const QList<FileFormat> &formatSettings)
 {
-    const QString baseName = FileWatcher::getBaseName(index);
-    const QVariantMap dataMap = index.data(contentType::data).toMap();
     const QVariantMap mimeToExtension = dataMap.value(mimeExtensionMap).toMap();
 
     QStringList fileNames;
@@ -293,6 +291,11 @@ QString iconForItem(const QModelIndex &index, const QList<FileFormat> &formatSet
  * Return true only if the item was created by CopyQ
  * (i.e. has no file assigned or the file name matches internal format).
  */
+bool isOwnItem(const QString &baseName)
+{
+    return baseName.isEmpty() || FileWatcher::isOwnBaseName(baseName);
+}
+
 bool isOwnItem(const QModelIndex &index)
 {
     const QString baseName = FileWatcher::getBaseName(index);
@@ -619,17 +622,15 @@ QVariantMap ItemSyncSaver::copyItem(const QAbstractItemModel &, const QVariantMa
     return copiedItemData;
 }
 
-ItemSyncScriptable::ItemSyncScriptable(
-        const ItemSyncTabPaths &tabPaths, QObject *parent)
-    : ItemScriptable(parent)
+QString ItemSyncScriptable::getMimeBaseName() const
 {
-    for (auto it = tabPaths.constBegin(); it != tabPaths.constEnd(); ++it)
-        m_tabPaths.insert( it.key(), it.value() );
+    return mimeBaseName;
 }
 
-QVariantMap ItemSyncScriptable::getTabPaths() const
+QString ItemSyncScriptable::selectedTabPath()
 {
-    return m_tabPaths;
+    const auto tab = call("selectedTab", QVariantList()).toString();
+    return m_tabPaths.value(tab).toString();
 }
 
 ItemSyncLoader::ItemSyncLoader()
@@ -774,14 +775,15 @@ ItemSaverPtr ItemSyncLoader::initializeTab(const QString &tabName, QAbstractItem
     return loadItems(tabName, model, QStringList(), maxItems);
 }
 
-ItemWidget *ItemSyncLoader::transform(ItemWidget *itemWidget, const QModelIndex &index)
+ItemWidget *ItemSyncLoader::transform(ItemWidget *itemWidget, const QVariantMap &data)
 {
-    if ( isOwnItem(index) )
+    const auto baseName = FileWatcher::getBaseName(data);
+    if ( isOwnItem(baseName) )
         return nullptr;
 
     itemWidget->setTagged(true);
-    const QString baseName = FileWatcher::getBaseName(index);
-    return new ItemSync(baseName, iconForItem(index, m_formatSettings), itemWidget);
+    const auto icon = iconForItem(data, baseName, m_formatSettings);
+    return new ItemSync(baseName, icon, itemWidget);
 }
 
 bool ItemSyncLoader::matches(const QModelIndex &index, const QRegExp &re) const
@@ -826,9 +828,12 @@ QObject *ItemSyncLoader::tests(const TestInterfacePtr &test) const
 #endif
 }
 
-ItemScriptable *ItemSyncLoader::scriptableObject(QObject *parent)
+ItemScriptable *ItemSyncLoader::scriptableObject()
 {
-    return new ItemSyncScriptable(m_tabPaths, parent);
+    QVariantMap tabPaths;
+    for (auto it = m_tabPaths.constBegin(); it != m_tabPaths.constEnd(); ++it)
+        tabPaths.insert( it.key(), it.value() );
+    return new ItemSyncScriptable(tabPaths);
 }
 
 void ItemSyncLoader::onBrowseButtonClicked()
