@@ -106,11 +106,20 @@ class ItemSyncScriptable : public ItemScriptable
 {
     Q_OBJECT
     Q_PROPERTY(QVariantMap tabPaths READ getTabPaths)
+    Q_PROPERTY(QString mimeBaseName READ getMimeBaseName)
 
 public:
-    ItemSyncScriptable(const ItemSyncTabPaths &tabPaths, QObject *parent);
+    explicit ItemSyncScriptable(const QVariantMap &tabPaths)
+        : m_tabPaths(tabPaths)
+    {
+    }
 
-    QVariantMap getTabPaths() const;
+    QVariantMap getTabPaths() const { return m_tabPaths; }
+
+    QString getMimeBaseName() const;
+
+public slots:
+    QString selectedTabPath();
 
 private:
     QVariantMap m_tabPaths;
@@ -150,7 +159,7 @@ public:
     QString name() const override { return tr("Synchronize"); }
     QString author() const override { return QString(); }
     QString description() const override { return tr("Synchronize items and notes with a directory on disk."); }
-    QVariant icon() const override { return QVariant(IconUploadAlt); }
+    QVariant icon() const override { return QVariant(IconUpload); }
 
     QVariantMap applySettings() override;
 
@@ -166,7 +175,7 @@ public:
 
     ItemSaverPtr initializeTab(const QString &tabName, QAbstractItemModel *model, int maxItems) override;
 
-    ItemWidget *transform(ItemWidget *itemWidget, const QModelIndex &index) override;
+    ItemWidget *transform(ItemWidget *itemWidget, const QVariantMap &data) override;
 
     bool matches(const QModelIndex &index, const QRegExp &re) const override;
 
@@ -174,7 +183,7 @@ public:
 
     const QObject *signaler() const override { return this; }
 
-    ItemScriptable *scriptableObject(QObject *parent) override;
+    ItemScriptable *scriptableObject() override;
 
 signals:
     void error(const QString &);
