@@ -142,11 +142,15 @@ QVariant cellWidgetProperty(QTableWidget *table, int row, int column, const char
     return table->cellWidget(row, column)->property(property);
 }
 
-QString tags(const QModelIndex &index)
+QStringList tags(const QVariant &tags)
 {
-    const QByteArray tagsData =
-            index.data(contentType::data).toMap().value(mimeTags).toByteArray();
-    return getTextData(tagsData);
+    return getTextData( tags.toByteArray() )
+            .split(',', QString::SkipEmptyParts);
+}
+
+QStringList tags(const QVariantMap &itemData)
+{
+    return tags( itemData.value(mimeTags) );
 }
 
 QString toScriptString(const QString &text)
@@ -172,7 +176,7 @@ Command dummyTagCommand()
     return c;
 }
 
-void addTagCommands(const QString &tagName, const QString &match, QList<Command> *commands)
+void addTagCommands(const QString &tagName, const QString &match, QVector<Command> *commands)
 {
     Command c;
 
@@ -406,17 +410,16 @@ void ItemTags::updateSize(QSize maximumSize, int idealWidth)
     setFixedHeight(m_childItem->widget()->height() + m_tagWidget->height());
 }
 
-ItemTagsScriptable::ItemTagsScriptable(const QStringList &userTags, QObject *parent)
-    : ItemScriptable(parent)
-    , m_userTags(userTags)
-{
-}
-
 QStringList ItemTagsScriptable::getUserTags() const
 {
     return m_userTags;
 }
 
+QString ItemTagsScriptable::getMimeTags() const
+{
+    return mimeTags;
+}
+
 QStringList ItemTagsScriptable::tags()
 {
     const auto args = currentArguments();
@@ -447,7 +450,7 @@ void ItemTagsScriptable::tag()
         dataList.reserve( dataValueList.size() );
         for (const auto &itemDataValue : dataValueList) {
             auto itemData = itemDataValue.toMap();
-            auto itemTags = tags(itemData);
+            auto itemTags = ::tags(itemData);
             if ( addTag(tagName, &itemTags) )
                 itemData.insert( mimeTags, itemTags.join(",") );
             dataList.append(itemData);
@@ -475,7 +478,7 @@ void ItemTagsScriptable::untag()
             QStringList allTags;
             for (const auto &itemDataValue : dataValueList) {
                 const auto itemData = itemDataValue.toMap();
-                allTags.append( tags(itemData) );
+                allTags.append( ::tags(itemData) );
             }
 
             tagName = askRemoveTagName(allTags);
@@ -487,7 +490,7 @@ void ItemTagsScriptable::untag()
         dataList.reserve( dataValueList.size() );
         for (const auto &itemDataValue : dataValueList) {
             auto itemData = itemDataValue.toMap();
-            auto itemTags = tags(itemData);
+            auto itemTags = ::tags(itemData);
             if ( removeTag(tagName, &itemTags) )
                 itemData.insert( mimeTags, itemTags.join(",") );
             dataList.append(itemData);
@@ -541,6 +544,18 @@ bool ItemTagsScriptable::hasTag()
 {
     const auto args = currentArguments();
     const auto tagName = args.value(0).toString();
+
+    if ( args.size() <= 1 ) {
+        const auto dataValueList = call("selectedItemsData").toList();
+        for (const auto &itemDataValue : dataValueList) {
+            const auto itemData = itemDataValue.toMap();
+            const auto itemTags = ::tags(itemData);
+            if ( itemTags.contains(tagName) )
+                return true;
+        }
+        return false;
+    }
+
     const auto row = args.value(1).toInt();
     return tags(row).contains(tagName);
 }
@@ -582,18 +597,7 @@ QList<int> ItemTagsScriptable::rows(const QVariantList &arguments, int skip)
 QStringList ItemTagsScriptable::tags(int row)
 {
     const auto value = call("read", QVariantList() << mimeTags << row);
-    return tags(value);
-}
-
-QStringList ItemTagsScriptable::tags(const QVariant &tags)
-{
-    return getTextData( tags.toByteArray() )
-            .split(',', QString::SkipEmptyParts);
-}
-
-QStringList ItemTagsScriptable::tags(const QVariantMap &itemData)
-{
-    return tags( itemData.value(mimeTags) );
+    return ::tags(value);
 }
 
 void ItemTagsScriptable::setTags(int row, const QStringList &tags)
@@ -670,9 +674,6 @@ QWidget *ItemTagsLoader::createSettingsWidget(QWidget *parent)
     QWidget *w = new QWidget(parent);
     ui->setupUi(w);
 
-    connect( ui->pushButtonAddCommands, SIGNAL(clicked()),
-             this, SLOT(addCommands()) );
-
     // Init tag table.
     for (const auto &tag : m_tags)
         addTagToSettingsTable(tag);
@@ -692,10 +693,9 @@ QWidget *ItemTagsLoader::createSettingsWidget(QWidget *parent)
     return w;
 }
 
-ItemWidget *ItemTagsLoader::transform(ItemWidget *itemWidget, const QModelIndex &index)
+ItemWidget *ItemTagsLoader::transform(ItemWidget *itemWidget, const QVariantMap &data)
 {
-    const QString tagsContent = tags(index);
-    const Tags tags = toTags(tagsContent);
+    const Tags tags = toTags(::tags(data));
     if ( tags.isEmpty() )
         return nullptr;
 
@@ -705,7 +705,10 @@ ItemWidget *ItemTagsLoader::transform(ItemWidget *itemWidget, const QModelIndex
 
 bool ItemTagsLoader::matches(const QModelIndex &index, const QRegExp &re) const
 {
-    return re.indexIn(tags(index)) != -1;
+    const QByteArray tagsData =
+            index.data(contentType::data).toMap().value(mimeTags).toByteArray();
+    const auto tags = getTextData(tagsData);
+    return re.indexIn(tags) != -1;
 }
 
 QObject *ItemTagsLoader::tests(const TestInterfacePtr &test) const
@@ -731,14 +734,14 @@ QObject *ItemTagsLoader::tests(const TestInterfacePtr &test) const
 #endif
 }
 
-ItemScriptable *ItemTagsLoader::scriptableObject(QObject *parent)
+ItemScriptable *ItemTagsLoader::scriptableObject()
 {
-    return new ItemTagsScriptable(userTags(), parent);
+    return new ItemTagsScriptable(userTags());
 }
 
-QList<Command> ItemTagsLoader::commands() const
+QVector<Command> ItemTagsLoader::commands() const
 {
-    QList<Command> commands;
+    QVector<Command> commands;
 
     if (m_tags.isEmpty()) {
         addTagCommands(tr("Important", "Tag name for example command"), QString(), &commands);
@@ -780,11 +783,6 @@ QStringList ItemTagsLoader::userTags() const
     return tags;
 }
 
-void ItemTagsLoader::addCommands()
-{
-    emit addCommands(commands());
-}
-
 void ItemTagsLoader::onColorButtonClicked()
 {
     QPushButton *button = qobject_cast<QPushButton*>(sender());
@@ -846,11 +844,11 @@ ItemTagsLoader::Tag ItemTagsLoader::deserializeTag(const QString &tagText)
     return tag;
 }
 
-ItemTagsLoader::Tags ItemTagsLoader::toTags(const QString &tagsContent)
+ItemTagsLoader::Tags ItemTagsLoader::toTags(const QStringList &tagList)
 {
     Tags tags;
 
-    for (const auto &tagText : tagsContent.split(',', QString::SkipEmptyParts)) {
+    for (const auto &tagText : tagList) {
         QString tagName = tagText.trimmed();
         Tag tag = findMatchingTag(tagName, m_tags);
 
