@@ -85,12 +85,18 @@ class ItemTagsScriptable : public ItemScriptable
 {
     Q_OBJECT
     Q_PROPERTY(QStringList userTags READ getUserTags)
+    Q_PROPERTY(QString mimeTags READ getMimeTags)
 
 public:
-    explicit ItemTagsScriptable(const QStringList &userTags, QObject *parent);
+    explicit ItemTagsScriptable(const QStringList &userTags)
+        : m_userTags(userTags)
+    {
+    }
 
     QStringList getUserTags() const;
 
+    QString getMimeTags() const;
+
 public slots:
     QStringList tags();
     void tag();
@@ -103,8 +109,6 @@ private:
     QString askRemoveTagName(const QStringList &tags);
     QList<int> rows(const QVariantList &arguments, int skip);
     QStringList tags(int row);
-    QStringList tags(const QVariant &tags);
-    QStringList tags(const QVariantMap &itemData);
     void setTags(int row, const QStringList &tags);
     bool addTag(const QString &tagName, QStringList *tags);
     bool removeTag(const QString &tagName, QStringList *tags);
@@ -136,7 +140,7 @@ public:
 
     QWidget *createSettingsWidget(QWidget *parent) override;
 
-    ItemWidget *transform(ItemWidget *itemWidget, const QModelIndex &index) override;
+    ItemWidget *transform(ItemWidget *itemWidget, const QVariantMap &data) override;
 
     bool matches(const QModelIndex &index, const QRegExp &re) const override;
 
@@ -144,15 +148,9 @@ public:
 
     const QObject *signaler() const override { return this; }
 
-    ItemScriptable *scriptableObject(QObject *parent) override;
+    ItemScriptable *scriptableObject() override;
 
-    QList<Command> commands() const override;
-
-signals:
-    void addCommands(const QList<Command> &commands);
-
-private slots:
-    void addCommands();
+    QVector<Command> commands() const override;
 
 private slots:
     void onColorButtonClicked();
@@ -168,7 +166,7 @@ private:
     static QString serializeTag(const Tag &tag);
     static Tag deserializeTag(const QString &tagText);
 
-    Tags toTags(const QString &tagsContent);
+    Tags toTags(const QStringList &tagList);
 
     void addTagToSettingsTable(const Tag &tag = Tag());
 
