@@ -20,15 +20,14 @@
 #include "itemtext.h"
 #include "ui_itemtextsettings.h"
 
-#include "common/contenttype.h"
 #include "common/mimetypes.h"
+#include "common/textdata.h"
 
 #include <QAbstractTextDocumentLayout>
 #include <QCoreApplication>
 #include <QContextMenuEvent>
 #include <QCursor>
 #include <QMimeData>
-#include <QModelIndex>
 #include <QMouseEvent>
 #include <QScrollBar>
 #include <QTextBlock>
@@ -63,27 +62,30 @@ void removeTrailingNull(QString *text)
         text->chop(1);
 }
 
-bool getRichText(const QModelIndex &index, QString *text)
+bool getRichText(const QVariantMap &dataMap, QString *text)
 {
-    if ( index.data(contentType::hasHtml).toBool() ) {
-        *text = index.data(contentType::html).toString();
+    if ( dataMap.contains(mimeHtml) ) {
+        *text = getTextData(dataMap, mimeHtml);
         return true;
     }
 
-    const QVariantMap dataMap = index.data(contentType::data).toMap();
-    if ( !dataMap.contains(mimeRichText) )
-        return false;
+    if ( dataMap.contains(mimeRichText) ) {
+        *text = getTextData(dataMap, mimeRichText);
+        return true;
+    }
 
-    const QByteArray data = dataMap[mimeRichText].toByteArray();
-    *text = QString::fromUtf8(data);
-
-    return true;
+    return false;
 }
 
-bool getText(const QModelIndex &index, QString *text)
+bool getText(const QVariantMap &dataMap, QString *text)
 {
-    if ( index.data(contentType::hasText).toBool() ) {
-        *text = index.data(contentType::text).toString();
+    if ( dataMap.contains(mimeText) ) {
+        *text = getTextData(dataMap, mimeText);
+        return true;
+    }
+
+    if ( dataMap.contains(mimeUriList) ) {
+        *text = getTextData(dataMap, mimeUriList);
         return true;
     }
 
@@ -106,7 +108,7 @@ void insertEllipsis(QTextCursor *tc)
 
 } // namespace
 
-ItemText::ItemText(const QString &text, const QString &richText, int maxLines, int maxLineLength, int maximumHeight, QWidget *parent)
+ItemText::ItemText(const QString &text, const QString &richText, int maxLines, int lineLength, int maximumHeight, QWidget *parent)
     : QTextEdit(parent)
     , ItemWidget(this)
     , m_textDocument()
@@ -150,11 +152,11 @@ ItemText::ItemText(const QString &text, const QString &richText, int maxLines, i
         }
     }
 
-    if (maxLineLength > 0) {
+    if (lineLength > 0) {
         for ( auto block = m_textDocument.begin(); block.isValid(); block = block.next() ) {
-            if ( block.length() > maxLineLength ) {
+            if ( block.length() > lineLength ) {
                 QTextCursor tc(&m_textDocument);
-                tc.setPosition(block.position() + maxLineLength);
+                tc.setPosition(block.position() + lineLength);
                 tc.setPosition(block.position() + block.length() - 1, QTextCursor::KeepAnchor);
                 insertEllipsis(&tc);
             }
@@ -223,8 +225,14 @@ void ItemText::updateSize(QSize maximumSize, int idealWidth)
 
     QTextCursor tc(&m_textDocument);
     tc.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
-    const auto h = static_cast<int>( cursorRect(tc).bottom() + 4 * logicalDpiY() / 96.0 );
-    setFixedHeight(0 < m_maximumHeight && m_maximumHeight < h ? m_maximumHeight : h);
+    const auto h = static_cast<int>( cursorRect(tc).bottom() + 2 * logicalDpiY() / 96.0 );
+    if (0 < m_maximumHeight && m_maximumHeight < h) {
+        setFixedHeight(m_maximumHeight);
+        setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+    } else {
+        setFixedHeight(h);
+        setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    }
 }
 
 bool ItemText::eventFilter(QObject *, QEvent *event)
@@ -272,17 +280,17 @@ ItemTextLoader::ItemTextLoader()
 
 ItemTextLoader::~ItemTextLoader() = default;
 
-ItemWidget *ItemTextLoader::create(const QModelIndex &index, QWidget *parent, bool preview) const
+ItemWidget *ItemTextLoader::create(const QVariantMap &data, QWidget *parent, bool preview) const
 {
-    if ( index.data(contentType::isHidden).toBool() )
+    if ( data.value(mimeHidden).toBool() )
         return nullptr;
 
     QString richText;
     const bool isRichText = m_settings.value(optionUseRichText, true).toBool()
-            && getRichText(index, &richText);
+            && getRichText(data, &richText);
 
     QString text;
-    const bool isPlainText = getText(index, &text);
+    const bool isPlainText = getText(data, &text);
 
     if (!isRichText && !isPlainText)
         return nullptr;
