@@ -38,7 +38,7 @@ class ItemText : public QTextEdit, public ItemWidget
     Q_OBJECT
 
 public:
-    ItemText(const QString &text, const QString &richText, int maxLines, int maxLineLength, int maximumHeight, QWidget *parent);
+    ItemText(const QString &text, const QString &richText, int maxLines, int lineLength, int maximumHeight, QWidget *parent);
 
 protected:
     void highlight(const QRegExp &re, const QFont &highlightFont,
@@ -71,7 +71,7 @@ public:
     ItemTextLoader();
     ~ItemTextLoader();
 
-    ItemWidget *create(const QModelIndex &index, QWidget *parent, bool preview) const override;
+    ItemWidget *create(const QVariantMap &data, QWidget *parent, bool preview) const override;
 
     QString id() const override { return "itemtext"; }
     QString name() const override { return tr("Text"); }
