@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -32,10 +32,12 @@ public:
     explicit Client(QObject *parent = nullptr);
 
 protected:
-    void startClientSocket(const QString &serverName, int argc, char **argv, int skipArgc, int messageCode);
+    void startClientSocket(const QString &serverName);
 
     void sendMessage(const QByteArray &message, int messageCode);
 
+    void waitForReadyRead();
+
 private slots:
     /** Message received from server. */
     virtual void onMessageReceived(const QByteArray &message, int messageCode) = 0;
