@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -21,11 +21,16 @@
 
 #include "common/client_server.h"
 #include "common/commandstatus.h"
+#include "common/commandstore.h"
 #include "common/log.h"
 #include "platform/platformnativeinterface.h"
+#include "scriptable/scriptable.h"
+#include "scriptable/scriptableproxy.h"
 
-#include <QCoreApplication>
+#include <QApplication>
 #include <QFile>
+#include <QScriptEngine>
+#include <QSettings>
 #include <QThread>
 
 namespace {
@@ -43,8 +48,8 @@ QString messageCodeToString(int code)
         return "CommandException";
     case CommandPrint:
         return "CommandPrint";
-    case CommandReadInput:
-        return "CommandReadInput";
+    case CommandFunctionCallReturnValue:
+        return "CommandFunctionCallReturnValue";
     default:
         return QString("Unknown(%1)").arg(code);
     }
@@ -66,6 +71,20 @@ void printClientStderr(const QByteArray &output)
         f.write("\n");
 }
 
+QCoreApplication *createClientApplication(int &argc, char **argv, const QStringList &arguments)
+{
+    // Clipboard access requires QApplication.
+    if ( !arguments.isEmpty() && (
+             arguments[0] == "monitorClipboard"
+             || arguments[0] == "provideClipboard"
+             || arguments[0] == "provideSelection") )
+    {
+        return createPlatformNativeInterface()->createMonitorApplication(argc, argv);
+    }
+
+    return createPlatformNativeInterface()->createClientApplication(argc, argv);
+}
+
 } // namespace
 
 void InputReader::readInput()
@@ -77,19 +96,29 @@ void InputReader::readInput()
     emit inputRead(input);
 }
 
-ClipboardClient::ClipboardClient(int &argc, char **argv, int skipArgc, const QString &sessionName)
+ClipboardClient::ClipboardClient(int &argc, char **argv, const QStringList &arguments, const QString &sessionName)
     : Client()
-    , App("Client", createPlatformNativeInterface()->createClientApplication(argc, argv), sessionName)
+    , App("Client", createClientApplication(argc, argv, arguments), sessionName)
     , m_inputReaderThread(nullptr)
 {
     restoreSettings();
 
-    startClientSocket(clipboardServerName(), argc, argv, skipArgc, CommandArguments);
+    startClientSocket(clipboardServerName());
+
+    bool hasActionData;
+    const int id = qgetenv("COPYQ_ACTION_ID").toInt(&hasActionData);
+    QByteArray message;
+    if (hasActionData) {
+        QDataStream out(&message, QIODevice::WriteOnly);
+        out << id;
+    }
+
+    start(arguments);
 }
 
 void ClipboardClient::onMessageReceived(const QByteArray &data, int messageCode)
 {
-    COPYQ_LOG( "Message received: " + messageCodeToString(messageCode) );
+    COPYQ_LOG_VERBOSE( "Message received: " + messageCodeToString(messageCode) );
 
     switch (messageCode) {
     case CommandFinished:
@@ -98,6 +127,9 @@ void ClipboardClient::onMessageReceived(const QByteArray &data, int messageCode)
         break;
 
     case CommandError:
+        exit(1);
+        break;
+
     case CommandBadSyntax:
     case CommandException:
         printClientStderr(data);
@@ -108,11 +140,12 @@ void ClipboardClient::onMessageReceived(const QByteArray &data, int messageCode)
         printClientStdout(data);
         break;
 
-    case CommandReadInput:
-        startInputReader();
+    case CommandFunctionCallReturnValue:
+        emit functionCallResultReceived(data);
         break;
 
     default:
+        log( "Unhandled message: " + messageCodeToString(messageCode), LogError );
         break;
     }
 }
@@ -145,15 +178,26 @@ void ClipboardClient::setInput(const QByteArray &input)
 void ClipboardClient::sendInput()
 {
     if ( !wasClosed() )
-        sendMessage(m_input, CommandReadInputReply);
+        emit inputReceived(m_input);
 }
 
 void ClipboardClient::exit(int exitCode)
 {
+    emit functionCallResultReceived(QByteArray());
     abortInputReader();
     App::exit(exitCode);
 }
 
+void ClipboardClient::sendFunctionCall(const QByteArray &bytes)
+{
+    sendMessage(bytes, CommandFunctionCall);
+
+    QEventLoop loop;
+    connect(this, SIGNAL(functionCallResultReceived(QByteArray)), &loop, SLOT(quit()));
+    connect(qApp, SIGNAL(aboutToQuit()), &loop, SLOT(quit()));
+    loop.exec();
+}
+
 void ClipboardClient::startInputReader()
 {
     if ( wasClosed() || m_inputReaderThread )
@@ -188,3 +232,24 @@ bool ClipboardClient::isInputReaderFinished() const
 {
     return m_inputReaderThread && m_inputReaderThread->isFinished();
 }
+
+void ClipboardClient::start(const QStringList &arguments)
+{
+    QScriptEngine engine;
+    ScriptableProxy scriptableProxy(nullptr, nullptr);
+    Scriptable scriptable(&engine, &scriptableProxy);
+
+    connect( &scriptable, SIGNAL(sendMessage(QByteArray,int)),
+             this, SLOT(onMessageReceived(QByteArray,int)) );
+    connect( &scriptable, SIGNAL(readInput()),
+             this, SLOT(startInputReader()) );
+    connect( &scriptableProxy, SIGNAL(sendFunctionCall(QByteArray)),
+             this, SLOT(sendFunctionCall(QByteArray)) );
+
+    connect( this, SIGNAL(inputReceived(QByteArray)),
+             &scriptable, SLOT(setInput(QByteArray)) );
+    connect( this, SIGNAL(functionCallResultReceived(QByteArray)),
+             &scriptableProxy, SLOT(setReturnValue(QByteArray)) );
+
+    scriptable.executeArguments(arguments);
+}
