@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -19,14 +19,13 @@
 
 #include "clipboardserver.h"
 
-#include "app/remoteprocess.h"
-#include "common/appconfig.h"
+#include "common/action.h"
 #include "common/clientsocket.h"
 #include "common/client_server.h"
+#include "common/commandstatus.h"
 #include "common/display.h"
 #include "common/log.h"
 #include "common/mimetypes.h"
-#include "common/monitormessagecode.h"
 #include "common/shortcuts.h"
 #include "common/sleeptimer.h"
 #include "gui/clipboardbrowser.h"
@@ -36,7 +35,7 @@
 #include "gui/mainwindow.h"
 #include "item/itemfactory.h"
 #include "item/serialize.h"
-#include "scriptable/scriptableworker.h"
+#include "scriptable/scriptableproxy.h"
 
 #include <QAction>
 #include <QApplication>
@@ -45,7 +44,6 @@
 #include <QMessageBox>
 #include <QMimeData>
 #include <QSessionManager>
-#include <QThread>
 
 #ifdef NO_GLOBAL_SHORTCUTS
 class QxtGlobalShortcut {};
@@ -53,13 +51,25 @@ class QxtGlobalShortcut {};
 #include "../qxt/qxtglobalshortcut.h"
 #endif
 
+namespace {
+
+bool hasScriptCommand(const QVector<Command> &commands)
+{
+    for (const auto &command : commands) {
+        if (command.type() == CommandType::Script)
+            return true;
+    }
+
+    return false;
+}
+
+} // namespace
+
 ClipboardServer::ClipboardServer(QApplication *app, const QString &sessionName)
     : QObject()
     , App("Server", app, sessionName)
     , m_wnd(nullptr)
-    , m_monitor(nullptr)
     , m_shortcutActions()
-    , m_clientThreads()
     , m_ignoreKeysTimer()
 {
     const QString serverName = clipboardServerName();
@@ -86,8 +96,16 @@ ClipboardServer::ClipboardServer(QApplication *app, const QString &sessionName)
     m_itemFactory = new ItemFactory(this);
     m_wnd = new MainWindow(m_itemFactory);
 
+    m_itemFactory->loadPlugins();
+    if ( !m_itemFactory->hasLoaders() )
+        log("No plugins loaded", LogNote);
+
+    m_wnd->loadSettings();
+    m_wnd->setCurrentTab(0);
+    m_wnd->enterBrowseMode();
+
     connect( server, SIGNAL(newConnection(ClientSocketPtr)),
-             this, SLOT(doCommand(ClientSocketPtr)) );
+             this, SLOT(onClientNewConnection(ClientSocketPtr)) );
 
     connect( qApp, SIGNAL(aboutToQuit()),
              this, SLOT(onAboutToQuit()));
@@ -98,11 +116,10 @@ ClipboardServer::ClipboardServer(QApplication *app, const QString &sessionName)
     connect( qApp, SIGNAL(saveStateRequest(QSessionManager&)),
              this, SLOT(onSaveState(QSessionManager&)) );
 
-    connect( m_wnd, SIGNAL(changeClipboard(QVariantMap,QClipboard::Mode)),
-             this, SLOT(changeClipboard(QVariantMap,QClipboard::Mode)) );
-
     connect( m_wnd, SIGNAL(requestExit()),
              this, SLOT(maybeQuit()) );
+    connect( m_wnd, SIGNAL(disableClipboardStoringRequest(bool)),
+             this, SLOT(onDisableClipboardStoringRequest(bool)) );
 
     loadSettings();
 
@@ -110,14 +127,9 @@ ClipboardServer::ClipboardServer(QApplication *app, const QString &sessionName)
     connect( m_wnd, SIGNAL(configurationChanged()),
              this, SLOT(loadSettings()) );
 
-#ifndef NO_GLOBAL_SHORTCUTS
     connect( m_wnd, SIGNAL(commandsSaved()),
-             this, SLOT(createGlobalShortcuts()) );
-    createGlobalShortcuts();
-#endif
-
-    // Allow to run at least few client and internal threads concurrently.
-    m_clientThreads.setMaxThreadCount( qMax(m_clientThreads.maxThreadCount(), 8) );
+             this, SLOT(onCommandsSaved()) );
+    onCommandsSaved();
 
     // run clipboard monitor
     startMonitoring();
@@ -139,13 +151,24 @@ ClipboardServer::~ClipboardServer()
 
 void ClipboardServer::stopMonitoring()
 {
-    if (m_monitor == nullptr)
+    if (!m_monitor)
         return;
 
     COPYQ_LOG("Terminating monitor");
 
-    m_monitor->disconnect();
-    delete m_monitor;
+    disconnect( m_monitor, SIGNAL(destroyed()),
+                this, SLOT(onMonitorFinished()) );
+
+    for (auto it = m_clients.constBegin(); it != m_clients.constEnd(); ++it) {
+        const auto actionId = it.value().proxy->actionId();
+        if ( actionId == m_monitor->id() ) {
+            const auto client = it.key();
+            client->sendMessage(QByteArray(), CommandFinished);
+            break;
+        }
+    }
+
+    m_monitor->terminate();
     m_monitor = nullptr;
 
     COPYQ_LOG("Monitor terminated");
@@ -153,46 +176,16 @@ void ClipboardServer::stopMonitoring()
 
 void ClipboardServer::startMonitoring()
 {
-    COPYQ_LOG("Starting monitor");
-
-    if ( m_monitor == nullptr ) {
-        m_monitor = new RemoteProcess(this);
-        connect( m_monitor, SIGNAL(newMessage(QByteArray)),
-                 this, SLOT(newMonitorMessage(QByteArray)) );
-        connect( m_monitor, SIGNAL(connectionError(QString)),
-                 this, SLOT(monitorConnectionError(QString)) );
-        connect( m_monitor, SIGNAL(connected()),
-                 this, SLOT(loadMonitorSettings()) );
-
-        const QString name = serverName("m");
-        const auto session = qApp->property("CopyQ_session_name").toString();
-        m_monitor->start( name, QStringList() << "-s" << session << "monitor" << name );
-    }
-}
-
-void ClipboardServer::loadMonitorSettings()
-{
-    if ( !isMonitoring() )
+    if (m_monitor || !m_wnd->isMonitoringEnabled())
         return;
 
-    COPYQ_LOG("Configuring monitor");
+    COPYQ_LOG("Starting monitor");
 
-    QVariantMap settings;
-    settings["formats"] = m_itemFactory->formatsToSave();
-#ifdef HAS_MOUSE_SELECTIONS
-    settings["check_selection"] = AppConfig().option<Config::check_selection>();
-#endif
-
-    QByteArray settingsData;
-    QDataStream settingsOut(&settingsData, QIODevice::WriteOnly);
-    settingsOut << settings;
-
-    m_monitor->writeMessage(settingsData, MonitorSettings);
-}
-
-bool ClipboardServer::isMonitoring()
-{
-    return m_monitor != nullptr && m_monitor->isConnected();
+    m_monitor = new Action();
+    m_monitor->setCommand("copyq monitorClipboard");
+    connect( m_monitor, SIGNAL(destroyed()),
+             this, SLOT(onMonitorFinished()) );
+    m_wnd->runInternalAction(m_monitor);
 }
 
 void ClipboardServer::removeGlobalShortcuts()
@@ -202,14 +195,17 @@ void ClipboardServer::removeGlobalShortcuts()
     m_shortcutActions.clear();
 }
 
-void ClipboardServer::createGlobalShortcuts()
+void ClipboardServer::onCommandsSaved()
 {
+    const auto commands = loadEnabledCommands();
+
+#ifndef NO_GLOBAL_SHORTCUTS
     removeGlobalShortcuts();
 
     QList<QKeySequence> usedShortcuts;
 
-    for ( const auto &command : loadEnabledCommands() ) {
-        if ( !command.globalShortcuts.contains("DISABLED") ) {
+    for (const auto &command : commands) {
+        if (command.type() & CommandType::GlobalShortcut) {
             for (const auto &shortcutText : command.globalShortcuts) {
                 QKeySequence shortcut(shortcutText, QKeySequence::PortableText);
                 if ( !shortcut.isEmpty() && !usedShortcuts.contains(shortcut) ) {
@@ -219,6 +215,12 @@ void ClipboardServer::createGlobalShortcuts()
             }
         }
     }
+#endif
+
+    if ( m_monitor && hasScriptCommand(commands) ) {
+        stopMonitoring();
+        startMonitoring();
+    }
 }
 
 void ClipboardServer::onAboutToQuit()
@@ -227,10 +229,9 @@ void ClipboardServer::onAboutToQuit()
 
     m_wnd->saveTabs();
 
-    if( isMonitoring() )
-        stopMonitoring();
+    stopMonitoring();
 
-    terminateThreads();
+    emit terminateClients();
 }
 
 void ClipboardServer::onCommitData(QSessionManager &sessionManager)
@@ -272,15 +273,26 @@ void ClipboardServer::onSaveState(QSessionManager &sessionManager)
     settings.setValue(lastSessionIdKey, sessionNameKey);
 }
 
+void ClipboardServer::onDisableClipboardStoringRequest(bool disabled)
+{
+    stopMonitoring();
+    if (!disabled)
+        startMonitoring();
+}
+
 void ClipboardServer::maybeQuit()
 {
+    stopMonitoring();
+
     // Wait a moment for commands to finish.
     for ( int i = 0; i < 50 && hasRunningCommands(); ++i )
         waitFor(50);
 
     if (askToQuit()) {
-        terminateThreads();
+        emit terminateClients();
         QCoreApplication::exit();
+    } else {
+        startMonitoring();
     }
 }
 
@@ -303,85 +315,72 @@ bool ClipboardServer::askToQuit()
     return true;
 }
 
-void ClipboardServer::terminateThreads()
-{
-    const auto activeThreads = m_clientThreads.activeThreadCount();
-    if (activeThreads > 0)
-        return;
-
-    COPYQ_LOG( QString("Terminating %1 clients").arg(activeThreads) );
-    emit terminateClientThreads();
-    while ( !m_clientThreads.waitForDone(0) )
-        QCoreApplication::processEvents(QEventLoop::WaitForMoreEvents, 10);
-}
-
 bool ClipboardServer::hasRunningCommands() const
 {
-    return m_clientThreads.activeThreadCount() > 0 || m_wnd->hasRunningAction();
-}
+    if ( m_wnd->hasRunningAction() )
+        return true;
 
-void ClipboardServer::doCommand(const ClientSocketPtr &client)
-{
-    // Worker object without parent needs to be deleted afterwards!
-    // There is no parent so as it's possible to move the worker to another thread.
-    // QThreadPool takes ownership and worker will be automatically deleted
-    // after run() (see QRunnable::setAutoDelete()).
-    auto worker = new ScriptableWorker(m_wnd, client, m_itemFactory->scriptableObjects(this));
-
-    // Terminate worker at application exit.
-    connect( this, SIGNAL(terminateClientThreads()),
-             client.get(), SLOT(close()) );
-
-    // Add client thread to pool.
-    m_clientThreads.start(worker);
-}
-
-void ClipboardServer::newMonitorMessage(const QByteArray &message)
-{
-    if ( !m_wnd->isMonitoringEnabled() )
-        return;
-
-    QVariantMap data;
-
-    if ( !deserializeData(&data, message) ) {
-        log("Failed to read message from monitor.", LogError);
-        return;
+    for (auto it = m_clients.constBegin(); it != m_clients.constEnd(); ++it) {
+        const auto actionId = it.value().proxy->actionId();
+        if ( !m_wnd->isInternalActionId(actionId) )
+            return true;
     }
 
-    m_wnd->clipboardChanged(data);
+    return false;
 }
 
-void ClipboardServer::monitorConnectionError(const QString &error)
+void ClipboardServer::onClientNewConnection(const ClientSocketPtr &client)
 {
-    log("Restarting clipboard monitor (" + error + ")", LogError);
+    auto proxy = new ScriptableProxy(m_wnd, client.get());
+    m_clients.insert( client.get(), ClientData(client, proxy) );
+    connect( this, SIGNAL(terminateClients()),
+             client.get(), SLOT(close()) );
+    connect( client.get(), SIGNAL(messageReceived(QByteArray,int,ClientSocket*)),
+             this, SLOT(onClientMessageReceived(QByteArray,int,ClientSocket*)) );
+    connect( client.get(), SIGNAL(disconnected(ClientSocket*)),
+             this, SLOT(onClientDisconnected(ClientSocket*)) );
+    connect( client.get(), SIGNAL(connectionFailed(ClientSocket*)),
+             this, SLOT(onClientConnectionFailed(ClientSocket*)) );
+    client->start();
+}
+
+void ClipboardServer::onClientMessageReceived(
+        const QByteArray &message, int messageCode, ClientSocket *client)
+{
+    Q_UNUSED(client);
+    switch (messageCode) {
+    case CommandFunctionCall: {
+        auto proxy = m_clients.value(client).proxy;
+        if (!proxy)
+            return;
+        const auto result = proxy->callFunction(message);
+        client->sendMessage(result, CommandFunctionCallReturnValue);
+        break;
+    }
+    default:
+        log(QString("Unhandled command status: %1").arg(messageCode));
+        break;
+    }
+}
+
+void ClipboardServer::onClientDisconnected(ClientSocket *client)
+{
+    m_clients.remove(client);
+}
+
+void ClipboardServer::onClientConnectionFailed(ClientSocket *client)
+{
+    log("Client connection failed", LogWarning);
+    m_clients.remove(client);
+}
+
+void ClipboardServer::onMonitorFinished()
+{
+    COPYQ_LOG("Monitor finished");
     stopMonitoring();
     startMonitoring();
 }
 
-void ClipboardServer::changeClipboard(const QVariantMap &data, QClipboard::Mode mode)
-{
-    if ( !isMonitoring() ) {
-        COPYQ_LOG("Waiting for monitor to start");
-        SleepTimer t(5000);
-        while ( t.sleep() && !isMonitoring() ) {}
-        if ( !isMonitoring() ) {
-            log("Failed to send message to clipboard monitor.", LogError);
-            return;
-        }
-    }
-
-    const MonitorMessageCode code =
-            mode == QClipboard::Clipboard ? MonitorChangeClipboard : MonitorChangeSelection;
-
-    const auto message = serializeData(data);
-
-    COPYQ_LOG( QString("Sending change %1 request to monitor (%2 KiB)")
-               .arg(code == MonitorChangeClipboard ? "clipboard" : "selection")
-               .arg(message.size() / 1024.0) );
-
-    m_monitor->writeMessage(message, code);
-}
-
 void ClipboardServer::createGlobalShortcut(const QKeySequence &shortcut, const Command &command)
 {
 #ifdef NO_GLOBAL_SHORTCUTS
@@ -418,6 +417,13 @@ bool ClipboardServer::eventFilter(QObject *object, QEvent *ev)
         return true;
     }
 
+    if ( type == QEvent::KeyPress
+         || type == QEvent::Shortcut
+         || type == QEvent::ShortcutOverride )
+    {
+        m_wnd->updateShortcuts();
+    }
+
     // Close menu on Escape key and give focus back to search edit or browser.
     if (type == QEvent::KeyPress) {
         QKeyEvent *keyevent = static_cast<QKeyEvent *>(ev);
@@ -440,9 +446,10 @@ bool ClipboardServer::eventFilter(QObject *object, QEvent *ev)
 
 void ClipboardServer::loadSettings()
 {
-    // reload clipboard monitor configuration
-    if ( isMonitoring() )
-        loadMonitorSettings();
+    if (m_monitor) {
+        stopMonitoring();
+        startMonitoring();
+    }
 }
 
 void ClipboardServer::shortcutActivated(QxtGlobalShortcut *shortcut)
@@ -458,7 +465,7 @@ void ClipboardServer::shortcutActivated(QxtGlobalShortcut *shortcut)
         QVariantMap data;
         const QString shortcutText = portableShortcutText(shortcut->shortcut());
         data.insert(mimeShortcut, shortcutText.toUtf8());
-        m_wnd->action(data, it.value());
+        m_wnd->action(data, it.value(), QModelIndex());
     }
 #endif
 }
