@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -21,23 +21,21 @@
 #define CLIPBOARDSERVER_H
 
 #include "app.h"
+#include "common/clipboardmode.h"
 #include "common/server.h"
-#include "gui/configtabshortcuts.h"
-#include "gui/mainwindow.h"
 
 #include <QMap>
-#include <QProcess>
+#include <QPointer>
 #include <QTimer>
-#include <QThreadPool>
-#include <QVariantMap>
-#include <QWidget>
 
-class ClientSocket;
+class Action;
 class ItemFactory;
-class RemoteProcess;
+class MainWindow;
+class ScriptableProxy;
 class QxtGlobalShortcut;
 class QApplication;
 class QSessionManager;
+struct Command;
 
 /**
  * The application main server.
@@ -60,9 +58,6 @@ public:
     /** Start monitor application. */
     void startMonitoring();
 
-    /** Return true if monitor is running. */
-    bool isMonitoring();
-
     /**
      * Create global shortcut for command.
      *
@@ -71,39 +66,31 @@ public:
     void createGlobalShortcut(const QKeySequence &shortcut, const Command &command);
 
 public slots:
-    /** Load @a item data to clipboard. */
-    void changeClipboard(const QVariantMap &data, QClipboard::Mode mode);
-
     /** Load settings. */
     void loadSettings();
 
-    /** Send configuration to monitor. */
-    void loadMonitorSettings();
-
 signals:
-    void terminateClientThreads();
+    void terminateClients();
 
 protected:
     bool eventFilter(QObject *object, QEvent *ev) override;
 
 private slots:
-    /**
-     * Execute command in different thread.
-     */
-    void doCommand(const ClientSocketPtr &client = nullptr //!< For sending responses.
-            );
-
-    /** New message from monitor process. */
-    void newMonitorMessage(const QByteArray &message);
+    void onClientNewConnection(const ClientSocketPtr &client);
+    void onClientMessageReceived(const QByteArray &message, int messageCode, ClientSocket *client);
+    void onClientDisconnected(ClientSocket *client);
+    void onClientConnectionFailed(ClientSocket *client);
 
     /** An error occurred on monitor connection. */
-    void monitorConnectionError(const QString &error);
+    void onMonitorFinished();
 
     /** Shortcut was pressed on host system. */
     void shortcutActivated(QxtGlobalShortcut *shortcut);
 
     void removeGlobalShortcuts();
-    void createGlobalShortcuts();
+
+    /** Called when new commands are available. */
+    void onCommandsSaved();
 
     /** Clean up before quitting. */
     void onAboutToQuit();
@@ -119,6 +106,8 @@ private slots:
      */
     void onSaveState(QSessionManager &sessionManager);
 
+    void onDisableClipboardStoringRequest(bool disabled);
+
     /** Quit application, but ask to cancel exit if there are any active commands. */
     void maybeQuit();
 
@@ -126,16 +115,25 @@ private:
     /** Ask to cancel application exit if there are any active commands. */
     bool askToQuit();
 
-    void terminateThreads();
-
     bool hasRunningCommands() const;
 
     MainWindow* m_wnd;
-    RemoteProcess *m_monitor;
+    QPointer<Action> m_monitor;
     QMap<QxtGlobalShortcut*, Command> m_shortcutActions;
-    QThreadPool m_clientThreads;
     QTimer m_ignoreKeysTimer;
     ItemFactory *m_itemFactory;
+
+    struct ClientData {
+        ClientData() = default;
+        ClientData(const ClientSocketPtr &client, ScriptableProxy *proxy)
+            : client(client)
+            , proxy(proxy)
+        {
+        }
+        ClientSocketPtr client;
+        ScriptableProxy *proxy = nullptr;
+    };
+    QMap<ClientSocket*, ClientData> m_clients;
 };
 
 #endif // CLIPBOARDSERVER_H
