@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -26,13 +26,16 @@
 #include "item/serialize.h"
 
 #include <QCoreApplication>
+#include <QEventLoop>
+#include <QPointer>
 #include <QProcessEnvironment>
+#include <QTimer>
 
 #include <cstring>
 
 namespace {
 
-void startProcess(QProcess *process, const QStringList &args)
+void startProcess(QProcess *process, const QStringList &args, QIODevice::OpenModeFlag mode)
 {
     QString executable = args.value(0);
 
@@ -40,7 +43,7 @@ void startProcess(QProcess *process, const QStringList &args)
     if (executable == "copyq")
         executable = QCoreApplication::applicationFilePath();
 
-    process->start(executable, args.mid(1), QIODevice::ReadWrite);
+    process->start(executable, args.mid(1), mode);
 }
 
 template <typename Entry, typename Container>
@@ -271,11 +274,20 @@ void Action::start()
     connect( m_processes.first(), SIGNAL(bytesWritten(qint64)),
              this, SLOT(onBytesWritten()), Qt::QueuedConnection );
 
-    if (m_outputFormat.isEmpty())
-        m_processes.last()->closeReadChannel(QProcess::StandardOutput);
-
-    for (int i = 0; i < m_processes.size(); ++i)
-        startProcess(m_processes[i], cmds[i]);
+    const bool needWrite = !m_input.isEmpty();
+    if (m_processes.size() == 1) {
+        const auto mode =
+                (needWrite && m_readOutput) ? QIODevice::ReadWrite
+              : needWrite ? QIODevice::WriteOnly
+              : m_readOutput ? QIODevice::ReadOnly
+              : QIODevice::NotOpen;
+        startProcess(m_processes.first(), cmds.first(), mode);
+    } else {
+        startProcess(m_processes.first(), cmds.first(), needWrite ? QIODevice::ReadWrite : QIODevice::ReadOnly);
+        for (int i = 1; i < m_processes.size() - 1; ++i)
+            startProcess(m_processes[i], cmds[i], QIODevice::ReadWrite);
+        startProcess(m_processes.last(), cmds.last(), m_readOutput ? QIODevice::ReadWrite : QIODevice::WriteOnly);
+    }
 }
 
 bool Action::waitForStarted(int msecs)
@@ -288,14 +300,15 @@ bool Action::waitForFinished(int msecs)
     if ( !isRunning() )
         return true;
 
-    for ( int waitMsec = 0;
-          waitMsec < msecs && !m_processes.isEmpty() && !m_processes.last()->waitForFinished(100);
-          waitMsec += 100 )
-    {
-        QCoreApplication::processEvents();
-    }
+    QPointer<QObject> self(this);
+    QEventLoop loop;
+    QTimer t;
+    connect(m_processes.last(), SIGNAL(finished(int)), &loop, SLOT(quit()));
+    connect(&t, SIGNAL(timeout()), &loop, SLOT(quit()));
+    t.start(msecs);
+    loop.exec(QEventLoop::ExcludeUserInputEvents);
 
-    return !isRunning();
+    return !self || !isRunning();
 }
 
 bool Action::isRunning() const
@@ -349,23 +362,14 @@ void Action::onSubProcessOutput()
         return;
 
     auto p = m_processes.last();
+    if ( !p->isReadable() )
+        return;
+
     const auto output = p->readAll();
     if ( output.isEmpty() )
         return;
 
-    if ( !m_outputFormat.isEmpty() ) {
-        m_outputData.append(output);
-
-        if ( !m_sep.isEmpty() ) {
-            m_lastOutput.append( getTextData(output) );
-            auto items = m_lastOutput.split(m_sep);
-            m_lastOutput = items.takeLast();
-            if ( !items.isEmpty() )
-                emit newItems(items, m_outputFormat, m_tab);
-        } else if ( m_outputFormat == mimeText && m_index.isValid() ) {
-            emit changeItem(m_outputData, m_outputFormat, m_index);
-        }
-    }
+    emit actionOutput(output);
 }
 
 void Action::onSubProcessErrorOutput()
@@ -373,7 +377,8 @@ void Action::onSubProcessErrorOutput()
     QProcess *p = qobject_cast<QProcess*>(sender());
     Q_ASSERT(p);
 
-    m_errorOutput.append( getTextData(p->readAllStandardError()) );
+    if ( p->isReadable() )
+        m_errorOutput.append( getTextData(p->readAllStandardError()) );
 }
 
 void Action::writeInput()
@@ -428,18 +433,5 @@ void Action::closeSubCommands()
 void Action::actionFinished()
 {
     closeSubCommands();
-
-    if ( !m_outputFormat.isEmpty() ) {
-        if ( !m_sep.isEmpty() ) {
-            if ( !m_lastOutput.isEmpty() )
-                emit newItems(QStringList() << m_lastOutput, m_outputFormat, m_tab);
-        } else if ( !m_outputData.isEmpty() ) {
-            if ( m_index.isValid() )
-                emit changeItem(m_outputData, m_outputFormat, m_index);
-            else
-                emit newItem(m_outputData, m_outputFormat, m_tab);
-        }
-    }
-
     emit actionFinished(this);
 }
