@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -66,21 +66,6 @@ public:
     /** Return input formats. */
     const QStringList &inputFormats() const { return m_inputFormats; }
 
-    /** Return output format. */
-    const QString &outputFormat() const { return m_outputFormat; }
-    void setOutputFormat(const QString &outputItemFormat) { m_outputFormat = outputItemFormat; }
-
-    /// Set separator for items on standard output.
-    void setItemSeparator(const QRegExp &itemSeparator) { m_sep = itemSeparator; }
-
-    /// Return tab name for output items.
-    QString outputTab() const { return m_tab; }
-    void setOutputTab(const QString &outputTabName) { m_tab = outputTabName; }
-
-    /** Return destination index. */
-    QModelIndex index() const { return m_index; }
-    void setIndex(const QModelIndex &index) { m_index = index; }
-
     /** Set working directory path (default is empty so it doesn't change working directory). */
     void setWorkingDirectory(const QString &path) { m_workingDirectoryPath = path; }
 
@@ -99,8 +84,6 @@ public:
     /** Return human-readable name for action. */
     QString name() const { return m_name; }
 
-    QByteArray outputData() const { return m_outputData; }
-
     int exitCode() const { return m_exitCode; }
     QString errorString() const { return m_errorString; }
 
@@ -110,42 +93,19 @@ public:
     void setId(int actionId) { m_id = actionId; }
     int id() const { return m_id; }
 
-    void setIgnoreExitCode(bool ignore) { m_ignoreExitCode = ignore; }
-    bool ignoreExitCode() const { return m_ignoreExitCode; }
+    void setReadOutput(bool read) { m_readOutput = read; }
 
 public slots:
     /** Terminate (kill) process. */
     void terminate();
 
 signals:
-    /** Emitted on error. */
-    void actionError(Action *act);
     /** Emitted when finished. */
     void actionFinished(Action *act);
     /** Emitter when started. */
     void actionStarted(Action *act);
 
-    /**
-     * Emitted if standard output can be split into items.
-     *
-     * Output format and separator's must be set to valid values.
-     */
-    void newItems(const QStringList &, const QString &format, const QString &outputTabName);
-
-    /**
-     * Emitted after all standard output is read.
-     *
-     * Output format must be set to a valid value and both separator and index invalid.
-     */
-    void newItem(const QByteArray &data, const QString &format, const QString &outputTabName);
-
-    /**
-     * Emitted if standard output has some items available.
-     *
-     * Output format and index must be set to a valid value and separator empty.
-     */
-    void changeItem(
-            const QByteArray &data, const QString &format, const QModelIndex &index);
+    void actionOutput(const QByteArray &output);
 
     void dataChanged(const QVariantMap &data);
 
@@ -163,17 +123,12 @@ private:
     void actionFinished();
 
     QByteArray m_input;
-    QRegExp m_sep;
     QList< QList<QStringList> > m_cmds;
-    QString m_tab;
     QStringList m_inputFormats;
-    QString m_outputFormat;
-    QPersistentModelIndex m_index;
     QString m_workingDirectoryPath;
     QString m_errorOutput;
-    QString m_lastOutput;
-    QByteArray m_outputData;
     bool m_failed;
+    bool m_readOutput = false;
     int m_currentLine;
     QString m_name;
     QVariantMap m_data;
@@ -181,7 +136,6 @@ private:
 
     int m_exitCode;
     QString m_errorString;
-    bool m_ignoreExitCode = false;
 
     int m_id = -1;
 };
