@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -27,6 +27,7 @@
 class QString;
 
 QString defaultClipboardTabName();
+QString defaultTabName();
 
 namespace Config {
 
@@ -135,6 +136,11 @@ struct always_on_top : Config<bool> {
     static QString name() { return "always_on_top"; }
 };
 
+struct close_on_unfocus : Config<bool> {
+    static QString name() { return "close_on_unfocus"; }
+    static Value defaultValue() { return true; }
+};
+
 struct open_windows_on_current_screen : Config<bool> {
     static QString name() { return "open_windows_on_current_screen"; }
     static Value defaultValue() { return true; }
@@ -191,13 +197,14 @@ struct activate_closes : Config<bool> {
 
 struct activate_focuses : Config<bool> {
     static QString name() { return "activate_focuses"; }
+    static Value defaultValue() { return true; }
 };
 
 struct activate_pastes : Config<bool> {
     static QString name() { return "activate_pastes"; }
+    static Value defaultValue() { return true; }
 };
 
-
 struct tray_items : Config<int> {
     static QString name() { return "tray_items"; }
     static Value defaultValue() { return 5; }
@@ -286,12 +293,7 @@ struct autocompletion : Config<bool> {
 class AppConfig
 {
 public:
-    enum Category {
-        OptionsCategory,
-        ThemeCategory
-    };
-
-    explicit AppConfig(Category category = OptionsCategory);
+    explicit AppConfig();
 
     QVariant option(const QString &name) const;
 
@@ -309,12 +311,6 @@ public:
         return T::value(value);
     }
 
-    bool isOptionOn(const QString &name, bool defaultValue = false) const
-    {
-        const QVariant value = option(name);
-        return value.isValid() ? value.toBool() : defaultValue;
-    }
-
     void setOption(const QString &name, const QVariant &value);
 
     void removeOption(const QString &name);
