@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -24,6 +24,19 @@
 #include <QStringList>
 #include <QRegExp>
 
+namespace CommandType {
+enum CommandType {
+    None = 0,
+    Invalid = 1,
+    Automatic = 1 << 1,
+    GlobalShortcut = 1 << 2,
+    Menu = 1 << 3,
+    Script = 1 << 4,
+    Display = 1 << 5,
+    Disabled = 1 << 6
+};
+} // namespace CommandType
+
 /**
  * Command for matched items.
  *
@@ -45,7 +58,10 @@ struct Command {
         , output()
         , wait(false)
         , automatic(false)
+        , display(false)
         , inMenu(false)
+        , isGlobalShortcut(false)
+        , isScript(false)
         , transform(false)
         , remove(false)
         , hideWindow(false)
@@ -68,7 +84,10 @@ struct Command {
             && output == other.output
             && wait == other.wait
             && automatic == other.automatic
+            && display == other.display
             && inMenu == other.inMenu
+            && isGlobalShortcut == other.isGlobalShortcut
+            && isScript == other.isScript
             && transform == other.transform
             && remove == other.remove
             && hideWindow == other.hideWindow
@@ -84,6 +103,27 @@ struct Command {
         return !(*this == other);
     }
 
+    int type() const
+    {
+        auto type =
+               (automatic ? CommandType::Automatic : 0)
+             | (display ? CommandType::Display : 0)
+             | (isGlobalShortcut ? CommandType::GlobalShortcut : 0)
+             | (inMenu && !name.isEmpty() ? CommandType::Menu : 0);
+
+        // Scripts cannot be used in other types of commands.
+        if (isScript)
+            type = CommandType::Script;
+
+        if (type == CommandType::None)
+            type = CommandType::Invalid;
+
+        if (!enable)
+            type |= CommandType::Disabled;
+
+        return type;
+    }
+
     /** Name or short description. Used for menu item. */
     QString name;
 
@@ -125,9 +165,18 @@ struct Command {
     /** If true run command automatically on new matched items. */
     bool automatic;
 
+    /** If true, run command automatically when displaying matched items. */
+    bool display;
+
     /** If true show command in context menu on matching items. */
     bool inMenu;
 
+    /** If true command can be triggered by global shortcut. */
+    bool isGlobalShortcut;
+
+    /** If true, overrides scripting functionality. */
+    bool isScript;
+
     /** If true change item data, don't create any new items. */
     bool transform;
 
@@ -146,7 +195,7 @@ struct Command {
     /** Shortcut for menu item. */
     QStringList shortcuts;
 
-    /** Global/sytem shortcut. */
+    /** Global/system shortcut. */
     QStringList globalShortcuts;
 
     /** Copy item to other tab (automatically on new matched item). */
