@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -32,13 +32,10 @@ enum CommandStatus {
     CommandException = 4,
 
     /** Print on stdout */
-    CommandPrint,
-    /** Ask client to send data from its stdin */
-    CommandReadInput,
-    /** Arguments/script from client */
-    CommandArguments,
-    /** Client data from its stdin */
-    CommandReadInputReply
+    CommandPrint = 5,
+
+    CommandFunctionCall = 8,
+    CommandFunctionCallReturnValue = 9,
 };
 
 #endif // COMMANDSTATUS_H
