@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -56,6 +56,7 @@ void loadCommand(const QSettings &settings, CommandFilter filter, Commands *comm
 
     c.wait = settings.value("Wait").toBool();
     c.automatic = settings.value("Automatic").toBool();
+    c.display = settings.value("Display").toBool();
     c.transform = settings.value("Transform").toBool();
     c.hideWindow = settings.value("HideWindow").toBool();
     c.icon = settings.value("Icon").toString();
@@ -64,9 +65,17 @@ void loadCommand(const QSettings &settings, CommandFilter filter, Commands *comm
     c.tab = settings.value("Tab").toString();
     c.outputTab = settings.value("OutputTab").toString();
     c.inMenu = settings.value("InMenu").toBool();
+    c.isScript = settings.value("IsScript").toBool();
 
-    if (c.globalShortcuts.size() == 1 && c.globalShortcuts[0] == "DISABLED")
-        c.globalShortcuts = QStringList();
+    const auto globalShortcutsOption = settings.value("IsGlobalShortcut");
+    if ( globalShortcutsOption.isValid() ) {
+        c.isGlobalShortcut = settings.value("IsGlobalShortcut").toBool();
+    } else {
+        // Backwards compatibility with v3.1.2 and below.
+        if ( c.globalShortcuts.contains("DISABLED") )
+            c.globalShortcuts.clear();
+        c.isGlobalShortcut = !c.globalShortcuts.isEmpty();
+    }
 
     if (settings.value("Ignore").toBool())
         c.remove = c.automatic = true;
@@ -111,7 +120,10 @@ void saveCommand(const Command &c, QSettings *settings)
 
     saveNewValue("Wait", c, &Command::wait, settings);
     saveNewValue("Automatic", c, &Command::automatic, settings);
+    saveNewValue("Display", c, &Command::display, settings);
     saveNewValue("InMenu", c, &Command::inMenu, settings);
+    saveNewValue("IsGlobalShortcut", c, &Command::isGlobalShortcut, settings);
+    saveNewValue("IsScript", c, &Command::isScript, settings);
     saveNewValue("Transform", c, &Command::transform, settings);
     saveNewValue("Remove", c, &Command::remove, settings);
     saveNewValue("HideWindow", c, &Command::hideWindow, settings);
