@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -164,6 +164,16 @@ QByteArray getUtf8Data(const QMimeData &data, const QString &format)
     return data.data(format);
 }
 
+bool hasNonInternalFormats(const QStringList &formats) {
+    for (auto it = formats.constBegin(); it != formats.constEnd(); ++it) {
+        if ( !it->startsWith(COPYQ_MIME_PREFIX) ) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 } // namespace
 
 bool isMainThread()
@@ -171,14 +181,18 @@ bool isMainThread()
     return QThread::currentThread() == qApp->thread();
 }
 
-const QMimeData *clipboardData(QClipboard::Mode mode)
+const QMimeData *clipboardData(ClipboardMode mode)
 {
     Q_ASSERT( isMainThread() );
 
-    const auto modeText = mode == QClipboard::Clipboard ? "clipboard" : "selection";
+    const auto modeText = mode == ClipboardMode::Clipboard ? "clipboard" : "selection";
     COPYQ_LOG_VERBOSE( QString("Getting %1 data.").arg(modeText) );
 
-    const QMimeData *data = QApplication::clipboard()->mimeData(mode);
+    const auto qtmode = mode == ClipboardMode::Clipboard
+            ? QClipboard::Clipboard
+            : QClipboard::Selection;
+    const QMimeData *data = QApplication::clipboard()->mimeData(qtmode);
+
     if (data)
         COPYQ_LOG_VERBOSE( QString("Got %1 data.").arg(modeText) );
     else
@@ -305,7 +319,7 @@ QString elideText(const QString &text, const QFont &font, const QString &format,
     QStringList lines = text.split('\n');
 
     // Ignore empty lines at beginning.
-    const QRegExp reNonEmpty("\\S");
+    const QRegExp reNonEmpty(".*\\S.*");
     const int firstLine = qMax(0, lines.indexOf(reNonEmpty));
     const int lastLine = qMax(0, lines.lastIndexOf(reNonEmpty, firstLine + maxLines - 1));
 
@@ -369,7 +383,7 @@ QString textLabelForData(const QVariantMap &data, const QFont &font, const QStri
 
     if ( data.contains(mimeHidden) ) {
         label = QObject::tr("<HIDDEN>", "Label for hidden/secret clipboard content");
-    } else if ( formats.contains(mimeText) ) {
+    } else if ( formats.contains(mimeText) || formats.contains(mimeUriList) ) {
         const QString text = getTextData(data);
         const int n = text.count(QChar('\n')) + 1;
 
@@ -385,9 +399,7 @@ QString textLabelForData(const QVariantMap &data, const QFont &font, const QStri
         return elideText(textWithNotes, font, label, escapeAmpersands, maxWidthPixels, maxLines);
     } else if ( formats.indexOf(QRegExp("^image/.*")) != -1 ) {
         label = QObject::tr("<IMAGE>", "Label for image in clipboard");
-    } else if ( formats.indexOf(mimeUriList) != -1 ) {
-        label = QObject::tr("<FILES>", "Label for URLs/files in clipboard");
-    } else if ( formats.isEmpty() ) {
+    } else if ( !hasNonInternalFormats(formats) ) {
         label = QObject::tr("<EMPTY>", "Label for empty clipboard");
     } else {
         label = QObject::tr("<DATA>", "Label for data in clipboard");
@@ -404,6 +416,11 @@ QString textLabelForData(const QVariantMap &data, const QFont &font, const QStri
     return label;
 }
 
+QString textLabelForData(const QVariantMap &data)
+{
+    return textLabelForData(data, QFont(), QString());
+}
+
 void renameToUnique(QString *name, const QStringList &names)
 {
     const QString baseName = *name;
@@ -412,23 +429,6 @@ void renameToUnique(QString *name, const QStringList &names)
         *name = baseName + " (" + QString::number(++i) + ')';
 }
 
-bool containsAnyData(const QVariantMap &data)
-{
-    for (auto it = data.constBegin(); it != data.constEnd(); ++it) {
-        const auto &mime = it.key();
-        if (mime != mimeOwner
-                && mime != mimeWindowTitle
-                && mime != mimeHidden
-                && mime != mimeClipboardMode
-                && mime != mimeItems)
-        {
-            return true;
-        }
-    }
-
-    return false;
-}
-
 bool openTemporaryFile(QTemporaryFile *file, const QString &suffix)
 {
     const QString tmpFileName = "CopyQ.XXXXXX" + suffix;
@@ -460,7 +460,7 @@ QString dataToText(const QByteArray &bytes, const QString &mime)
     return codec->toUnicode(bytes);
 }
 
-bool clipboardContains(QClipboard::Mode mode, const QVariantMap &data)
+bool clipboardContains(ClipboardMode mode, const QVariantMap &data)
 {
     const QMimeData *clipboardData = ::clipboardData(mode);
     if (!clipboardData)
