@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -18,6 +18,7 @@
 */
 
 #include "config.h"
+#include "log.h"
 
 #include <QApplication>
 #include <QByteArray>
@@ -29,6 +30,9 @@
 #include <QVariant>
 #include <QWidget>
 
+#define GEOMETRY_LOG(window, message) \
+    COPYQ_LOG( QString("Geometry: Window \"%1\": %2").arg(window->objectName(), message) )
+
 namespace {
 
 const char propertyGeometryLockedUntilHide[] = "CopyQ_geometry_locked_until_hide";
@@ -38,6 +42,15 @@ enum class GeometryAction {
     Restore
 };
 
+QString toString(const QRect &geometry)
+{
+    return QString("%1x%2,%3,%4")
+            .arg(geometry.width())
+            .arg(geometry.height())
+            .arg(geometry.x())
+            .arg(geometry.y());
+}
+
 int screenNumber(const QWidget &widget, GeometryAction geometryAction)
 {
     return geometryAction == GeometryAction::Save
@@ -126,7 +139,8 @@ void restoreWindowGeometry(QWidget *w, bool openOnCurrentScreen)
     QByteArray geometry = geometryOptionValue(optionName + tag).toByteArray();
 
     // If geometry for screen resolution doesn't exist, use last saved one.
-    if (geometry.isEmpty()) {
+    const bool hasGeometryTag = geometry.isEmpty();
+    if (hasGeometryTag) {
         geometry = geometryOptionValue(optionName).toByteArray();
 
         // If geometry for the screen doesn't exist, move window to the middle of the screen.
@@ -143,18 +157,30 @@ void restoreWindowGeometry(QWidget *w, bool openOnCurrentScreen)
             w->move(position);
 
             geometry = w->saveGeometry();
+
+            GEOMETRY_LOG( w, QString("New geometry for \"%1%2\"").arg(optionName, tag) );
         }
     }
 
     if (w->saveGeometry() != geometry) {
+        const auto oldGeometry = w->geometry();
         w->restoreGeometry(geometry);
 
         // Workaround for broken geometry restore.
         if ( w->geometry().isEmpty() ) {
+            GEOMETRY_LOG( w, QString("Workaround for broken geometry restore") );
             w->showNormal();
             w->restoreGeometry(geometry);
             w->showMinimized();
         }
+
+        const auto newGeometry = w->geometry();
+
+        GEOMETRY_LOG( w, QString("Restore geometry \"%1%2\": %3 -> %4").arg(
+                          optionName,
+                          hasGeometryTag ? tag : QString(),
+                          toString(oldGeometry),
+                          toString(newGeometry)) );
     }
 }
 
@@ -165,6 +191,7 @@ void saveWindowGeometry(QWidget *w, bool openOnCurrentScreen)
     QSettings geometrySettings( getGeometryConfigurationFilePath(), QSettings::IniFormat );
     geometrySettings.setValue( optionName + tag, w->saveGeometry() );
     geometrySettings.setValue( optionName, w->saveGeometry() );
+    GEOMETRY_LOG( w, QString("Save geometry \"%1%2\": %3").arg(optionName, tag, toString(w->geometry())) );
 }
 
 QByteArray mainWindowState(const QString &mainWindowObjectName)
@@ -184,6 +211,7 @@ void moveToCurrentWorkspace(QWidget *w)
 #ifdef COPYQ_WS_X11
     /* Re-initialize window in window manager so it can popup on current workspace. */
     if (w->isVisible()) {
+        GEOMETRY_LOG(w, "Move to current workspace");
         const auto blockUntilHide = isGeometryGuardBlockedUntilHidden(w);
         w->hide();
         if (blockUntilHide)
@@ -198,15 +226,21 @@ void moveToCurrentWorkspace(QWidget *w)
 void moveWindowOnScreen(QWidget *w, QPoint pos)
 {
     const QRect availableGeometry = QApplication::desktop()->availableGeometry(pos);
-    const int x = qMax(0, qMin(pos.x(), availableGeometry.right() - w->width()));
-    const int y = qMax(0, qMin(pos.y(), availableGeometry.bottom() - w->height()));
+    const int x = qMax(availableGeometry.left(), qMin(pos.x(), availableGeometry.right() - w->width()));
+    const int y = qMax(availableGeometry.top(), qMin(pos.y(), availableGeometry.bottom() - w->height()));
+    GEOMETRY_LOG( w, QString("Move window on screen %1x%2")
+                  .arg(availableGeometry.right())
+                  .arg(availableGeometry.bottom()) );
     w->move(x, y);
     moveToCurrentWorkspace(w);
 }
 
 void setGeometryGuardBlockedUntilHidden(QWidget *w, bool blocked)
 {
-    w->setProperty(propertyGeometryLockedUntilHide, blocked);
+    if ( isGeometryGuardBlockedUntilHidden(w) != blocked ) {
+        GEOMETRY_LOG( w, QString("Geometry blocked until hidden: %1").arg(blocked) );
+        w->setProperty(propertyGeometryLockedUntilHide, blocked);
+    }
 }
 
 bool isGeometryGuardBlockedUntilHidden(const QWidget *w)
