@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -26,6 +26,7 @@
 
 #include <QCoreApplication>
 #include <QDataStream>
+#include <QEventLoop>
 #include <QLocalServer>
 #include <QLocalSocket>
 #include <QSharedMemory>
@@ -104,11 +105,15 @@ Server::Server(const QString &name, QObject *parent)
 {
     if ( m_systemMutex && !serverIsRunning(name) ) {
         QLocalServer::removeServer(name);
-        if ( !m_server->listen(name) )
-            log("Failed to create server: " + m_server->errorString(), LogError);
+        if ( !m_server->listen(name) ) {
+            log( QString("Failed to create server \"%1\": %2")
+                 .arg(m_server->fullServerName())
+                 .arg(m_server->errorString()),
+                 LogError);
+        }
     }
 
-    connect( qApp, SIGNAL(aboutToQuit()), SLOT(close()) );
+    connect( qApp, SIGNAL(aboutToQuit()), this, SLOT(close()) );
 }
 
 Server::~Server()
@@ -137,8 +142,11 @@ void Server::close()
 
     if (m_socketCount > 0) {
         COPYQ_LOG( QString("Waiting for %1 sockets to disconnect").arg(m_socketCount) );
-        while (m_socketCount > 0)
-            QCoreApplication::processEvents(QEventLoop::WaitForMoreEvents, 10);
+
+        QEventLoop loop;
+        m_loop = &loop;
+        loop.exec(QEventLoop::ExcludeUserInputEvents);
+        m_loop = nullptr;
     }
 
     delete m_systemMutex;
@@ -167,4 +175,6 @@ void Server::onSocketDestroyed()
 {
     Q_ASSERT(m_socketCount > 0);
     --m_socketCount;
+    if (m_loop)
+        m_loop->quit();
 }
