@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -26,6 +26,7 @@
 #include <memory>
 
 class ClientSocket;
+class QEventLoop;
 class QLocalServer;
 
 using ClientSocketPtr = std::shared_ptr<ClientSocket>;
@@ -57,6 +58,7 @@ private:
     QLocalServer *m_server;
     QObject *m_systemMutex;
     int m_socketCount;
+    QEventLoop *m_loop = nullptr;
 };
 
 #endif // SERVER_H
