@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -117,9 +117,6 @@ Settings::~Settings()
 
 void Settings::restore()
 {
-    if (!canModifySettings())
-        return;
-
     Settings appSettings;
 
     if ( isLastSaveUnfinished() ) {
