@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -27,7 +27,7 @@
 #include "common/log.h"
 #include "common/mimetypes.h"
 #include "common/textdata.h"
-#include "gui/actiondialog.h"
+#include "gui/notification.h"
 #include "gui/processmanagerdialog.h"
 #include "gui/clipboardbrowser.h"
 #include "gui/mainwindow.h"
@@ -38,35 +38,27 @@
 
 #include <cmath>
 
+namespace {
+
+QString actionDescription(const Action &action)
+{
+    const auto name = action.name();
+    if ( !name.isEmpty() )
+        return QString("Command \"%1\"").arg(name);
+
+    return action.command();
+}
+
+} // namespace
+
 ActionHandler::ActionHandler(MainWindow *mainWindow)
     : QObject(mainWindow)
     , m_wnd(mainWindow)
-    , m_actionCounter(0)
     , m_activeActionDialog(new ProcessManagerDialog(mainWindow))
 {
     Q_ASSERT(mainWindow);
 }
 
-ActionDialog *ActionHandler::createActionDialog(const QStringList &tabs)
-{
-    auto actionDialog = new ActionDialog(m_wnd);
-    actionDialog->setAttribute(Qt::WA_DeleteOnClose, true);
-    actionDialog->setOutputTabs(tabs, QString());
-    actionDialog->setCommand(m_lastActionDialogCommand);
-
-    connect( actionDialog, SIGNAL(accepted(Action*)),
-             this, SLOT(action(Action*)) );
-    connect( actionDialog, SIGNAL(closed(ActionDialog*)),
-             this, SLOT(actionDialogClosed(ActionDialog*)) );
-
-    return actionDialog;
-}
-
-bool ActionHandler::hasRunningAction() const
-{
-    return m_actionCounter > 0;
-}
-
 void ActionHandler::showProcessManagerDialog()
 {
     m_activeActionDialog->show();
@@ -90,6 +82,18 @@ void ActionHandler::setActionData(int id, const QVariantMap &data)
         action->setData(data);
 }
 
+void ActionHandler::internalAction(Action *action)
+{
+    this->action(action);
+    if ( m_actions.contains(action->id()) )
+        m_internalActions.insert(action->id());
+}
+
+bool ActionHandler::isInternalActionId(int id) const
+{
+    return m_internalActions.contains(id);
+}
+
 void ActionHandler::action(Action *action)
 {
     action->setParent(this);
@@ -98,28 +102,13 @@ void ActionHandler::action(Action *action)
     action->setId(id);
     m_actions.insert(id, action);
 
-    m_lastAction = action;
-
-    connect( action, SIGNAL(newItems(QStringList,QString,QString)),
-             this, SLOT(addItems(QStringList,QString,QString)) );
-    connect( action, SIGNAL(newItem(QByteArray,QString,QString)),
-             this, SLOT(addItem(QByteArray,QString,QString)) );
-    connect( action, SIGNAL(changeItem(QByteArray,QString,QModelIndex)),
-             this, SLOT(changeItem(QByteArray,QString,QModelIndex)) );
     connect( action, SIGNAL(actionStarted(Action*)),
              this, SLOT(actionStarted(Action*)) );
     connect( action, SIGNAL(actionFinished(Action*)),
              this, SLOT(closeAction(Action*)) );
-    connect( action, SIGNAL(actionError(Action*)),
-             this, SLOT(closeAction(Action*)) );
-
-    ++m_actionCounter;
-
-    if ( !action->outputFormat().isEmpty() && action->outputTab().isEmpty() )
-        action->setOutputTab(m_currentTabName);
 
     m_activeActionDialog->actionAboutToStart(action);
-    COPYQ_LOG( QString("Executing: %1").arg(action->command()) );
+    COPYQ_LOG( QString("Executing: %1").arg(actionDescription(*action)) );
     action->start();
 }
 
@@ -132,32 +121,24 @@ void ActionHandler::actionStarted(Action *action)
 void ActionHandler::closeAction(Action *action)
 {
     m_actions.remove(action->id());
+    m_internalActions.remove(action->id());
 
     QString msg;
 
     QSystemTrayIcon::MessageIcon icon = QSystemTrayIcon::Information;
 
     if ( action->actionFailed() ) {
-        msg = tr("Error: %1\n").arg(action->errorString()) + action->errorOutput();
+        msg = tr("Error: %1").arg(action->errorString());
         icon = QSystemTrayIcon::Critical;
     } else if ( action->exitCode() != 0 ) {
-        if ( !action->ignoreExitCode() ) {
-            msg = tr("Exit code: %1\n").arg(action->exitCode()) + "\n" + action->errorOutput();
-            icon = QSystemTrayIcon::Warning;
-        }
-    } else if ( !action->inputFormats().isEmpty() ) {
-        const QModelIndex index = action->index();
-        ClipboardBrowser *c = m_wnd->browserForItem(index);
-        if (c) {
-            QStringList removeFormats = action->inputFormats();
-            removeFormats.removeAll( action->outputFormat() );
-
-            if ( !removeFormats.isEmpty() )
-                c->model()->setData(index, removeFormats, contentType::removeFormats);
-        }
+        msg = tr("Exit code: %1").arg(action->exitCode());
+        icon = QSystemTrayIcon::Warning;
     }
 
     if ( !msg.isEmpty() ) {
+        if ( !action->errorOutput().isEmpty() )
+            msg.append( "\n" + action->errorOutput() );
+
         const int maxWidthPoints =
                 AppConfig().option<Config::notification_maximum_width>();
         const QString command = action->command()
@@ -171,63 +152,22 @@ void ActionHandler::closeAction(Action *action)
         // Print command with line numbers.
         int lineNumber = 0;
         const auto lines = command.split("\n");
-        const auto lineNumberWidth = std::log10(lines.size()) + 1;
+        const auto lineNumberWidth = static_cast<int>(std::log10(lines.size())) + 1;
         for (const auto &line : lines)
             msg.append(QString("\n%1. %2").arg(++lineNumber, lineNumberWidth).arg(line));
 
         log(title + "\n" + msg);
-        m_wnd->showMessage(title, msg, icon);
+
+        auto notification = m_wnd->createNotification();
+        notification->setTitle(title);
+        notification->setMessage(msg);
+        notification->setIcon(icon);
     }
 
     m_activeActionDialog->actionFinished(action);
-    Q_ASSERT(m_actionCounter > 0);
-    --m_actionCounter;
+    Q_ASSERT(runningActionCount() >= 0);
 
     emit runningActionsCountChanged();
 
     action->deleteLater();
 }
-
-void ActionHandler::actionDialogClosed(ActionDialog *dialog)
-{
-    m_lastActionDialogCommand = dialog->command();
-}
-
-void ActionHandler::addItems(const QStringList &items, const QString &format, const QString &tabName)
-{
-    ClipboardBrowser *c = tabName.isEmpty() ? m_wnd->browser() : m_wnd->tab(tabName);
-    for (const auto &item : items)
-        c->add( createDataMap(format, item) );
-
-    if (m_lastAction) {
-        if (m_lastAction == sender())
-            c->setCurrent(items.size() - 1);
-        m_lastAction = nullptr;
-    }
-}
-
-void ActionHandler::addItem(const QByteArray &data, const QString &format, const QString &tabName)
-{
-    ClipboardBrowser *c = tabName.isEmpty() ? m_wnd->browser() : m_wnd->tab(tabName);
-    c->add( createDataMap(format, data) );
-
-    if (m_lastAction) {
-        if (m_lastAction == sender())
-            c->setCurrent(0);
-        m_lastAction = nullptr;
-    }
-}
-
-void ActionHandler::changeItem(const QByteArray &data, const QString &format, const QModelIndex &index)
-{
-    ClipboardBrowser *c = m_wnd->browserForItem(index);
-    if (c == nullptr)
-        return;
-
-    QVariantMap dataMap;
-    if (format == mimeItems)
-        deserializeData(&dataMap, data);
-    else
-        dataMap.insert(format, data);
-    c->model()->setData(index, dataMap, contentType::updateData);
-}
