@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -27,11 +27,11 @@
 #include <QObject>
 #include <QPointer>
 #include <QMap>
+#include <QSet>
 
 class Action;
 class ActionDialog;
 class ProcessManagerDialog;
-class ClipboardBrowser;
 class QDialog;
 class MainWindow;
 class QModelIndex;
@@ -45,24 +45,19 @@ class ActionHandler : public QObject
 public:
     explicit ActionHandler(MainWindow *mainWindow);
 
-    /**
-     * Create new action dialog.
-     * Dialog is destroyed when closed.
-     */
-    ActionDialog *createActionDialog(const QStringList &tabs);
-
-    bool hasRunningAction() const;
+    int runningActionCount() const { return m_actions.size() - m_internalActions.size(); }
 
     /** Open dialog with active commands. */
     void showProcessManagerDialog();
 
-    void setCurrentTab(const QString &tabName) { m_currentTabName = tabName; }
-
     void addFinishedAction(const QString &name);
 
     QVariantMap actionData(int id) const;
     void setActionData(int id, const QVariantMap &data);
 
+    void internalAction(Action *action);
+    bool isInternalActionId(int id) const;
+
 public slots:
     /** Execute action. */
     void action(Action *action);
@@ -78,20 +73,11 @@ private slots:
     /** Delete finished action and its menu item. */
     void closeAction(Action *action);
 
-    void actionDialogClosed(ActionDialog *dialog);
-
-    void addItems(const QStringList &items, const QString &format, const QString &tabName);
-    void addItem(const QByteArray &data, const QString &format, const QString &tabName);
-    void changeItem(const QByteArray &data, const QString &format, const QModelIndex &index);
-
 private:
     MainWindow *m_wnd;
-    QPointer<Action> m_lastAction;
-    int m_actionCounter;
     ProcessManagerDialog *m_activeActionDialog;
-    QString m_currentTabName;
-    Command m_lastActionDialogCommand;
     QHash<int, Action*> m_actions;
+    QSet<int> m_internalActions;
     int m_lastActionId = -1;
 };
 
