@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -19,8 +19,6 @@
 
 #include "clipboardbrowser.h"
 
-#include "common/appconfig.h"
-#include "common/action.h"
 #include "common/common.h"
 #include "common/contenttype.h"
 #include "common/log.h"
@@ -276,6 +274,8 @@ ClipboardBrowser::ClipboardBrowser(
     initSingleShotTimer( &m_timerSave, 30000, this, SLOT(saveItems()) );
     initSingleShotTimer( &m_timerEmitItemCount, 0, this, SLOT(emitItemCount()) );
     initSingleShotTimer( &m_timerUpdateSizes, 0, this, SLOT(updateSizes()) );
+    initSingleShotTimer( &m_timerUpdateItemWidgets, 0, this, SLOT(updateItemWidgets()) );
+    initSingleShotTimer( &m_timerUpdateCurrent, 0, this, SLOT(updateCurrent()) );
 
     // ScrollPerItem doesn't work well with hidden items
     setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
@@ -288,12 +288,11 @@ ClipboardBrowser::ClipboardBrowser(
 
     m_sharedData->theme.decorateBrowser(this);
     updateItemMaximumSize();
-
-    d.setSaveOnEnterKey(m_sharedData->saveOnReturnKey);
 }
 
 ClipboardBrowser::~ClipboardBrowser()
 {
+    delete m_editor.data();
     d.invalidateCache();
     saveUnsavedItems();
 }
@@ -366,6 +365,11 @@ bool ClipboardBrowser::isFiltered(int row) const
             && !m_sharedData->itemFactory->matches( ind, d.searchExpression() );
 }
 
+QVariantMap ClipboardBrowser::itemData(const QModelIndex &index) const
+{
+    return index.data(contentType::data).toMap();
+}
+
 bool ClipboardBrowser::hideFiltered(int row)
 {
     const bool hide = isFiltered(row);
@@ -427,10 +431,14 @@ void ClipboardBrowser::setEditorWidget(ItemEditorWidget *editor, bool changeClip
     bool active = editor != nullptr;
 
     if (m_editor != editor) {
+        if (m_editor) {
+            focusEditedIndex();
+            m_editor->hide();
+            m_editor->deleteLater();
+        }
+
         m_editor = editor;
         if (active) {
-            connect( editor, SIGNAL(destroyed()),
-                     this, SLOT(onEditorDestroyed()) );
             connect( editor, SIGNAL(save()),
                      this, SLOT(onEditorSave()) );
             if (changeClipboard) {
@@ -440,7 +448,7 @@ void ClipboardBrowser::setEditorWidget(ItemEditorWidget *editor, bool changeClip
             connect( editor, SIGNAL(cancel()),
                      this, SLOT(onEditorCancel()) );
             connect( editor, SIGNAL(invalidate()),
-                     editor, SLOT(deleteLater()) );
+                     this, SLOT(onEditorInvalidate()) );
             connect( editor, SIGNAL(searchRequest()),
                      this, SIGNAL(searchRequest()) );
             updateEditorGeometry();
@@ -450,22 +458,21 @@ void ClipboardBrowser::setEditorWidget(ItemEditorWidget *editor, bool changeClip
             setFocus();
             maybeEmitEditingFinished();
         }
+
+        emit internalEditorStateChanged(this);
     }
 
     setFocusProxy(m_editor);
+    setFocus();
 
     setAcceptDrops(!active);
 
     // Hide scrollbars while editing.
-    Qt::ScrollBarPolicy scrollbarPolicy = Qt::ScrollBarAlwaysOff;
-    if (!active) {
-        scrollbarPolicy = AppConfig(AppConfig::ThemeCategory).isOptionOn("show_scrollbars", true)
-                ? Qt::ScrollBarAsNeeded : Qt::ScrollBarAlwaysOff;
-    }
+    const auto scrollbarPolicy = active
+            ? Qt::ScrollBarAlwaysOff
+            : m_sharedData->theme.scrollbarPolicy();
     setVerticalScrollBarPolicy(scrollbarPolicy);
     setHorizontalScrollBarPolicy(scrollbarPolicy);
-
-    emit updateContextMenu(this);
 }
 
 void ClipboardBrowser::editItem(const QModelIndex &index, bool editNotes, bool changeClipboard)
@@ -549,12 +556,15 @@ void ClipboardBrowser::connectModelAndDelegate()
     // Save on change
     connect( &m, SIGNAL(rowsInserted(QModelIndex,int,int)),
              SLOT(delayedSaveItems()) );
-    connect( &m, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
+    connect( &m, SIGNAL(rowsRemoved(QModelIndex,int,int)),
              SLOT(delayedSaveItems()) );
-    connect( &m, SIGNAL(rowsAboutToBeMoved(QModelIndex,int,int,QModelIndex,int)),
+    connect( &m, SIGNAL(rowsMoved(QModelIndex,int,int,QModelIndex,int)),
              SLOT(delayedSaveItems()) );
     connect( &m, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
              SLOT(delayedSaveItems()) );
+
+    connect( &d, SIGNAL(itemWidgetCreated(PersistentDisplayItem)),
+             this, SIGNAL(itemWidgetCreated(PersistentDisplayItem)) );
 }
 
 void ClipboardBrowser::updateItemMaximumSize()
@@ -599,35 +609,21 @@ int ClipboardBrowser::findPreviousVisibleRow(int row)
     return row >= 0 ? row : -1;
 }
 
-void ClipboardBrowser::preload(int pixelsAboveCurrent, int pixelsBelowCurrent, const QModelIndex &current)
-{
-    preload(pixelsAboveCurrent, true, current);
-    preload(pixelsBelowCurrent, false, current);
-}
-
-void ClipboardBrowser::preload(int pixels, bool above, const QModelIndex &current)
+void ClipboardBrowser::preload(int pixels, bool above, const QModelIndex &start)
 {
     const int s = 2 * spacing();
     const int direction = above ? -1 : 1;
-    int row = current.row() + direction;
+    int row = start.row();
+    QModelIndex ind = index(row);
     int y = 0;
-    for ( auto ind = index(row); ind.isValid() && y < pixels; ind = index(row) ) {
-        if ( isRowHidden(row) ) {
-            d.invalidateCache(row);
-        } else {
-            itemWidget(ind);
+
+    for ( ; ind.isValid() && y < pixels; ind = index(row) ) {
+        if ( !isRowHidden(row) ) {
+            d.cache(ind);
             y += s + d.sizeHint(ind).height();
         }
         row += direction;
     }
-
-    // Unload item widgets below the threshold (unloding pixels above would change the scroll offset).
-    if (!above) {
-        for ( auto ind = index(row); ind.isValid(); ind = index(row) ) {
-            d.invalidateCache(row);
-            row += direction;
-        }
-    }
 }
 
 void ClipboardBrowser::moveToTop(const QModelIndex &index)
@@ -651,6 +647,21 @@ void ClipboardBrowser::maybeEmitEditingFinished()
         emit editingFinished();
 }
 
+QModelIndex ClipboardBrowser::firstUnpinnedIndex() const
+{
+    if (!m_itemSaver)
+        return QModelIndex();
+
+    auto indexes = QModelIndexList() << QModelIndex();
+    for (int row = 0; row < length(); ++row) {
+        indexes[0] = index(row);
+        if ( m_itemSaver->canMoveItems(indexes) )
+            return indexes[0];
+    }
+
+    return QModelIndex();
+}
+
 QVariantMap ClipboardBrowser::copyIndex(const QModelIndex &index) const
 {
     auto data = index.data(contentType::data).toMap();
@@ -776,36 +787,56 @@ QPixmap ClipboardBrowser::renderItemPreview(const QModelIndexList &indexes, int
     if (h == 0)
         return QPixmap();
 
-    const QRect rect(0, 0, qMin(maxWidth, viewport()->contentsRect().width()), qMin(maxHeight, h) );
+#if QT_VERSION >= 0x050000
+    const auto ratio = devicePixelRatio();
+#else
+    const auto ratio = 1;
+#endif
+    const auto frameLineWidth = 2 * ratio;
+
+    const auto height = qMin(maxHeight, h + s + 2 * frameLineWidth);
+    const auto width = qMin(maxWidth, viewport()->contentsRect().width() + 2 * frameLineWidth);
+    const QSize size(width, height);
+
+    QPixmap pix(size * ratio);
+#if QT_VERSION >= 0x050000
+    pix.setDevicePixelRatio(ratio);
+#endif
 
-    QPixmap pix( rect.size() );
-    pix.fill(Qt::transparent);
     QPainter p(&pix);
 
-    h = 0;
-    const QPoint pos = viewport()->pos() - QPoint(s / 2 + 1, s / 2 + 1);
+    h = frameLineWidth;
+    const QPoint pos = viewport()->pos();
     for (const auto &index : indexes) {
         if ( isIndexHidden(index) )
             continue;
-        render( &p, QPoint(0, h), visualRect(index).translated(pos).adjusted(0, 0, s, s) );
+        const QPoint position(frameLineWidth, h);
+        const auto rect = visualRect(index).translated(pos).adjusted(-s, -s, 2 * s, 2 * s);
+        render(&p, position, rect);
         h += visualRect(index).height() + s;
-        if ( h > rect.height() )
+        if ( h > height )
             break;
     }
 
-    p.setPen(Qt::black);
     p.setBrush(Qt::NoBrush);
-    p.drawRect( 0, 0, rect.width() - 1, rect.height() - 1 );
-    p.setPen(Qt::white);
-    p.drawRect( 1, 1, rect.width() - 3, rect.height() - 3 );
+
+    const auto x = frameLineWidth / 2;
+    const QRect rect(x / 2, x / 2, width - x, height - x);
+
+    QPen pen(Qt::black, x, Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin);
+    p.setPen(pen);
+    p.drawRect(rect);
+
+    pen.setColor(Qt::white);
+    p.setPen(pen);
+    p.drawRect( rect.adjusted(x, x, -x, -x) );
 
     return pix;
 }
 
 void ClipboardBrowser::onDataChanged(const QModelIndex &, const QModelIndex &)
 {
-    if (!isInternalEditorOpen())
-        emit updateContextMenu(this);
+    emit itemsChanged(this);
 }
 
 void ClipboardBrowser::onRowsInserted(const QModelIndex &, int first, int last)
@@ -834,16 +865,9 @@ void ClipboardBrowser::onItemCountChanged()
         m_timerEmitItemCount.start();
 }
 
-void ClipboardBrowser::onEditorDestroyed()
-{
-    setEditorWidget(nullptr);
-    // Set the focus back on the browser
-    setFocus();
-}
-
 void ClipboardBrowser::onEditorSave()
 {
-    Q_ASSERT(m_editor != nullptr);
+    Q_ASSERT(!m_editor.isNull());
     m_editor->commitData(&m);
     focusEditedIndex();
     saveItems();
@@ -851,12 +875,16 @@ void ClipboardBrowser::onEditorSave()
 
 void ClipboardBrowser::onEditorCancel()
 {
-    focusEditedIndex();
-
-    if ( isInternalEditorOpen() && m_editor->hasFocus() )
+    if ( isInternalEditorOpen() && m_editor->hasFocus() ) {
         maybeCloseEditors();
-    else
+    } else {
         emit searchHideRequest();
+    }
+}
+
+void ClipboardBrowser::onEditorInvalidate()
+{
+    setEditorWidget(nullptr);
 }
 
 void ClipboardBrowser::onEditorNeedsChangeClipboard()
@@ -913,38 +941,12 @@ void ClipboardBrowser::showEvent(QShowEvent *event)
 
 void ClipboardBrowser::currentChanged(const QModelIndex &current, const QModelIndex &previous)
 {
+    QListView::currentChanged(current, previous);
+
     if (previous.isValid())
         d.setItemWidgetCurrent(previous, false);
 
-    if ( current.isValid() ) {
-        d.setItemWidgetCurrent(current, true);
-        int row = -1;
-        if ( previous.isValid() && current < previous) {
-            row = findPreviousVisibleRow(current.row());
-            if (row == -1)
-                row = findNextVisibleRow(current.row());
-        } else {
-            row = findNextVisibleRow(current.row());
-            if (row == -1)
-                row = findPreviousVisibleRow(current.row());
-        }
-
-        if ( row != -1 && row != current.row() ) {
-            setCurrentIndex( index(row) );
-            return;
-        }
-
-        itemWidget(current);
-
-        // Preload next and previous pages so that up/down and page up/down keys scroll correctly.
-        const int h = viewport()->contentsRect().height();
-        preload(h, h, current);
-
-        scheduleDelayedItemsLayout();
-        executeDelayedItemsLayout();
-    }
-
-    QListView::currentChanged(current, previous);
+    m_timerUpdateCurrent.start();
 }
 
 void ClipboardBrowser::selectionChanged(const QItemSelection &selected,
@@ -955,7 +957,7 @@ void ClipboardBrowser::selectionChanged(const QItemSelection &selected,
         d.setItemWidgetSelected(index, true);
     for ( auto index : deselected.indexes() )
         d.setItemWidgetSelected(index, false);
-    emit updateContextMenu(this);
+    emit selectionChanged(this);
 }
 
 void ClipboardBrowser::focusInEvent(QFocusEvent *event)
@@ -1032,6 +1034,9 @@ void ClipboardBrowser::paintEvent(QPaintEvent *e)
 
     QListView::paintEvent(e);
 
+    if (!m_itemWidgetsToUpdate.isEmpty())
+        m_timerUpdateItemWidgets.start();
+
     // If dragging an item into list, draw indicator for dropping items.
     if (m_dragTargetRow != -1) {
         const int s = spacing();
@@ -1080,6 +1085,15 @@ void ClipboardBrowser::mouseReleaseEvent(QMouseEvent *event)
 void ClipboardBrowser::mouseMoveEvent(QMouseEvent *event)
 {
     if ( m_dragStartPosition.isNull() ) {
+        // WORKAROUND: After double-click, if window is hidden while mouse button is pressed,
+        //             release button event is received after window is show which can result
+        //             in items being selected.
+        if (m_ignoreMouseMoveWithButtonPressed) {
+            if ( QApplication::mouseButtons() & Qt::LeftButton )
+                return;
+            m_ignoreMouseMoveWithButtonPressed = false;
+        }
+
         QListView::mouseMoveEvent(event);
         return;
     }
@@ -1144,6 +1158,41 @@ void ClipboardBrowser::mouseMoveEvent(QMouseEvent *event)
         temporaryImage->drop();
 }
 
+void ClipboardBrowser::enterEvent(QEvent *event)
+{
+    m_ignoreMouseMoveWithButtonPressed = true;
+    QListView::enterEvent(event);
+}
+
+void ClipboardBrowser::doItemsLayout()
+{
+    // Keep visible current item (or the first one visible)
+    // on the same position visually after relayout.
+
+    // FIXME: Virtual method QListView::doItemsLayout() is undocumented
+    //        so other way should be used instead.
+
+    const auto current = currentIndex();
+    const auto currentRect = visualRect(current);
+    const auto viewRect = viewport()->contentsRect();
+
+    const bool currentIsNotVisible =
+            viewRect.bottom() < currentRect.top()
+            || currentRect.bottom() < viewRect.top();
+    const auto index = currentIsNotVisible ? indexNear(0) : current;
+
+    const auto rectBefore = visualRect(index);
+
+    QListView::doItemsLayout();
+
+    const auto rectAfter = visualRect(index);
+    const auto offset = rectAfter.top() - rectBefore.top();
+    if (offset != 0) {
+        QScrollBar *v = verticalScrollBar();
+        v->setValue(v->value() + offset);
+    }
+}
+
 bool ClipboardBrowser::openEditor()
 {
     const QModelIndexList selected = selectionModel()->selectedRows();
@@ -1321,6 +1370,21 @@ void ClipboardBrowser::keyPressEvent(QKeyEvent *event)
         int row = current.row();
         const int h = viewport()->contentsRect().height();
 
+        // Preload next and previous pages so that up/down and page up/down keys scroll correctly.
+        if ( m_itemWidgetsToUpdate.isEmpty() ) {
+            UpdatesLocker locker(this);
+            locker.lock();
+
+            if (key == Qt::Key_PageDown || key == Qt::Key_PageUp)
+                preload(h, (key == Qt::Key_PageUp), current);
+            else if (key == Qt::Key_Down || key == Qt::Key_Up)
+                preload(2 * spacing(), (key == Qt::Key_Up), current);
+            else if (key == Qt::Key_End)
+                preload(h, true, index(length() - 1));
+            scheduleDelayedItemsLayout();
+            executeDelayedItemsLayout();
+        }
+
         if (key == Qt::Key_PageDown || key == Qt::Key_PageUp) {
             event->accept();
 
@@ -1522,39 +1586,26 @@ bool ClipboardBrowser::add(const QVariantMap &data, int row)
 
 void ClipboardBrowser::addUnique(const QVariantMap &data)
 {
-    if ( moveToTop(hash(data)) ) {
+    auto newData = data;
+
+    if ( moveToTop(hash(newData)) ) {
         COPYQ_LOG("New item: Moving existing to top");
         return;
     }
 
-    QVariantMap newData = data;
-
-    // Don't store internal formats.
-    newData.remove(mimeWindowTitle);
-    newData.remove(mimeOwner);
-    newData.remove(mimeClipboardMode);
-    newData.remove(mimeCurrentTab);
-    newData.remove(mimeSelectedItems);
-    newData.remove(mimeCurrentItem);
-    newData.remove(mimeHidden);
-    newData.remove(mimeShortcut);
-    newData.remove(mimeOutputTab);
-    newData.remove(mimeSyncToClipboard);
-    newData.remove(mimeSyncToSelection);
-
 #ifdef HAS_MOUSE_SELECTIONS
     // When selecting text under X11, clipboard data may change whenever selection changes.
     // Instead of adding item for each selection change, this updates previously added item.
     // Also update previous item if the same selected text is copied to clipboard afterwards.
-    if ( newData.contains(mimeText)
-         // Don't update edited item.
-         && (!isInternalEditorOpen() || currentIndex().row() != 0)
-         )
-    {
-        const QModelIndex firstIndex = index(0);
+    if ( newData.contains(mimeText) ) {
+        const auto firstIndex = firstUnpinnedIndex();
         const QVariantMap previousData = copyIndex(firstIndex);
 
-        if ( previousData.contains(mimeText) ) {
+        if ( firstIndex.isValid()
+             && previousData.contains(mimeText)
+             // Don't update edited item.
+             && (!isInternalEditorOpen() || currentIndex() != firstIndex) )
+        {
             const auto newText = getTextData(newData);
             const auto oldText = getTextData(previousData);
             const auto isClipboard = isClipboardData(data);
@@ -1621,10 +1672,13 @@ void ClipboardBrowser::moveToClipboard()
 
 void ClipboardBrowser::delayedSaveItems()
 {
-    if ( !isLoaded() || tabName().isEmpty() || m_timerSave.isActive() )
+    if ( !isLoaded() || tabName().isEmpty() )
         return;
 
-    m_timerSave.start();
+    if ( !m_timerSave.isActive() )
+        m_timerSave.start();
+
+    emit itemsChanged(this);
 }
 
 void ClipboardBrowser::updateSizes()
@@ -1633,6 +1687,36 @@ void ClipboardBrowser::updateSizes()
     updateEditorGeometry();
 }
 
+void ClipboardBrowser::updateItemWidgets()
+{
+    UpdatesLocker locker(this);
+
+    const auto contents = viewport()->contentsRect();
+    for (auto index : m_itemWidgetsToUpdate) {
+        if (index.isValid()) {
+            const auto rect = visualRect(index);
+            if (contents.top() < rect.bottom() && rect.top() < contents.bottom()) {
+                locker.lock();
+                d.cache(index);
+            }
+        }
+    }
+
+    m_itemWidgetsToUpdate.clear();
+}
+
+void ClipboardBrowser::updateCurrent()
+{
+    if ( !updatesEnabled() ) {
+        m_timerUpdateCurrent.start();
+        return;
+    }
+
+    const auto current = currentIndex();
+    if ( current.isValid() )
+        d.setItemWidgetCurrent(current, true);
+}
+
 void ClipboardBrowser::saveUnsavedItems()
 {
     if ( m_timerSave.isActive() )
@@ -1697,8 +1781,9 @@ QWidget *ClipboardBrowser::currentItemPreview()
 
     const QModelIndex index = currentIndex();
     const bool antialiasing = m_sharedData->theme.isAntialiasingEnabled();
+    const auto data = itemData(index);
     ItemWidget *itemWidget =
-            m_sharedData->itemFactory->createItem(index, this, antialiasing, false, true);
+            m_sharedData->itemFactory->createItem(data, this, antialiasing, false, true);
     QWidget *w = itemWidget->widget();
 
     d.highlightMatches(itemWidget);
@@ -1718,14 +1803,14 @@ void ClipboardBrowser::findPrevious()
         m_editor->findPrevious(d.searchExpression());
 }
 
-ItemWidget *ClipboardBrowser::itemWidget(const QModelIndex &index)
+void ClipboardBrowser::updateItemWidget(const QModelIndex &index)
 {
-    return d.cache(index);
+    m_itemWidgetsToUpdate.append(index);
 }
 
 bool ClipboardBrowser::isInternalEditorOpen() const
 {
-    return m_editor != nullptr;
+    return !m_editor.isNull();
 }
 
 bool ClipboardBrowser::isExternalEditorOpen() const
@@ -1752,10 +1837,7 @@ bool ClipboardBrowser::maybeCloseEditors()
             return false;
     }
 
-    if (m_editor) {
-        m_editor->deleteLater();
-        m_editor = nullptr;
-    }
+    setEditorWidget(nullptr);
 
     emit closeExternalEditors();
 
