@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -32,14 +32,46 @@
 #include <QPointer>
 #include <QTimer>
 #include <QVariantMap>
+#include <QVector>
 
 #include <memory>
 
 class ItemEditorWidget;
 class ItemFactory;
+class PersistentDisplayItem;
 class QProgressBar;
 class QPushButton;
 
+class UpdatesLocker {
+public:
+    explicit UpdatesLocker(QWidget *widget)
+        : m_widget(widget)
+    {
+    }
+
+    ~UpdatesLocker()
+    {
+        if (m_updatesDisabled)
+            m_widget->setUpdatesEnabled(true);
+    }
+
+    void lock() {
+        if (m_updatesDisabled)
+            return;
+
+        m_updatesDisabled = m_widget->updatesEnabled();
+        if (m_updatesDisabled)
+            m_widget->setUpdatesEnabled(false);
+    }
+
+private:
+    UpdatesLocker(const UpdatesLocker &) = delete;
+    UpdatesLocker &operator =(const UpdatesLocker &) = delete;
+
+    QWidget *m_widget;
+    bool m_updatesDisabled = false;
+};
+
 /** List view of clipboard items. */
 class ClipboardBrowser : public QListView
 {
@@ -202,7 +234,7 @@ class ClipboardBrowser : public QListView
 
         void findPrevious();
 
-        ItemWidget *itemWidget(const QModelIndex & index);
+        void updateItemWidget(const QModelIndex &index);
 
         /**
          * Load items from configuration.
@@ -216,6 +248,8 @@ class ClipboardBrowser : public QListView
          */
         bool isFiltered(int row) const;
 
+        QVariantMap itemData(const QModelIndex &index) const;
+
     public slots:
         /**
          * Save items to configuration.
@@ -251,10 +285,6 @@ class ClipboardBrowser : public QListView
         bool eventFilter(QObject *watched, QEvent *event) override;
 
     signals:
-        /** Action dialog requested. */
-        void requestActionDialog(const QVariantMap &data);
-        /** Action dialog requested. */
-        void requestActionDialog(const QVariantMap &data, const Command &cmd, const QModelIndex &index);
         /** Show list request. */
         void requestShow(const ClipboardBrowser *self);
         /** Request clipboard change. */
@@ -269,13 +299,19 @@ class ClipboardBrowser : public QListView
 
         void showContextMenu(const QPoint &position);
 
-        void updateContextMenu(const ClipboardBrowser *self);
+        void itemsChanged(const ClipboardBrowser *self);
+
+        void selectionChanged(const ClipboardBrowser *self);
+
+        void internalEditorStateChanged(const ClipboardBrowser *self);
 
         void searchRequest();
         void searchHideRequest();
 
         void closeExternalEditors();
 
+        void itemWidgetCreated(const PersistentDisplayItem &selection);
+
     protected:
         void keyPressEvent(QKeyEvent *event) override;
         void contextMenuEvent(QContextMenuEvent *) override;
@@ -296,6 +332,10 @@ class ClipboardBrowser : public QListView
         void mouseReleaseEvent(QMouseEvent *event) override;
         void mouseMoveEvent(QMouseEvent *event) override;
 
+        void enterEvent(QEvent *event) override;
+
+        void doItemsLayout() override;
+
     private slots:
         void onDataChanged(const QModelIndex &a, const QModelIndex &b);
 
@@ -303,12 +343,12 @@ class ClipboardBrowser : public QListView
 
         void onItemCountChanged();
 
-        void onEditorDestroyed();
-
         void onEditorSave();
 
         void onEditorCancel();
 
+        void onEditorInvalidate();
+
         void onEditorNeedsChangeClipboard();
 
         void onEditorNeedsChangeClipboard(const QByteArray &bytes, const QString &mime);
@@ -323,6 +363,10 @@ class ClipboardBrowser : public QListView
          */
         void updateSizes();
 
+        void updateItemWidgets();
+
+        void updateCurrent();
+
     private:
         bool isLoaded() const;
 
@@ -370,8 +414,7 @@ class ClipboardBrowser : public QListView
         int findNextVisibleRow(int row);
         int findPreviousVisibleRow(int row);
 
-        void preload(int pixelsAboveCurrent, int pixelsBelowCurrent, const QModelIndex &current);
-        void preload(int pixels, bool above, const QModelIndex &current);
+        void preload(int pixels, bool above, const QModelIndex &start);
 
         void updateCurrentIndex();
 
@@ -379,6 +422,8 @@ class ClipboardBrowser : public QListView
 
         void maybeEmitEditingFinished();
 
+        QModelIndex firstUnpinnedIndex() const;
+
         ItemSaverPtr m_itemSaver;
         QString m_tabName;
         ClipboardModel m;
@@ -386,6 +431,9 @@ class ClipboardBrowser : public QListView
         QTimer m_timerSave;
         QTimer m_timerEmitItemCount;
         QTimer m_timerUpdateSizes;
+        QTimer m_timerUpdateItemWidgets;
+        QTimer m_timerUpdateCurrent;
+        bool m_ignoreMouseMoveWithButtonPressed = false;
         bool m_resizing = false;
 
         QPointer<ItemEditorWidget> m_editor;
@@ -397,6 +445,8 @@ class ClipboardBrowser : public QListView
         QPoint m_dragStartPosition;
 
         int m_filterRow = -1;
+
+        QVector<QPersistentModelIndex> m_itemWidgetsToUpdate;
 };
 
 #endif // CLIPBOARDBROWSER_H
