@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -61,7 +61,9 @@ ClipboardBrowser *ClipboardBrowserPlaceholder::createBrowser()
     }
 
     if (m_timerExpire.interval() > 0) {
-        connect( c.get(), SIGNAL(updateContextMenu(const ClipboardBrowser*)),
+        connect( c.get(), SIGNAL(selectionChanged(const ClipboardBrowser*)),
+                 &m_timerExpire, SLOT(start()) );
+        connect( c.get(), SIGNAL(itemsChanged(const ClipboardBrowser*)),
                  &m_timerExpire, SLOT(start()) );
     }
 
@@ -151,7 +153,7 @@ void ClipboardBrowserPlaceholder::createLoadButton()
     m_loadButton = new QPushButton(this);
     m_loadButton->setFlat(true);
 
-    const QIcon icon( getIcon("", IconRepeat) );
+    const QIcon icon( getIcon("", IconRedo) );
     m_loadButton->setIconSize( QSize(64, 64) );
     m_loadButton->setIcon(icon);
 
