@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -28,23 +28,18 @@
 class ItemFactory;
 
 struct ClipboardBrowserShared {
-    explicit ClipboardBrowserShared(ItemFactory *itemFactory);
-
-    void loadFromConfiguration();
-
     QString editor;
-    int maxItems;
-    bool textWrap;
-    bool viMode;
-    bool saveOnReturnKey;
-    bool moveItemOnReturnKey;
-    bool showSimpleItems;
-    int minutesToExpire;
-
-    ItemFactory *itemFactory;
-
+    int maxItems = 100;
+    bool textWrap = true;
+    bool viMode = false;
+    bool saveOnReturnKey = false;
+    bool moveItemOnReturnKey = false;
+    bool showSimpleItems = false;
+    int minutesToExpire = 0;
+    ItemFactory *itemFactory = nullptr;
     Theme theme;
 };
+
 using ClipboardBrowserSharedPtr = std::shared_ptr<ClipboardBrowserShared>;
 
 #endif // CLIPBOARDBROWSERSHARED_H
