@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -178,7 +178,7 @@ void ClipboardDialog::init()
 
     WindowGeometryGuard::create(this);
 
-    ui->actionRemove_Format->setIcon( getIcon("list-remove", IconRemove) );
+    ui->actionRemove_Format->setIcon( getIcon("list-remove", IconTrash) );
     ui->actionRemove_Format->setShortcut(shortcutToRemove());
     ui->listWidgetFormats->addAction(ui->actionRemove_Format);
 
