@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -37,16 +37,19 @@ public:
 
 signals:
     void triggerCommand(CommandAction *self, const QString &triggeredShortcut);
+    void enabled(bool enabled);
 
 protected:
     bool event(QEvent *event) override;
 
 private slots:
     void onTriggered();
+    void onChanged();
 
 private:
     Command m_command;
     QString m_triggeredShortcut;
+    bool m_wasEnabled = true;
 };
 
 #endif // COMMANDACTION_H
