@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -58,10 +58,10 @@ private slots:
     void tryPasteCommandFromClipboard();
     void copySelectedCommandsToClipboard();
     void onCommandDropped(const QString &text, int row);
+    void onCommandEnabledDisabled(int row);
 
-    void onCurrentCommandWidgetIconChanged(const QString &iconString);
+    void onCurrentCommandWidgetIconChanged();
     void onCurrentCommandWidgetNameChanged(const QString &name);
-    void onCurrentCommandWidgetAutomaticChanged(bool automatic);
 
     void onFinished(int result);
 
@@ -74,11 +74,12 @@ private slots:
     void on_lineEditFilterCommands_textChanged(const QString &text);
     void on_buttonBox_clicked(QAbstractButton* button);
 
-    void onAddCommands(const QList<Command> &commands);
+    void onAddCommands(const QVector<Command> &commands);
 
     void onClipboardChanged();
 
 private:
+    Command currentCommand(int row) const;
     Commands currentCommands() const;
 
     void addCommandsWithoutSave(const Commands &commands, int targetRow);
@@ -86,6 +87,8 @@ private:
     QString serializeSelectedCommands();
     bool hasUnsavedChanges() const;
 
+    void updateIcon(int row);
+
     Ui::CommandDialog *ui;
     Commands m_savedCommands;
 
