@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -134,7 +134,7 @@ CommandHelpButton::CommandHelpButton(QWidget *parent)
 
     const int x = smallIconSize();
     m_button->setIconSize(QSize(x, x));
-    m_button->setIcon( getIcon("help-faq", IconInfoSign) );
+    m_button->setIcon( getIcon("help-faq", IconInfoCircle) );
 
     connect( m_button, SIGNAL(clicked()),
              this, SLOT(showHelp()) );
