@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -37,47 +37,33 @@
 
 namespace {
 
-const char globalShortcutsDisabled[] = "DISABLED";
-
-const QIcon iconClipboard() { return getIcon("", IconPaste); }
+const QIcon iconClipboard() { return getIcon("", IconClipboard); }
 const QIcon iconMenu() { return getIcon("", IconBars); }
 const QIcon iconShortcut() { return getIcon("", IconKeyboard); }
+const QIcon iconScript() { return getIcon("", IconCog); }
+const QIcon iconDisplay() { return getIcon("", IconEye); }
 
-QStringList serializeShortcuts(const QList<QKeySequence> &shortcuts, bool enabled = true)
+QStringList serializeShortcuts(const QList<QKeySequence> &shortcuts)
 {
     if ( shortcuts.isEmpty() )
         return QStringList();
 
     QStringList shortcutTexts;
-    shortcutTexts.reserve( shortcuts.size() + (enabled ? 1 : 0) );
+    shortcutTexts.reserve( shortcuts.size() );
 
     for (const auto &shortcut : shortcuts)
         shortcutTexts.append(portableShortcutText(shortcut));
 
-    if (!enabled)
-        shortcutTexts.append(globalShortcutsDisabled);
-
     return shortcutTexts;
 }
 
 void deserializeShortcuts(
-        const QStringList &serializedShortcuts, ShortcutButton *shortcutButton,
-        QCheckBox *checkBoxEnabled = nullptr
-        )
+        const QStringList &serializedShortcuts, ShortcutButton *shortcutButton)
 {
     shortcutButton->resetShortcuts();
 
-    bool enabled = !serializedShortcuts.isEmpty();
-
-    for (const auto &shortcutText : serializedShortcuts) {
-        if (shortcutText == globalShortcutsDisabled)
-            enabled = false;
-        else
-            shortcutButton->addShortcut(shortcutText);
-    }
-
-    if (checkBoxEnabled)
-        checkBoxEnabled->setChecked(enabled);
+    for (const auto &shortcutText : serializedShortcuts)
+        shortcutButton->addShortcut(shortcutText);
 }
 
 } // namespace
@@ -103,6 +89,8 @@ CommandWidget::CommandWidget(QWidget *parent)
 
     ui->checkBoxAutomatic->setIcon(iconClipboard());
     ui->checkBoxInMenu->setIcon(iconMenu());
+    ui->checkBoxIsScript->setIcon(iconScript());
+    ui->checkBoxDisplay->setIcon(iconDisplay());
 
     // Add tab names to combo boxes.
     initTabComboBox(ui->comboBoxCopyToTab);
@@ -130,16 +118,17 @@ Command CommandWidget::command() const
     c.output = ui->comboBoxOutputFormat->currentText();
     c.wait   = ui->checkBoxWait->isChecked();
     c.automatic = ui->checkBoxAutomatic->isChecked();
+    c.display = ui->checkBoxDisplay->isChecked();
     c.inMenu  = ui->checkBoxInMenu->isChecked();
+    c.isGlobalShortcut  = ui->checkBoxGlobalShortcut->isChecked();
+    c.isScript  = ui->checkBoxIsScript->isChecked();
     c.transform = ui->checkBoxTransform->isChecked();
     c.remove = ui->checkBoxIgnore->isChecked();
     c.hideWindow = ui->checkBoxHideWindow->isChecked();
     c.enable = true;
     c.icon   = ui->buttonIcon->currentIcon();
     c.shortcuts = serializeShortcuts( ui->shortcutButton->shortcuts() );
-    c.globalShortcuts = serializeShortcuts(
-                ui->shortcutButtonGlobalShortcut->shortcuts(),
-                ui->checkBoxGlobalShortcut->isChecked() );
+    c.globalShortcuts = serializeShortcuts( ui->shortcutButtonGlobalShortcut->shortcuts() );
     c.tab    = ui->comboBoxCopyToTab->currentText();
     c.outputTab = ui->comboBoxOutputTab->currentText();
 
@@ -158,7 +147,10 @@ void CommandWidget::setCommand(const Command &c)
     ui->comboBoxOutputFormat->setEditText(c.output);
     ui->checkBoxWait->setChecked(c.wait);
     ui->checkBoxAutomatic->setChecked(c.automatic);
+    ui->checkBoxDisplay->setChecked(c.display);
     ui->checkBoxInMenu->setChecked(c.inMenu);
+    ui->checkBoxGlobalShortcut->setChecked(c.isGlobalShortcut);
+    ui->checkBoxIsScript->setChecked(c.isScript);
     ui->checkBoxTransform->setChecked(c.transform);
     ui->checkBoxIgnore->setChecked(c.remove);
     ui->checkBoxHideWindow->setChecked(c.hideWindow);
@@ -166,8 +158,7 @@ void CommandWidget::setCommand(const Command &c)
     deserializeShortcuts(c.shortcuts, ui->shortcutButton);
     deserializeShortcuts(
                 c.globalShortcuts,
-                ui->shortcutButtonGlobalShortcut,
-                ui->checkBoxGlobalShortcut);
+                ui->shortcutButtonGlobalShortcut);
     ui->comboBoxCopyToTab->setEditText(c.tab);
     ui->comboBoxOutputTab->setEditText(c.outputTab);
 }
@@ -193,14 +184,14 @@ void CommandWidget::showEvent(QShowEvent *event)
     QWidget::showEvent(event);
 }
 
-void CommandWidget::on_lineEditName_textChanged(const QString &name)
+void CommandWidget::on_lineEditName_textChanged(const QString &text)
 {
-    emit nameChanged(name);
+    emit nameChanged(text);
 }
 
-void CommandWidget::on_buttonIcon_currentIconChanged(const QString &iconString)
+void CommandWidget::on_buttonIcon_currentIconChanged()
 {
-    emit iconChanged(iconString);
+    emitIconChanged();
 }
 
 void CommandWidget::on_checkBoxShowAdvanced_stateChanged(int state)
@@ -215,7 +206,11 @@ void CommandWidget::on_checkBoxShowAdvanced_stateChanged(int state)
 void CommandWidget::on_checkBoxAutomatic_stateChanged(int)
 {
     updateWidgets();
-    emit automaticChanged(ui->checkBoxAutomatic->isChecked());
+}
+
+void CommandWidget::on_checkBoxDisplay_stateChanged(int)
+{
+    updateWidgets();
 }
 
 void CommandWidget::on_checkBoxInMenu_stateChanged(int)
@@ -223,6 +218,11 @@ void CommandWidget::on_checkBoxInMenu_stateChanged(int)
     updateWidgets();
 }
 
+void CommandWidget::on_checkBoxIsScript_stateChanged(int)
+{
+    updateWidgets();
+}
+
 void CommandWidget::on_checkBoxGlobalShortcut_stateChanged(int)
 {
     updateWidgets();
@@ -245,27 +245,35 @@ void CommandWidget::on_commandEdit_changed()
 
 void CommandWidget::updateWidgets()
 {
-    bool inMenu = ui->checkBoxInMenu->isChecked();
-    bool copyOrExecute = inMenu || ui->checkBoxAutomatic->isChecked();
-#ifdef NO_GLOBAL_SHORTCUTS
-    bool globalShortcut = false;
-#else
-    bool globalShortcut =
-            ui->checkBoxGlobalShortcut->isChecked()
-            && ui->shortcutButtonGlobalShortcut->shortcutCount() > 0;
-#endif
+    const bool isScript = ui->checkBoxIsScript->isChecked();
+    const bool isAutomatic = !isScript
+            && (ui->checkBoxAutomatic->isChecked() || ui->checkBoxDisplay->isChecked());
+    const bool inMenu = !isScript && ui->checkBoxInMenu->isChecked();
+    const bool isGlobalShortcut = !isScript && ui->checkBoxGlobalShortcut->isChecked();
+    const bool copyOrExecute = inMenu || isAutomatic;
 
-    ui->widgetGlobalShortcut->setVisible(ui->checkBoxGlobalShortcut->isChecked());
+    ui->checkBoxAutomatic->setVisible(!isScript);
+    ui->checkBoxDisplay->setVisible(!isScript);
+    ui->checkBoxInMenu->setVisible(!isScript);
+    ui->checkBoxGlobalShortcut->setVisible(!isScript);
+    ui->checkBoxIsScript->setVisible(!isAutomatic && !inMenu && !isGlobalShortcut);
+
+    ui->widgetGlobalShortcut->setVisible(isGlobalShortcut);
     ui->widgetMenuShortcut->setVisible(inMenu);
 
     ui->groupBoxMatchItems->setVisible(copyOrExecute);
-    ui->groupBoxCommand->setVisible(copyOrExecute || globalShortcut);
     ui->groupBoxAction->setVisible(copyOrExecute);
     ui->groupBoxInMenu->setVisible(inMenu);
     ui->groupBoxCommandOptions->setHidden(!copyOrExecute || ui->commandEdit->isEmpty());
 
-    ui->widgetSpacer->setVisible(
-                ui->widgetAdvanced->isHidden() || ui->groupBoxCommand->isHidden());
+    ui->widgetSpacer->setVisible(ui->widgetAdvanced->isHidden());
 
     ui->commandEdit->resizeToContent();
+
+    emitIconChanged();
+}
+
+void CommandWidget::emitIconChanged()
+{
+    emit iconChanged();
 }
