@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -48,27 +48,29 @@ public:
     void setFormats(const QStringList &formats);
 
 signals:
-    void iconChanged(const QString &iconString);
+    void iconChanged();
 
     void nameChanged(const QString &name);
 
-    void automaticChanged(bool automatic);
-
 protected:
     void resizeEvent(QResizeEvent *event) override;
     void showEvent(QShowEvent *event) override;
 
 private slots:
-    void on_lineEditName_textChanged(const QString &name);
+    void on_lineEditName_textChanged(const QString &text);
 
-    void on_buttonIcon_currentIconChanged(const QString &iconString);
+    void on_buttonIcon_currentIconChanged();
 
     void on_checkBoxShowAdvanced_stateChanged(int state);
 
     void on_checkBoxAutomatic_stateChanged(int);
 
+    void on_checkBoxDisplay_stateChanged(int);
+
     void on_checkBoxInMenu_stateChanged(int);
 
+    void on_checkBoxIsScript_stateChanged(int);
+
     void on_checkBoxGlobalShortcut_stateChanged(int);
 
     void on_shortcutButtonGlobalShortcut_shortcutAdded(const QKeySequence &shortcut);
@@ -82,6 +84,8 @@ private:
 
     void updateWidgets();
 
+    void emitIconChanged();
+
     Ui::CommandWidget *ui;
 };
 
