@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -219,7 +219,7 @@ void ConfigTabAppearance::on_comboBoxThemes_activated(const QString &text)
 void ConfigTabAppearance::onThemeModified(const QByteArray &bytes)
 {
     QTemporaryFile tmpfile;
-    if ( !openTemporaryFile(&tmpfile) )
+    if ( !openTemporaryFile(&tmpfile, ".ini") )
         return;
 
     tmpfile.write(bytes);
@@ -445,7 +445,10 @@ void ConfigTabAppearance::decoratePreview()
         m_preview = nullptr;
     }
 
-    const auto sharedData = std::make_shared<ClipboardBrowserShared>(m_itemFactory);
+    m_theme.updateTheme();
+
+    const auto sharedData = std::make_shared<ClipboardBrowserShared>();
+    sharedData->itemFactory = m_itemFactory;
     sharedData->theme = m_theme;
 
     auto c = new ClipboardBrowser(QString(), sharedData, this);
@@ -459,10 +462,7 @@ void ConfigTabAppearance::decoratePreview()
     c->addItems( QStringList()
                  << tr("Search string is %1.").arg( quoteString(searchFor) )
                  << tr("Select an item and\n"
-                       "press F2 to edit.")
-                 << tr("Select items and move them with\n"
-                       "CTRL and up or down key.")
-                 << tr("Remove item with Delete key.") );
+                       "press F2 to edit.") );
     for (int i = 1; i <= 20; ++i)
         c->add( tr("Example item %1").arg(i), -1 );
 
