@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -31,8 +31,8 @@ ConfigTabShortcuts::ConfigTabShortcuts(QWidget *parent)
 {
     ui->setupUi(this);
 
-    connect( ui->pushButtonOpenCommandDialog, SIGNAL(clicked()),
-             this, SIGNAL(openCommandDialogRequest()) );
+    connect( ui->shortcutsWidgetGeneral, SIGNAL(commandsSaved()),
+             this, SIGNAL(commandsSaved()) );
 }
 
 ConfigTabShortcuts::~ConfigTabShortcuts()
@@ -43,12 +43,14 @@ ConfigTabShortcuts::~ConfigTabShortcuts()
 void ConfigTabShortcuts::loadShortcuts(const QSettings &settings)
 {
     ui->shortcutsWidgetGeneral->loadShortcuts(settings);
-
-    const QIcon iconCommandDialog = getIcon("system-run", IconCog);
-    ui->pushButtonOpenCommandDialog->setIcon(iconCommandDialog);
 }
 
 void ConfigTabShortcuts::saveShortcuts(QSettings *settings) const
 {
     ui->shortcutsWidgetGeneral->saveShortcuts(settings);
 }
+
+void ConfigTabShortcuts::addCommands(const QVector<Command> &commands)
+{
+    ui->shortcutsWidgetGeneral->addCommands(commands);
+}
