@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,6 +20,7 @@
 #ifndef CONFIGTABSHORTCUTS_H
 #define CONFIGTABSHORTCUTS_H
 
+#include <QVector>
 #include <QWidget>
 
 namespace Ui {
@@ -28,6 +29,8 @@ class ConfigTabShortcuts;
 
 class QSettings;
 
+struct Command;
+
 class ConfigTabShortcuts : public QWidget
 {
     Q_OBJECT
@@ -42,8 +45,10 @@ public:
     /** Save shortcuts to settings file. */
     void saveShortcuts(QSettings *settings) const;
 
+    void addCommands(const QVector<Command> &commands);
+
 signals:
-    void openCommandDialogRequest();
+    void commandsSaved();
 
 private:
     Ui::ConfigTabShortcuts *ui;
