@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -90,6 +90,9 @@ ConfigurationManager::ConfigurationManager(ItemFactory *itemFactory, QWidget *pa
     ui->setupUi(this);
     setWindowIcon(appIcon());
 
+    connect( ui->configTabShortcuts, SIGNAL(commandsSaved()),
+             this, SIGNAL(commandsSaved()) );
+
     ui->spinBoxItems->setMaximum(Config::maxItems);
 
     if ( itemFactory && itemFactory->hasLoaders() )
@@ -99,13 +102,13 @@ ConfigurationManager::ConfigurationManager(ItemFactory *itemFactory, QWidget *pa
 
     initOptions();
 
-    connect( ui->configTabShortcuts, SIGNAL(openCommandDialogRequest()),
-             this, SIGNAL(openCommandDialogRequest()));
-
     if (itemFactory)
         ui->configTabAppearance->createPreview(itemFactory);
 
     loadSettings();
+
+    if (itemFactory)
+        ui->configTabShortcuts->addCommands( itemFactory->commands() );
 }
 
 ConfigurationManager::ConfigurationManager()
@@ -132,9 +135,9 @@ void ConfigurationManager::initTabIcons()
     tw->setTabIcon( tw->indexOf(ui->tabHistory), getIcon("", IconListAlt) );
     tw->setTabIcon( tw->indexOf(ui->tabItems), getIcon("", IconThList) );
     tw->setTabIcon( tw->indexOf(ui->tabTray), getIcon("", IconInbox) );
-    tw->setTabIcon( tw->indexOf(ui->tabNotifications), getIcon("", IconInfoSign) );
+    tw->setTabIcon( tw->indexOf(ui->tabNotifications), getIcon("", IconInfoCircle) );
     tw->setTabIcon( tw->indexOf(ui->tabShortcuts), getIcon("", IconKeyboard) );
-    tw->setTabIcon( tw->indexOf(ui->tabAppearance), getIcon("", IconPicture) );
+    tw->setTabIcon( tw->indexOf(ui->tabAppearance), getIcon("", IconImage) );
 }
 
 void ConfigurationManager::initPluginWidgets(ItemFactory *itemFactory)
@@ -145,7 +148,7 @@ void ConfigurationManager::initPluginWidgets(ItemFactory *itemFactory)
         ItemOrderList::ItemPtr pluginItem(new PluginItem(loader));
         const QIcon icon = getIcon(loader->icon());
         ui->itemOrderListPlugins->appendItem(
-                    loader->name(), itemFactory->isLoaderEnabled(loader), false, icon, pluginItem );
+                    loader->name(), itemFactory->isLoaderEnabled(loader), icon, pluginItem );
     }
 }
 
@@ -223,6 +226,7 @@ void ConfigurationManager::initOptions()
     bind<Config::save_filter_history>(ui->checkBoxSaveFilterHistory);
     bind<Config::autocompletion>(ui->checkBoxAutocompleteCommands);
     bind<Config::always_on_top>(ui->checkBoxAlwaysOnTop);
+    bind<Config::close_on_unfocus>(ui->checkBoxCloseOnUnfocus);
     bind<Config::open_windows_on_current_screen>(ui->checkBoxOpenWindowsOnCurrentScreen);
     bind<Config::transparency_focused>(ui->spinBoxTransparencyFocused);
     bind<Config::transparency>(ui->spinBoxTransparencyUnfocused);
