@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -79,7 +79,7 @@ signals:
 
     void error(const QString &error);
 
-    void openCommandDialogRequest();
+    void commandsSaved();
 
 private slots:
     void apply();
