@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -19,16 +19,18 @@
 
 #include "iconfactory.h"
 
-#include "common/appconfig.h"
 #include "gui/icons.h"
 #include "gui/iconfont.h"
 
 #include <QBitmap>
 #include <QCoreApplication>
 #include <QFile>
+#include <QFont>
 #include <QFontDatabase>
+#include <QFontMetrics>
 #include <QIcon>
 #include <QPainter>
+#include <QPainterPath>
 #include <QPaintDevice>
 #include <QPaintEngine>
 #include <QPixmap>
@@ -44,6 +46,10 @@ using QtIconEngine = QIconEngineV2;
 using QtIconEngine = QIconEngine;
 #endif
 
+#ifndef COPYQ_ICON_NAME
+# define COPYQ_ICON_NAME "copyq"
+#endif
+
 namespace {
 
 const char imagesRecourcePath[] = ":/images/";
@@ -71,10 +77,15 @@ QString sessionName()
     return qApp->property("CopyQ_session_name").toString();
 }
 
-QColor colorFromEnv(const char *envVaribleName)
+QString textFromEnv(const char *envVaribleName)
 {
     const auto name = qgetenv(envVaribleName);
-    return QColor( QString::fromUtf8(name) );
+    return QString::fromUtf8(name);
+}
+
+QColor colorFromEnv(const char *envVaribleName)
+{
+    return QColor( textFromEnv(envVaribleName) );
 }
 
 QColor appIconColorHelper()
@@ -128,6 +139,18 @@ QColor &sessionIconColorVariable()
     return color;
 }
 
+QString &sessionIconTagVariable()
+{
+    static QString tag = textFromEnv("COPYQ_SESSION_TAG");
+    return tag;
+}
+
+QColor &sessionIconTagColorVariable()
+{
+    static QColor color = colorFromEnv("COPYQ_SESSION_TAG_COLOR");
+    return color;
+}
+
 QPixmap imageFromPrefix(const QString &iconSuffix, const QString &resources)
 {
 #ifdef COPYQ_ICON_PREFIX
@@ -142,10 +165,11 @@ QPixmap imageFromPrefix(const QString &iconSuffix, const QString &resources)
 
 void drawFontIcon(QPixmap *pix, ushort id, int w, int h, const QColor &color)
 {
+    id = fixIconId(id);
+
     QPainter painter(pix);
-    QFont font = iconFont();
-    const int margins = h / 8;
-    font.setPixelSize(h - margins);
+    const QFont font = iconFontFitSize(w, h);
+
     painter.setFont(font);
     painter.setPen(color);
 
@@ -153,7 +177,10 @@ void drawFontIcon(QPixmap *pix, ushort id, int w, int h, const QColor &color)
     const auto flags = Qt::AlignTop | Qt::AlignLeft;
     const auto iconText = QString(QChar(id));
     auto boundingRect = painter.boundingRect(0, 0, w, h, flags, iconText);
-    const auto pos = QPoint(w - boundingRect.width(), h - boundingRect.height()) / 2;
+    const auto x = w - boundingRect.width();
+    const auto y = h - boundingRect.height();
+    // If icon is wider, assume that a tag will be rendered and align image to the right.
+    const auto pos = (w > h) ? QPoint(x, y / 2) : QPoint(x, y) / 2;
     boundingRect.moveTopLeft(pos);
 
     painter.drawText(boundingRect, flags, iconText);
@@ -170,21 +197,11 @@ QColor getDefaultIconColor(const QColor &color)
     return c;
 }
 
-bool loadIconFont()
-{
-    static bool iconFontLoaded = QFontDatabase::addApplicationFont(":/images/fontawesome-webfont.ttf") != -1;
-    return iconFontLoaded;
-}
-
-bool useSystemIcons()
-{
-    return !loadIconFont()
-            || AppConfig(AppConfig::ThemeCategory).isOptionOn("use_system_icons");
-}
-
 class IconEngine : public QtIconEngine
 {
 public:
+    static bool useSystemIcons;
+
     void paint(QPainter *painter, const QRect &rect, QIcon::Mode mode, QIcon::State state) override
     {
         painter->drawPixmap( rect, createPixmap(rect.size(), mode, state, painter) );
@@ -202,7 +219,12 @@ public:
 
     QPixmap createPixmap(QSize size, QIcon::Mode mode, QIcon::State state, QPainter *painter = nullptr)
     {
-        if ( m_iconId == 0 || useSystemIcons()) {
+#if QT_VERSION >= 0x050000
+        if (painter)
+            size *= painter->paintEngine()->paintDevice()->devicePixelRatio();
+#endif
+
+        if ( useSystemIcons || m_iconId == 0 || !loadIconFont() ) {
             // Tint tab icons.
             if ( m_iconName.startsWith(imagesRecourcePath + QString("tab_")) ) {
                 QPixmap pixmap(m_iconName);
@@ -210,27 +232,35 @@ public:
                 QPainter painter2(&pixmap);
                 painter2.setCompositionMode(QPainter::CompositionMode_SourceIn);
                 painter2.fillRect( pixmap.rect(), color(painter, mode) );
-                return pixmap;
+                return taggedIcon(&pixmap);
             }
 
             QIcon icon = m_iconName.startsWith(':') ? QIcon(m_iconName) : QIcon::fromTheme(m_iconName);
-            if ( !icon.isNull() )
-                return icon.pixmap(size, mode, state);
+            if ( !icon.isNull() ) {
+                auto pixmap = icon.pixmap(size, mode, state);
+                if (pixmap.size() != size) {
+                    pixmap = pixmap.scaled(size.width(), size.height(), Qt::KeepAspectRatio, Qt::SmoothTransformation);
+                    QPixmap pixmap2(size);
+                    pixmap2.fill(Qt::transparent);
+                    QPainter painter2(&pixmap2);
+                    const int x = size.width() - pixmap.width();
+                    const QRect rect( QPoint(x, 0), pixmap.size() );
+                    painter2.drawPixmap(rect, pixmap);
+                    pixmap = pixmap2;
+                }
+                return taggedIcon(&pixmap);
+            }
         }
 
-#if QT_VERSION >= 0x050000
-        if (painter)
-            size *= painter->paintEngine()->paintDevice()->devicePixelRatio();
-#endif
         QPixmap pixmap(size);
         pixmap.fill(Qt::transparent);
 
         if (m_iconId == 0)
-            return pixmap;
+            return taggedIcon(&pixmap);
 
         drawFontIcon( &pixmap, m_iconId, size.width(), size.height(), color(painter, mode) );
 
-        return pixmap;
+        return taggedIcon(&pixmap);
     }
 
     // QIconEngine doesn't seem to work in menus on OS X.
@@ -248,9 +278,9 @@ public:
         updateIcon(icon, size, QIcon::Active);
     }
 
-    static QIcon createIcon(ushort iconId, const QString &iconName)
+    static QIcon createIcon(ushort iconId, const QString &iconName, const QString &tag = QString(), const QColor &tagColor = QColor())
     {
-        IconEngine iconEngine(iconId, iconName);
+        IconEngine iconEngine(iconId, iconName, tag, tagColor);
         QIcon icon;
         iconEngine.updateIcon(&icon, 16);
         iconEngine.updateIcon(&icon, 32);
@@ -259,16 +289,58 @@ public:
         return icon;
     }
 #else
-    static QIcon createIcon(ushort iconId, const QString &iconName)
+    static QIcon createIcon(ushort iconId, const QString &iconName, const QString &tag = QString(), const QColor &tagColor = QColor())
     {
-        return QIcon( new IconEngine(iconId, iconName) );
+        return QIcon( new IconEngine(iconId, iconName, tag, tagColor) );
     }
 #endif
 
+    static void tagIcon(QPixmap *pix, const QString &tag, QColor color)
+    {
+        if ( tag.isEmpty() )
+            return;
+
+#if QT_VERSION >= 0x050000
+        const auto ratio = pix->devicePixelRatio();
+        pix->setDevicePixelRatio(1);
+#else
+        const auto ratio = 1;
+#endif
+        QPainter painter(pix);
+        painter.setRenderHint(QPainter::HighQualityAntialiasing, true);
+
+        const int h = pix->height();
+        const int strokeWidth = static_cast<int>(ratio + h / 16);
+
+        QFont font;
+        const auto pixelSize = pix->width() / 2;
+        if ( tag.size() == 1 ) {
+            font = iconFontFitSize(pixelSize, pixelSize);
+        } else {
+            font.setPixelSize(pixelSize);
+            font.setBold(true);
+        }
+        painter.setFont(font);
+
+        const auto rect = painter.fontMetrics().tightBoundingRect(tag);
+        const auto baseLineY = rect.bottom();
+        const auto pos = QPoint(strokeWidth, h - strokeWidth - baseLineY);
+
+        QPainterPath path;
+        path.addText(pos, font, tag);
+        const auto strokeColor = color.lightness() < 100 ? Qt::white : Qt::black;
+        painter.strokePath(path, QPen(strokeColor, strokeWidth));
+
+        painter.setPen(color);
+        painter.drawText(pos, tag);
+    }
+
 private:
-    IconEngine(ushort iconId, const QString &iconName)
+    IconEngine(ushort iconId, const QString &iconName, const QString &tag, QColor tagColor)
         : m_iconId(iconId)
         , m_iconName(iconName)
+        , m_tag(tag)
+        , m_tagColor(tagColor)
     {
     }
 
@@ -287,10 +359,20 @@ private:
         return color;
     }
 
+    QPixmap taggedIcon(QPixmap *pix)
+    {
+        tagIcon(pix, m_tag, m_tagColor);
+        return *pix;
+    }
+
     ushort m_iconId;
     QString m_iconName;
+    QString m_tag;
+    QColor m_tagColor;
 };
 
+bool IconEngine::useSystemIcons = false;
+
 void updateIcon(QIcon *icon, const QPixmap &pix, int extent)
 {
     if (pix.height() > extent)
@@ -322,16 +404,16 @@ QIcon getIconFromResources(const QString &iconName)
     return IconEngine::createIcon(0, imagesRecourcePath + iconName);
 }
 
-QIcon iconFromFile(const QString &fileName)
+QIcon iconFromFile(const QString &fileName, const QString &tag, const QColor &color)
 {
-    if ( fileName.isEmpty() )
+    if ( fileName.isEmpty() && tag.isEmpty() )
         return QIcon();
 
     const auto unicode = toIconId(fileName);
     if (unicode != 0)
-        return loadIconFont() ? IconEngine::createIcon(unicode, "") : QIcon();
+        return loadIconFont() ? IconEngine::createIcon(unicode, "", tag, color) : QIcon();
 
-    return QIcon(fileName);
+    return IconEngine::createIcon(0, fileName, tag, color);
 }
 
 QPixmap createPixmap(unsigned short id, const QColor &color, int size)
@@ -354,12 +436,9 @@ QIcon appIcon(AppIconType iconType)
     QIcon icon;
 
     if (sessionName.isEmpty())
-        icon = QIcon::fromTheme("copyq" + suffix);
+        icon = QIcon::fromTheme(COPYQ_ICON_NAME + suffix);
     else
-        icon = QIcon::fromTheme("copyq_" + sessionName + "-" + suffix);
-
-    const auto sessionColor = sessionIconColor();
-    const auto appColor = appIconColor();
+        icon = QIcon::fromTheme(COPYQ_ICON_NAME "_" + sessionName + "-" + suffix);
 
     QPixmap pix;
 
@@ -372,9 +451,16 @@ QIcon appIcon(AppIconType iconType)
     pix.setDevicePixelRatio(1);
 #endif
 
+    const auto sessionColor = sessionIconColor();
+    const auto appColor = appIconColor();
+
     if (sessionColor != appColor)
         replaceColor(&pix, appColor, sessionColor);
 
+    const auto &tag = sessionIconTagVariable();
+    if ( !tag.isEmpty() )
+        IconEngine::tagIcon(&pix, tag, sessionIconTagColorVariable());
+
     QIcon icon2;
     icon2.addPixmap(pix);
 
@@ -411,8 +497,7 @@ unsigned short toIconId(const QString &fileNameOrId)
     if ( fileNameOrId.size() != 1 )
         return 0;
 
-    const auto unicode = fileNameOrId.at(0).unicode();
-    return unicode >= IconFirst ? unicode : 0;
+    return fileNameOrId.at(0).unicode();
 }
 
 void setSessionIconColor(QColor color)
@@ -420,7 +505,34 @@ void setSessionIconColor(QColor color)
     sessionIconColorVariable() = color;
 }
 
+void setSessionIconTag(const QString &tag)
+{
+    sessionIconTagVariable() = tag;
+}
+
+void setSessionIconTagColor(QColor color)
+{
+    sessionIconTagColorVariable() = color;
+}
+
 QColor sessionIconColor()
 {
     return ::sessionIconColorVariable();
 }
+
+QString sessionIconTag()
+{
+
+    return ::sessionIconTagVariable();
+}
+
+QColor sessionIconTagColor()
+{
+
+    return ::sessionIconTagColorVariable();
+}
+
+void setUseSystemIcons(bool useSystemIcons)
+{
+    IconEngine::useSystemIcons = useSystemIcons;
+}
