@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,7 +20,9 @@
 #ifndef ICONFACTORY_H
 #define ICONFACTORY_H
 
-class QColor;
+#include <QColor>
+#include <QString>
+
 class QIcon;
 class QPixmap;
 class QPainter;
@@ -40,7 +42,7 @@ QIcon getIcon(const QVariant &iconOrIconId);
 
 QIcon getIconFromResources(const QString &iconName);
 
-QIcon iconFromFile(const QString &fileName);
+QIcon iconFromFile(const QString &fileName, const QString &tag = QString(), const QColor &color = QColor());
 
 unsigned short toIconId(const QString &fileNameOrId);
 
@@ -55,6 +57,16 @@ QColor getDefaultIconColor(const QWidget &widget, bool selected = false);
 
 void setSessionIconColor(QColor color);
 
+void setSessionIconTag(const QString &tag);
+
+void setSessionIconTagColor(QColor color);
+
 QColor sessionIconColor();
 
+QString sessionIconTag();
+
+QColor sessionIconTagColor();
+
+void setUseSystemIcons(bool useSystemIcons);
+
 #endif // ICONFACTORY_H
