@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -21,9 +21,16 @@
 #define ICONFONT_H
 
 class QFont;
+class QString;
+
+const QString &iconFontFamily();
+
+bool loadIconFont();
 
 int iconFontSizePixels();
 
 QFont iconFont();
 
+QFont iconFontFitSize(int w, int h);
+
 #endif // ICONFONT_H
