@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -56,12 +56,10 @@ void IconSelectButton::setCurrentIcon(const QString &iconString)
 
     if ( iconString.size() == 1 ) {
         const QChar c = iconString[0];
-        if ( c.unicode() >= IconFirst && QFontMetrics(iconFont()).inFont(c) ) {
-            setFont(iconFont());
-            setText(iconString);
-        } else {
-            m_currentIcon = QString();
-        }
+        const ushort id = fixIconId( c.unicode() );
+        m_currentIcon = QString(QChar(id));
+        setFont(iconFont());
+        setText(m_currentIcon);
     } else if ( !iconString.isEmpty() ) {
         const QIcon icon(iconString);
         if ( icon.isNull() )
