@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -22,9 +22,10 @@
 
 #include <QDialog>
 
-class QListWidget;
 class QModelIndex;
 
+class IconListWidget;
+
 class IconSelectDialog : public QDialog
 {
     Q_OBJECT
@@ -49,9 +50,9 @@ private slots:
 
 private:
     void addIcons();
-    void addIcon(ushort unicode);
+    void addIcon(ushort unicode, bool isBrand, const QStringList &searchTerms);
 
-    QListWidget *m_iconList;
+    IconListWidget *m_iconList;
     QString m_selectedIcon;
 };
 
