@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -21,6 +21,7 @@
 #include "ui_itemorderlist.h"
 
 #include "gui/iconfactory.h"
+#include "gui/iconfont.h"
 #include "gui/icons.h"
 
 #include <QDragEnterEvent>
@@ -47,6 +48,10 @@ ItemOrderList::ItemOrderList(QWidget *parent)
     ui->pushButtonAdd->hide();
     setFocusProxy(ui->listWidgetItems);
     setCurrentItemWidget(nullptr);
+
+    // Make icon wider so icon tag can be bigger.
+    const auto size = iconFontSizePixels();
+    ui->listWidgetItems->setIconSize( QSize(size * 3/2, size) );
 }
 
 ItemOrderList::~ItemOrderList()
@@ -68,22 +73,21 @@ void ItemOrderList::clearItems()
     m_items.clear();
 }
 
-void ItemOrderList::appendItem(const QString &label, bool checked, bool highlight, const QIcon &icon, const ItemPtr &item)
+void ItemOrderList::appendItem(const QString &label, bool checked, const QIcon &icon, const ItemPtr &item)
 {
-    insertItem(label, checked, highlight, icon, item, -1);
+    insertItem(label, checked, icon, item, -1);
 }
 
-void ItemOrderList::insertItem(const QString &label, bool checked, bool highlight, const QIcon &icon,
+void ItemOrderList::insertItem(const QString &label, bool checked, const QIcon &icon,
                                const ItemPtr &item, int targetRow)
 {
     QListWidget *list = ui->listWidgetItems;
     auto listItem = new QListWidgetItem(icon, label);
+    listItem->setCheckState(checked ? Qt::Checked : Qt::Unchecked);
+    m_items[listItem] = ItemWidgetPair(item, checked);
+
     const int row = targetRow >= 0 ? qMin(list->count(), targetRow) : list->count();
     list->insertItem(row, listItem);
-    listItem->setCheckState(checked ? Qt::Checked : Qt::Unchecked);
-    setItemHighlight(listItem, highlight);
-
-    m_items[listItem] = ItemWidgetPair(item);
 
     // Resize list to minimal size.
     if ( !isVisible() ) {
@@ -143,11 +147,11 @@ void ItemOrderList::setCurrentItemLabel(const QString &label)
         current->setText(label);
 }
 
-void ItemOrderList::setCurrentItemHighlight(bool highlight)
+void ItemOrderList::setItemIcon(int row, const QIcon &icon)
 {
-    QListWidgetItem *current = ui->listWidgetItems->currentItem();
-    if(current != nullptr)
-        setItemHighlight(current, highlight);
+    QListWidgetItem *item = listItem(row);
+    if(item != nullptr)
+        item->setIcon(icon);
 }
 
 QString ItemOrderList::itemLabel(int row) const
@@ -283,6 +287,17 @@ void ItemOrderList::on_listWidgetItems_itemSelectionChanged()
     emit itemSelectionChanged();
 }
 
+void ItemOrderList::on_listWidgetItems_itemChanged(QListWidgetItem *item)
+{
+    const auto row = ui->listWidgetItems->row(item);
+    const bool checked = isItemChecked(row);
+
+    if ( m_items[item].lastCheckedState != checked ) {
+        m_items[item].lastCheckedState = checked;
+        emit itemCheckStateChanged(row, checked);
+    }
+}
+
 QListWidgetItem *ItemOrderList::listItem(int row) const
 {
     Q_ASSERT(row >= 0 && row < itemCount());
@@ -304,13 +319,6 @@ void ItemOrderList::setCurrentItemWidget(QWidget *widget)
     }
 }
 
-void ItemOrderList::setItemHighlight(QListWidgetItem *item, bool highlight)
-{
-    QFont font = item->font();
-    font.setBold(highlight);
-    item->setFont(font);
-}
-
 QWidget *ItemOrderList::createWidget(QListWidgetItem *item)
 {
     ItemWidgetPair &pair = m_items[item];
