@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -61,9 +61,9 @@ public:
 
     void clearItems();
 
-    void appendItem(const QString &label, bool checked, bool highlight, const QIcon &icon, const ItemPtr &item);
+    void appendItem(const QString &label, bool checked, const QIcon &icon, const ItemPtr &item);
 
-    void insertItem(const QString &label, bool checked, bool highlight, const QIcon &icon, const ItemPtr &item, int targetRow);
+    void insertItem(const QString &label, bool checked, const QIcon &icon, const ItemPtr &item, int targetRow);
 
     /// Returns widget created by Item::createWidget() given @a row
     /// (could be nullptr is not yet created).
@@ -84,7 +84,7 @@ public:
 
     void setCurrentItemLabel(const QString &label);
 
-    void setCurrentItemHighlight(bool highlight);
+    void setItemIcon(int row, const QIcon &icon);
 
     QString itemLabel(int row) const;
 
@@ -101,6 +101,7 @@ signals:
     void addButtonClicked();
     void itemSelectionChanged();
     void dropped(const QString &text, int row);
+    void itemCheckStateChanged(int row, bool checked);
 
 protected:
     void dragEnterEvent(QDragEnterEvent *event) override;
@@ -115,19 +116,23 @@ private slots:
 
     void on_listWidgetItems_currentItemChanged(QListWidgetItem *current, QListWidgetItem *previous);
     void on_listWidgetItems_itemSelectionChanged();
+    void on_listWidgetItems_itemChanged(QListWidgetItem *item);
 
 private:
     struct ItemWidgetPair {
         ItemWidgetPair() {}
-        explicit ItemWidgetPair(const ItemPtr &item) : item(item) {}
+        explicit ItemWidgetPair(const ItemPtr &item, bool checked)
+            : item(item)
+            , lastCheckedState(checked)
+        {}
 
         ItemPtr item;
         QPointer<QWidget> widget;
+        bool lastCheckedState = true;
     };
 
     QListWidgetItem *listItem(int row) const;
     void setCurrentItemWidget(QWidget *widget);
-    void setItemHighlight(QListWidgetItem *item, bool highlight);
     QWidget *createWidget(QListWidgetItem *item);
 
     Ui::ItemOrderList *ui;
