@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,22 +20,23 @@
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
-#include "common/commandtester.h"
+#include "common/clipboardmode.h"
+#include "common/command.h"
 #include "gui/clipboardbrowsershared.h"
 #include "gui/menuitems.h"
-#include "gui/notificationbutton.h"
+#include "item/persistentdisplayitem.h"
 
 #include "platform/platformnativeinterface.h"
 
-#include <QAction>
-#include <QClipboard>
 #include <QMainWindow>
+#include <QModelIndex>
 #include <QPointer>
 #include <QSystemTrayIcon>
 #include <QTimer>
-#include <QModelIndex>
+#include <QVector>
 
 class Action;
+class ActionDialog;
 class ActionHandler;
 class ClipboardBrowser;
 class ClipboardBrowserPlaceholder;
@@ -43,11 +44,14 @@ class CommandAction;
 class CommandDialog;
 class ConfigurationManager;
 class ItemFactory;
+class Notification;
 class NotificationDaemon;
+class QAction;
+class QMimeData;
 class Theme;
 class TrayMenu;
-struct Command;
 struct MainWindowOptions;
+struct NotificationButton;
 
 Q_DECLARE_METATYPE(QPersistentModelIndex)
 Q_DECLARE_METATYPE(QList<QPersistentModelIndex>)
@@ -81,8 +85,6 @@ struct MainWindowOptions {
         , trayItems(5)
         , trayImages(true)
         , trayMenuOpenOnLeftClick(false)
-        , itemPopupInterval(0)
-        , clipboardNotificationLines(0)
         , transparency(0)
         , transparencyFocused(0)
         , hideTabs(false)
@@ -105,8 +107,6 @@ struct MainWindowOptions {
     int trayItems;
     bool trayImages;
     bool trayMenuOpenOnLeftClick;
-    int itemPopupInterval;
-    int clipboardNotificationLines;
     int transparency;
     int transparencyFocused;
 
@@ -123,14 +123,6 @@ struct MainWindowOptions {
     QString clipboardTab;
 };
 
-class Callable {
-public:
-    Callable() = default;
-    Callable(Callable &&) = default;
-    virtual ~Callable() = default;
-    virtual void operator()() = 0;
-};
-
 /**
  * Application's main window.
  *
@@ -162,7 +154,7 @@ public:
      * Try to close command dialog and return true on success.
      *
      * Note that dialog won't be closed if it has unsaved changes
-     * and user canceles the closing.
+     * and user cancels the closing.
      */
     bool maybeCloseCommandDialog();
 
@@ -209,26 +201,6 @@ public:
 
     void enterSearchMode(const QString &txt);
 
-    /** Show notification. */
-    void showMessage(const QString &title, //!< Message title.
-            const QString &msg, //!< Message text.
-            ushort icon = QSystemTrayIcon::Information,
-            //!< Type of popup.
-            int msec = 8000, //!< Show interval.
-            const QString &notificationId = QString(), //!< ID of notification.
-            const NotificationButtons &buttons = NotificationButtons());
-
-    /** Show popup with icon. */
-    void showMessage(const QString &title,
-            const QString &msg,
-            const QString &icon,
-            int msec,
-            const QString &notificationId = QString(),
-            const NotificationButtons &buttons = NotificationButtons());
-
-    /** Show clipboard content in notification. */
-    void showClipboardMessage(const QVariantMap &data);
-
     /** Show and focus main window. */
     void showWindow();
     /** Hide window to tray or minimize if tray is not available. */
@@ -286,35 +258,45 @@ public:
      */
     bool exportAllData(const QString &fileName);
 
-    /** Called after clipboard content changes. */
-    void clipboardChanged(const QVariantMap &data);
-
     /** Temporarily disable monitoring (i.e. adding new clipboard content to the first tab). */
     void disableClipboardStoring(bool disable);
 
     /** Return true only if monitoring is enabled. */
     bool isMonitoringEnabled() const;
 
-    /** Abort execution of automatic commands. */
-    void abortAutomaticCommands();
-
     QStringList tabs() const;
 
-    /** Update the first item in the first tab. */
-    void updateFirstItem(QVariantMap data);
-
     /// Used by config() command.
     QVariant config(const QStringList &nameValue);
 
     QVariantMap actionData(int id) const;
     void setActionData(int id, const QVariantMap &data);
 
-    void setCommands(const QList<Command> &commands);
+    void setCommands(const QVector<Command> &commands);
 
     void setSessionIconColor(QColor color);
 
+    void setSessionIconTag(const QString &tag);
+
+    void setSessionIconTagColor(QColor color);
+
     QColor sessionIconColor() const;
 
+    QString sessionIconTag() const;
+
+    QColor sessionIconTagColor() const;
+
+    void setTrayTooltip(const QString &tooltip);
+
+    QStringList menuItemMatchCommands(int actionId);
+    bool setMenuItemEnabled(int actionId, int menuItemMatchCommandIndex, bool enabled);
+
+    QVariantMap setDisplayData(int actionId, const QVariantMap &data);
+
+    QVector<Command> automaticCommands() const { return m_automaticCommands; }
+    QVector<Command> displayCommands() const { return m_displayCommands; }
+    QVector<Command> scriptCommands() const { return m_scriptCommands; }
+
 public slots:
     /** Close main window and exit the application. */
     void exit();
@@ -338,7 +320,7 @@ public slots:
     void showProcessManagerDialog();
 
     /** Open action dialog with given input data. */
-    void openActionDialog(const QVariantMap &data);
+    ActionDialog *openActionDialog(const QVariantMap &data);
 
     /** Open action dialog with input data from selected items. */
     void openActionDialog();
@@ -379,31 +361,22 @@ public slots:
     /** Show error popup message. */
     void showError(const QString &msg);
 
+    Notification *createNotification(const QString &id = QString());
+
     /** Open command dialog and add commands. */
-    void addCommands(const QList<Command> &commands);
+    void addCommands(const QVector<Command> &commands);
 
     /** Execute command on given input data. */
     Action *action(
             const QVariantMap &data,
             const Command &cmd,
-            const QModelIndex &outputIndex = QModelIndex());
+            const QModelIndex &outputIndex);
 
-    /** Add @a data to tab with given name (create if tab doesn't exist). */
-    void addToTab(
-            /// Item data (it may be updated if item with same text exists).
-            const QVariantMap &data,
-            /// Tab name of target tab.
-            const QString &tabName
-            );
-
-    /**
-     * Run automatic commands and add @a new clipboard to the first tab
-     * if commands didn't remove or transform the data.
-     */
-    void runAutomaticCommands(QVariantMap data);
+    void runInternalAction(Action *action);
+    bool isInternalActionId(int id) const;
 
     /** Set clipboard. */
-    void setClipboard(const QVariantMap &data, QClipboard::Mode mode);
+    void setClipboard(const QVariantMap &data, ClipboardMode mode);
 
     /** Set clipboard and synchronize selection if needed. */
     void setClipboard(const QVariantMap &data);
@@ -446,17 +419,14 @@ public slots:
     /** Set previous or last tab as current. */
     void previousTab();
 
-    /** Set window title and tray tool tip from data. */
-    void updateTitle(const QVariantMap &data);
-
-    bool canUpdateTitleFromScript() const;
+    void setClipboardData(const QVariantMap &data);
 
     /** Set text for filtering items. */
     void setFilter(const QString &text);
 
     void hideSearchBar();
 
-    void invoke(Callable *callable);
+    void updateShortcuts();
 
 #ifdef HAS_TESTS
     /**
@@ -488,7 +458,7 @@ public slots:
 
 signals:
     /** Request clipboard change. */
-    void changeClipboard(const QVariantMap &data, QClipboard::Mode mode);
+    void changeClipboard(const QVariantMap &data, ClipboardMode mode);
 
     void tabGroupSelected(bool selected);
 
@@ -496,11 +466,10 @@ signals:
 
     void commandsSaved();
 
-    void stopItemMenuCommandTester();
-    void stopTrayMenuCommandTester();
-
     void configurationChanged();
 
+    void disableClipboardStoringRequest(bool disable);
+
 protected:
     void keyPressEvent(QKeyEvent *event) override;
     void keyReleaseEvent(QKeyEvent *event) override;
@@ -528,7 +497,6 @@ protected:
 private slots:
     ClipboardBrowser *getTabForMenu();
     ClipboardBrowser *getTabForTrayMenu();
-    void updateTrayMenuItems();
     void addMenuItems(const QString &searchText);
     void addTrayMenuItems(const QString &searchText);
     void trayActivated(QSystemTrayIcon::ActivationReason reason);
@@ -554,6 +522,8 @@ private slots:
 
     void updateContextMenuTimeout();
 
+    void updateTrayMenuTimeout();
+
     void updateItemPreview();
     void setItemPreviewVisible(bool visible);
 
@@ -573,14 +543,6 @@ private slots:
 
     void showContextMenu(QPoint position);
 
-    void updateContextMenu(const ClipboardBrowser *browser);
-
-    void automaticCommandTestFinished(const Command &command, bool passed);
-
-    QAction *enableActionForCommand(QMenu *menu, const Command &command, bool enable);
-    void addCommandsToItemMenu(const Command &command, bool passed);
-    void addCommandsToTrayMenu(const Command &command, bool passed);
-
     void nextItemFormat();
     void previousItemFormat();
 
@@ -591,27 +553,56 @@ private slots:
 
     void onBrowserCreated(ClipboardBrowser *browser);
 
+    void onSelectionChanged(const ClipboardBrowser *browser);
+    void onItemsChanged(const ClipboardBrowser *browser);
+    void onInternalEditorStateChanged(const ClipboardBrowser *self);
+
     void onNotificationButtonClicked(const NotificationButton &button);
 
+    void onItemWidgetCreated(const PersistentDisplayItem &item);
+
+    void onDisplayActionFinished();
+
+    void onActionDialogAccepted(const Command &command, const QStringList &arguments, const QVariantMap &data);
+
+    void updateCommands();
+
+    void disableHideWindowOnUnfocus();
+    void enableHideWindowOnUnfocus();
+    void hideWindowIfNotActive();
+
 private:
     enum TabNameMatching {
         MatchExactTabName,
         MatchSimilarTabName
     };
 
+    struct MenuMatchCommands {
+        int actionId = -1;
+        QStringList matchCommands;
+        QVector< QPointer<QAction> > actions;
+        QMenu *menu = nullptr;
+    };
+
+    void runDisplayCommands();
+
+    void clearHiddenDisplayData();
+
+    void reloadBrowsers();
+
     ClipboardBrowserPlaceholder *createTab(const QString &name, TabNameMatching nameMatch);
 
     int findTabIndexExactMatch(const QString &name);
 
-    void clearTitle() { updateTitle(QVariantMap()); }
-
     /** Create menu bar and tray menu with items. Called once. */
     void createMenu();
 
     /** Create context menu for @a tab. It will be automatically deleted after closed. */
     void popupTabBarMenu(QPoint pos, const QString &tab);
 
-    void updateContextMenu();
+    void updateContextMenu(int intervalMsec);
+
+    void updateTrayMenu();
 
     void updateNotifications();
 
@@ -651,21 +642,19 @@ private:
 
     QAction *addItemAction(int id, QObject *receiver, const char *slot);
 
-    QList<Command> commandsForMenu(const QVariantMap &data, const QString &tabName);
+    QVector<Command> commandsForMenu(const QVariantMap &data, const QString &tabName);
     void addCommandsToItemMenu(ClipboardBrowser *c);
     void addCommandsToTrayMenu(const QVariantMap &clipboardData);
+    void addMenuMatchCommand(MenuMatchCommands *menuMatchCommands, const QString &matchCommand, QAction *act);
+    void runMenuCommandFilters(MenuMatchCommands *menuMatchCommands, const QVariantMap &data);
 
     bool isItemMenuDefaultActionValid() const;
 
     void updateToolBar();
 
-    void updateWindowTitle();
-
-    void updateTrayTooltip();
-
     void initTray();
 
-    void runAutomaticCommand(const Command &command);
+    void runDisplayCommand(const Command &command);
 
     bool isWindowVisible() const;
 
@@ -692,6 +681,8 @@ private:
 
     const Theme &theme() const;
 
+    Action *runScript(const QString &script, const QVariantMap &data = QVariantMap());
+
     ConfigurationManager *cm;
     Ui::MainWindow *ui;
 
@@ -706,16 +697,22 @@ private:
     QPointer<QAction> m_actionToggleClipboardStoring;
 
     ClipboardBrowserSharedPtr m_sharedData;
-    QList<Command> m_commands;
+
+    QVector<Command> m_automaticCommands;
+    QVector<Command> m_displayCommands;
+    QVector<Command> m_menuCommands;
+    QVector<Command> m_scriptCommands;
 
     PlatformWindowPtr m_lastWindow;
 
     QTimer m_timerUpdateFocusWindows;
     QTimer m_timerUpdateContextMenu;
+    QTimer m_timerUpdateTrayMenu;
     QTimer m_timerShowWindow;
     QTimer m_timerTrayAvailable;
     QTimer m_timerTrayIconSnip;
     QTimer m_timerSaveTabPositions;
+    QTimer m_timerHideWindowIfNotActive;
 
     NotificationDaemon *m_notifications;
 
@@ -729,13 +726,6 @@ private:
 
     QPointer<CommandDialog> m_commandDialog;
 
-    CommandTester m_itemMenuCommandTester;
-    CommandTester m_trayMenuCommandTester;
-    CommandTester m_automaticCommandTester;
-
-    QPointer<Action> m_currentAutomaticCommand;
-    bool m_canUpdateTitleFromScript;
-
     bool m_iconSnip;
 
     bool m_wasMaximized;
@@ -747,6 +737,13 @@ private:
     QVector< QPointer<QAction> > m_actions;
     MenuItems m_menuItems;
 
+    QList<PersistentDisplayItem> m_displayItemList;
+    PersistentDisplayItem m_currentDisplayItem;
+    QPointer<Action> m_currentDisplayAction;
+
+    MenuMatchCommands m_trayMenuMatchCommands;
+    MenuMatchCommands m_itemMenuMatchCommands;
+
 #ifdef HAS_TESTS
     /// Key clicks sequence number last returned by sendKeyClicks().
     uint m_sentKeyClicks = 0;
