@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -19,7 +19,6 @@
 
 #include "gui/notification.h"
 
-#include "common/appconfig.h"
 #include "common/common.h"
 #include "common/display.h"
 #include "common/textdata.h"
@@ -27,7 +26,6 @@
 #include "gui/icons.h"
 
 #include <QApplication>
-#include <QClipboard>
 #include <QDialog>
 #include <QDialogButtonBox>
 #include <QGridLayout>
@@ -73,13 +71,7 @@ private:
 
 } // namespace
 
-Notification::Notification(const QString &id, const QString &title, const NotificationButtons &buttons)
-    : m_id(id)
-    , m_body(nullptr)
-    , m_titleLabel(nullptr)
-    , m_iconLabel(nullptr)
-    , m_msgLabel(nullptr)
-    , m_opacity(1.0)
+Notification::Notification()
 {
     auto bodyLayout = new QVBoxLayout(this);
     bodyLayout->setMargin(8);
@@ -87,8 +79,8 @@ Notification::Notification(const QString &id, const QString &title, const Notifi
     bodyLayout->addWidget(m_body);
     bodyLayout->setSizeConstraint(QLayout::SetMaximumSize);
 
-    auto layout = new QGridLayout(m_body);
-    layout->setMargin(0);
+    m_layout = new QGridLayout(m_body);
+    m_layout->setMargin(0);
 
     m_iconLabel = new QLabel(this);
     m_iconLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
@@ -96,39 +88,37 @@ Notification::Notification(const QString &id, const QString &title, const Notifi
     m_msgLabel = new QLabel(this);
     m_msgLabel->setAlignment(Qt::AlignTop | Qt::AlignAbsolute);
 
-    if ( !title.isEmpty() ) {
-        m_titleLabel = new QLabel(this);
-        m_titleLabel->setObjectName("NotificationTitle");
-        m_titleLabel->setTextFormat(Qt::PlainText);
-        m_titleLabel->setText(title);
-
-        layout->addWidget(m_iconLabel, 0, 0);
-        layout->addWidget(m_titleLabel, 0, 1, Qt::AlignCenter);
-        layout->addWidget(m_msgLabel, 1, 0, 1, 2);
-    } else {
-        layout->addWidget(m_iconLabel, 0, 0, Qt::AlignTop);
-        layout->addWidget(m_msgLabel, 0, 1);
-    }
-
-
-    if ( !buttons.isEmpty() ) {
-        auto buttonLayout = new QHBoxLayout();
-        buttonLayout->addStretch();
-        layout->addLayout(buttonLayout, 2, 0, 1, 2);
-
-        for (const auto &button : buttons) {
-            const auto buttonWidget = new NotificationButtonWidget(button, this);
-            connect( buttonWidget, SIGNAL(clicked(NotificationButton)),
-                     this, SLOT(onButtonClicked(NotificationButton)) );
-            buttonLayout->addWidget(buttonWidget);
-        }
-    }
+    setTitle(QString());
 
     setWindowFlags(Qt::ToolTip);
     setWindowOpacity(m_opacity);
     setAttribute(Qt::WA_ShowWithoutActivating);
 }
 
+void Notification::setTitle(const QString &title)
+{
+    if ( !title.isEmpty() ) {
+        if (!m_titleLabel)
+            m_titleLabel = new QLabel(this);
+
+        m_titleLabel->setObjectName("NotificationTitle");
+        m_titleLabel->setTextFormat(Qt::PlainText);
+        m_titleLabel->setText(title);
+
+        m_layout->addWidget(m_iconLabel, 0, 0);
+        m_layout->addWidget(m_titleLabel, 0, 1, Qt::AlignCenter);
+        m_layout->addWidget(m_msgLabel, 1, 0, 1, 2);
+    } else {
+        if (m_titleLabel) {
+            m_titleLabel->deleteLater();
+            m_titleLabel = nullptr;
+        }
+
+        m_layout->addWidget(m_iconLabel, 0, 0, Qt::AlignTop);
+        m_layout->addWidget(m_msgLabel, 0, 1);
+    }
+}
+
 void Notification::setMessage(const QString &msg, Qt::TextFormat format)
 {
     m_msgLabel->setTextFormat(format);
@@ -146,6 +136,11 @@ void Notification::setIcon(const QString &icon)
     m_icon = icon;
 }
 
+void Notification::setIcon(ushort icon)
+{
+    m_icon = QString(QChar(icon));
+}
+
 void Notification::setInterval(int msec)
 {
     if (msec >= 0) {
@@ -162,14 +157,50 @@ void Notification::setOpacity(qreal opacity)
     setWindowOpacity(m_opacity);
 }
 
+void Notification::setButtons(const NotificationButtons &buttons)
+{
+    for (const auto &buttonWidget : findChildren<NotificationButtonWidget*>())
+        buttonWidget->deleteLater();
+
+    if ( !buttons.isEmpty() ) {
+        if (!m_buttonLayout)
+            m_buttonLayout = new QHBoxLayout();
+
+        m_buttonLayout->addStretch();
+        m_layout->addLayout(m_buttonLayout, 2, 0, 1, 2);
+
+        for (const auto &button : buttons) {
+            const auto buttonWidget = new NotificationButtonWidget(button, this);
+            connect( buttonWidget, SIGNAL(clicked(NotificationButton)),
+                     this, SLOT(onButtonClicked(NotificationButton)) );
+            m_buttonLayout->addWidget(buttonWidget);
+        }
+    } else if (m_buttonLayout) {
+        m_buttonLayout->deleteLater();
+        m_buttonLayout = nullptr;
+    }
+}
+
 void Notification::updateIcon()
 {
     const QColor color = getDefaultIconColor(*this);
     const auto height = static_cast<int>( m_msgLabel->fontMetrics().lineSpacing() * 1.2 );
     const auto iconId = toIconId(m_icon);
-    const auto pixmap = iconId == 0
+
+#if QT_VERSION >= 0x050000
+    const auto ratio = devicePixelRatio();
+#else
+    const auto ratio = 1;
+#endif
+
+    auto pixmap = iconId == 0
             ? QPixmap(m_icon)
-            : createPixmap(iconId, color, height);
+            : createPixmap(iconId, color, height * ratio);
+
+#if QT_VERSION >= 0x050000
+    pixmap.setDevicePixelRatio(ratio);
+#endif
+
     m_iconLabel->setPixmap(pixmap);
     m_iconLabel->resize(pixmap.size());
 }
