@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -140,28 +140,7 @@ void Theme::loadTheme(const QSettings &settings)
             m_theme[key].setValue(value);
     }
 
-    const auto margin = itemMargin();
-    m_margins = QSize(margin * 2 + 6, margin);
-
-    // search style
-    m_searchPalette.setColor(QPalette::Base, color("find_bg"));
-    m_searchPalette.setColor(QPalette::Text, color("find_fg"));
-    m_searchFont = font("find_font");
-
-    // editor style
-    m_editorPalette.setColor(QPalette::Base, color("edit_bg"));
-    m_editorPalette.setColor(QPalette::Text, color("edit_fg"));
-    m_editorFont = font("edit_font");
-
-    // number style
-    m_showRowNumber = value("show_number").toBool();
-    m_rowNumberPalette.setColor(QPalette::Text, color("num_fg"));
-    m_rowNumberFont = font("num_font");
-    m_rowNumberSize = QFontMetrics(m_rowNumberFont).boundingRect( QString("0123") ).size()
-            + QSize(m_margins.width() / 2, 2 * m_margins.height());
-
-    m_antialiasing = isAntialiasingEnabled();
-
+    updateTheme();
 }
 
 void Theme::saveTheme(QSettings *settings) const
@@ -284,10 +263,7 @@ void Theme::decorateToolBar(QWidget *toolBar) const
 
 void Theme::decorateScrollArea(QAbstractScrollArea *scrollArea) const
 {
-    const Qt::ScrollBarPolicy scrollbarPolicy =
-            value("show_scrollbars").toBool()
-            ? Qt::ScrollBarAsNeeded
-            : Qt::ScrollBarAlwaysOff;
+    const auto scrollbarPolicy = this->scrollbarPolicy();
     scrollArea->setVerticalScrollBarPolicy(scrollbarPolicy);
     scrollArea->setHorizontalScrollBarPolicy(scrollbarPolicy);
 }
@@ -335,6 +311,18 @@ QString Theme::getNotificationStyleSheet() const
             ;
 }
 
+Qt::ScrollBarPolicy Theme::scrollbarPolicy() const
+{
+    return value("show_scrollbars").toBool()
+            ? Qt::ScrollBarAsNeeded
+            : Qt::ScrollBarAlwaysOff;
+}
+
+bool Theme::useSystemIcons() const
+{
+    return value("use_system_icons").toBool();
+}
+
 QFont Theme::themeFontFromString(const QString &fontString) const
 {
     QFont font;
@@ -487,6 +475,31 @@ void Theme::resetTheme()
     m_theme["style_main_window"] = Option(false, "checked", ui ? ui->checkBoxStyleMainWindow : nullptr);
 }
 
+void Theme::updateTheme()
+{
+    const auto margin = itemMargin();
+    m_margins = QSize(margin + 2, margin);
+
+    // search style
+    m_searchPalette.setColor(QPalette::Base, color("find_bg"));
+    m_searchPalette.setColor(QPalette::Text, color("find_fg"));
+    m_searchFont = font("find_font");
+
+    // editor style
+    m_editorPalette.setColor(QPalette::Base, color("edit_bg"));
+    m_editorPalette.setColor(QPalette::Text, color("edit_fg"));
+    m_editorFont = font("edit_font");
+
+    // number style
+    m_showRowNumber = value("show_number").toBool();
+    m_rowNumberPalette.setColor(QPalette::Text, color("num_fg"));
+    m_rowNumberFont = font("num_font");
+    m_rowNumberSize = QFontMetrics(m_rowNumberFont).boundingRect( QString("0123") ).size()
+            + QSize(m_margins.width(), m_margins.height());
+
+    m_antialiasing = value("font_antialiasing").toBool();
+}
+
 void Theme::decorateBrowser(QAbstractScrollArea *c) const
 {
     decorateScrollArea(c);
