@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -82,16 +82,22 @@ public:
     /** Return stylesheet for notifications. */
     QString getNotificationStyleSheet() const;
 
+    Qt::ScrollBarPolicy scrollbarPolicy() const;
+
+    bool useSystemIcons() const;
+
     QFont themeFontFromString(const QString &fontString) const;
 
     bool isAntialiasingEnabled() const;
 
     void resetTheme();
 
+    void updateTheme();
+
     bool showRowNumber() const { return m_showRowNumber; }
     const QFont &rowNumberFont() const { return m_rowNumberFont; }
     const QPalette &rowNumberPalette() const { return m_rowNumberPalette; }
-    QSize rowNumberSize() const { return m_rowNumberSize; }
+    QSize rowNumberSize() const { return m_showRowNumber ? m_rowNumberSize : QSize(0, 0); }
 
     const QFont &editorFont() const { return m_editorFont; }
     const QPalette &editorPalette() const { return m_editorPalette; }
