@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -35,7 +35,7 @@ const char propertyGeometryLocked[] = "CopyQ_geometry_locked";
 
 bool openOnCurrentScreen()
 {
-    return AppConfig().isOptionOn("open_windows_on_current_screen");
+    return AppConfig().option<Config::open_windows_on_current_screen>();
 }
 
 } // namespace
