@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -27,6 +27,7 @@
 #include "item/itemfactory.h"
 #include "item/itemwidget.h"
 #include "item/itemeditorwidget.h"
+#include "item/persistentdisplayitem.h"
 
 #include <QEvent>
 #include <QPainter>
@@ -37,19 +38,12 @@ namespace {
 
 const char propertySelectedItem[] = "CopyQ_selected";
 
-inline void reset(ItemWidget **ptr, ItemWidget *value = nullptr)
-{
-    delete *ptr;
-    *ptr = value != nullptr ? value : nullptr;
-}
-
 } // namespace
 
 ItemDelegate::ItemDelegate(ClipboardBrowser *view, const ClipboardBrowserSharedPtr &sharedData, QWidget *parent)
     : QItemDelegate(parent)
     , m_view(view)
     , m_sharedData(sharedData)
-    , m_saveOnReturnKey(true)
     , m_re()
     , m_maxSize(2048, 2048 * 8)
     , m_idealWidth(0)
@@ -61,9 +55,9 @@ ItemDelegate::~ItemDelegate() = default;
 
 QSize ItemDelegate::sizeHint(const QModelIndex &index) const
 {
-    int row = index.row();
-    if ( row < m_cache.size() ) {
-        const ItemWidget *w = m_cache[row];
+    const int row = index.row();
+    if ( static_cast<size_t>(row) < m_cache.size() ) {
+        const ItemWidget *w = cacheOrNull(row);
         if (w != nullptr) {
             QWidget *ww = w->widget();
             const auto margins = m_sharedData->theme.margins();
@@ -72,7 +66,7 @@ QSize ItemDelegate::sizeHint(const QModelIndex &index) const
                           qMax(ww->height() + 2 * margins.height(), rowNumberSize.height()) );
         }
     }
-    return QSize(0, 512);
+    return QSize(0, 100);
 }
 
 QSize ItemDelegate::sizeHint(const QStyleOptionViewItem &,
@@ -85,14 +79,8 @@ bool ItemDelegate::eventFilter(QObject *obj, QEvent *event)
 {
     // resize event for items
     if ( event->type() == QEvent::Resize ) {
-        int row = 0;
-        for (; row < m_cache.size(); ++row) {
-            auto w = m_cache[row];
-            if (w && w->widget() == obj)
-                break;
-        }
-
-        Q_ASSERT( row < m_cache.size() );
+        const int row = findWidgetRow(obj);
+        Q_ASSERT(row != -1);
 
         const auto index = m_view->model()->index(row, 0);
         if ( index.isValid() )
@@ -107,7 +95,7 @@ void ItemDelegate::dataChanged(const QModelIndex &a, const QModelIndex &b)
     for ( int row = a.row(); row <= b.row(); ++row ) {
         auto item = &m_cache[row];
         if (*item) {
-            reset(item);
+            item->reset();
             cache( m_view->index(row) );
         }
     }
@@ -115,82 +103,97 @@ void ItemDelegate::dataChanged(const QModelIndex &a, const QModelIndex &b)
 
 void ItemDelegate::rowsRemoved(const QModelIndex &, int start, int end)
 {
-    for( int i = end; i >= start; --i ) {
-        delete m_cache.takeAt(i);
-    }
+    m_cache.erase(std::begin(m_cache) + start, std::begin(m_cache) + end + 1);
 }
 
 void ItemDelegate::rowsMoved(const QModelIndex &, int sourceStart, int sourceEnd,
                              const QModelIndex &, int destinationRow)
 {
-    if (sourceStart < destinationRow) {
-        int dest = destinationRow;
-        for( int i = sourceEnd; i >= sourceStart; --i ) {
-            --dest;
-            m_cache.move(i, dest);
-        }
-    } else {
-        int dest = destinationRow;
-        for( int i = sourceStart; i <= sourceEnd; ++i ) {
-            m_cache.move(i, dest);
-            ++dest;
-        }
+    auto count = sourceEnd - sourceStart + 1;
+    auto from = sourceStart;
+    auto to = destinationRow;
+
+    if (to < from) {
+        std::swap(from, to);
+        to += count;
+        count = to - from - count;
     }
+
+    const auto start1 = std::begin(m_cache) + from;
+    const auto start2 = start1 + count;
+    const auto end2 = std::begin(m_cache) + to;
+    std::rotate(start1, start2, end2);
 }
 
 void ItemDelegate::rowsInserted(const QModelIndex &, int start, int end)
 {
-    for( int i = start; i <= end; ++i )
-        m_cache.insert(i, nullptr);
+    const auto count = static_cast<size_t>(end - start + 1);
+    const auto oldSize = m_cache.size();
+    m_cache.resize(oldSize + count);
+    std::rotate( std::begin(m_cache) + start,
+                 std::begin(m_cache) + oldSize,
+                 std::end(m_cache) );
 }
 
 ItemWidget *ItemDelegate::cache(const QModelIndex &index)
 {
-    int n = index.row();
-
-    ItemWidget *w = m_cache[n];
+    const int row = index.row();
+    ItemWidget *w = cacheOrNull(row);
     if (w == nullptr) {
-        QWidget *parent = m_view->viewport();
-        const bool antialiasing = m_sharedData->theme.isAntialiasingEnabled();
-        w = m_sharedData->showSimpleItems
-                ? m_sharedData->itemFactory->createSimpleItem(index, parent, antialiasing)
-                : m_sharedData->itemFactory->createItem(index, parent, antialiasing);
-        setIndexWidget(index, w);
+        auto data = m_view->itemData(index);
+        data.insert(mimeCurrentTab, m_view->tabName());
+        w = updateCache(index, data);
+        emit itemWidgetCreated(PersistentDisplayItem(this, data, w->widget()));
     }
 
     return w;
 }
 
+void ItemDelegate::updateCache(QObject *widget, const QVariantMap &data)
+{
+    const auto row = findWidgetRow(widget);
+    if (row == -1)
+        return;
+
+    const auto index = m_view->index(row);
+    updateCache(index, data);
+}
+
 ItemWidget *ItemDelegate::cacheOrNull(int row) const
 {
-    return m_cache[row];
+    return m_cache[static_cast<size_t>(row)].get();
 }
 
 bool ItemDelegate::hasCache(const QModelIndex &index) const
 {
-    return m_cache[index.row()] != nullptr;
+    const int row = index.row();
+    return cacheOrNull(row) != nullptr;
 }
 
 void ItemDelegate::setItemSizes(QSize size, int idealWidth)
 {
     const auto margins = m_sharedData->theme.margins();
     const auto rowNumberSize = m_sharedData->theme.rowNumberSize();
-    const int margin = 2 * margins.width() + rowNumberSize.width();
+    const int margin = 2 * margins.width() + rowNumberSize.width() + m_view->spacing();
     m_maxSize.setWidth(size.width() - margin);
     m_idealWidth = idealWidth - margin;
 
-    for(auto w : m_cache) {
-        if (w != nullptr)
-            w->updateSize(m_maxSize, m_idealWidth);
+    if (m_idealWidth > 0) {
+        for (auto &w : m_cache) {
+            if (w != nullptr)
+                w->updateSize(m_maxSize, m_idealWidth);
+        }
     }
 }
 
 bool ItemDelegate::otherItemLoader(const QModelIndex &index, bool next)
 {
-    ItemWidget *w = m_cache[index.row()];
+    const int row = index.row();
+    ItemWidget *w = cacheOrNull(row);
     if (w != nullptr) {
+        const auto data = m_view->itemData(index);
         const bool antialiasing = m_sharedData->theme.isAntialiasingEnabled();
-        auto w2 = m_sharedData->itemFactory->otherItemLoader(index, w, next, antialiasing);
+        auto w2 = m_sharedData->itemFactory->otherItemLoader(data, w, next, antialiasing);
         if (w2 != nullptr) {
             setIndexWidget(index, w2);
             return true;
@@ -200,14 +203,15 @@ bool ItemDelegate::otherItemLoader(const QModelIndex &index, bool next)
     return false;
 }
 
-ItemEditorWidget *ItemDelegate::createCustomEditor(QWidget *parent, const QModelIndex &index,
-                                                   bool editNotes)
+ItemEditorWidget *ItemDelegate::createCustomEditor(
+        QWidget *parent, const QModelIndex &index, bool editNotes)
 {
     cache(index);
-    auto editor = new ItemEditorWidget(m_cache[index.row()], index, editNotes, parent);
+    const int row = index.row();
+    auto editor = new ItemEditorWidget(m_cache[row], index, editNotes, parent);
     editor->setEditorPalette( m_sharedData->theme.editorPalette() );
     editor->setEditorFont( m_sharedData->theme.editorFont() );
-    editor->setSaveOnReturnKey(m_saveOnReturnKey);
+    editor->setSaveOnReturnKey(m_sharedData->saveOnReturnKey);
     return editor;
 }
 
@@ -233,18 +237,21 @@ void ItemDelegate::setItemWidgetCurrent(const QModelIndex &index, bool isCurrent
         for ( auto childWidget : ww->findChildren<QWidget*>() )
             childWidget->setPalette(palette);
     } else {
-        w = m_cache[index.row()];
+        const int row = index.row();
+        w = cacheOrNull(row);
         if (!w)
             return;
     }
 
     w->setCurrent(isCurrent);
-    w->updateSize(m_maxSize, m_idealWidth);
+    if (m_idealWidth > 0)
+        w->updateSize(m_maxSize, m_idealWidth);
 }
 
 void ItemDelegate::setItemWidgetSelected(const QModelIndex &index, bool isSelected)
 {
-    auto w = m_cache[index.row()];
+    const int row = index.row();
+    auto w = cacheOrNull(row);
     if (!w)
         return;
 
@@ -254,7 +261,10 @@ void ItemDelegate::setItemWidgetSelected(const QModelIndex &index, bool isSelect
 
 void ItemDelegate::setIndexWidget(const QModelIndex &index, ItemWidget *w)
 {
-    reset(&m_cache[index.row()], w);
+    const QSize oldSize = sizeHint(index);
+
+    const int row = index.row();
+    m_cache[row].reset(w);
     if (w == nullptr)
         return;
 
@@ -262,7 +272,6 @@ void ItemDelegate::setIndexWidget(const QModelIndex &index, ItemWidget *w)
 
     // Make background transparent.
     ww->setAttribute(Qt::WA_NoSystemBackground);
-    ww->setAttribute(Qt::WA_TranslucentBackground);
 
     const bool isCurrent = m_view->currentIndex() == index;
     setItemWidgetCurrent(index, isCurrent);
@@ -270,15 +279,10 @@ void ItemDelegate::setIndexWidget(const QModelIndex &index, ItemWidget *w)
     const bool isSelected = m_view->selectionModel()->isSelected(index);
     setWidgetSelected(ww, isSelected);
 
-    // Try to get proper size by showing item momentarily.
-    ww->show();
-    w->updateSize(m_maxSize, m_idealWidth);
-    ww->hide();
-
     ww->installEventFilter(this);
 
-    // TODO: Check if sizeHint() really changes.
-    emit sizeHintChanged(index);
+    if ( oldSize != sizeHint(index) )
+        emit sizeHintChanged(index);
 }
 
 void ItemDelegate::setWidgetSelected(QWidget *ww, bool selected)
@@ -295,15 +299,46 @@ void ItemDelegate::setWidgetSelected(QWidget *ww, bool selected)
     ww->update();
 }
 
+int ItemDelegate::findWidgetRow(const QObject *obj) const
+{
+    for (int row = 0; static_cast<size_t>(row) < m_cache.size(); ++row) {
+        auto w = cacheOrNull(row);
+        if (w && w->widget() == obj)
+            return row;
+    }
+
+    return -1;
+}
+
+ItemWidget *ItemDelegate::updateCache(const QModelIndex &index, const QVariantMap &data)
+{
+    const bool antialiasing = m_sharedData->theme.isAntialiasingEnabled();
+    QWidget *parent = m_view->viewport();
+
+    auto w = m_sharedData->showSimpleItems
+            ? m_sharedData->itemFactory->createSimpleItem(data, parent, antialiasing)
+            : m_sharedData->itemFactory->createItem(data, parent, antialiasing);
+
+    setIndexWidget(index, w);
+    return w;
+}
+
 void ItemDelegate::invalidateCache()
 {
     for(auto &w : m_cache)
-        reset(&w);
+        w.reset();
 }
 
-void ItemDelegate::invalidateCache(int row)
+bool ItemDelegate::invalidateHidden(QWidget *widget)
 {
-    reset(&m_cache[row]);
+    if ( widget->isVisible() && m_view->isVisible() )
+        return false;
+
+    const auto row = findWidgetRow(widget);
+    Q_ASSERT(row != -1);
+    const auto index = m_view->index(row);
+    setIndexWidget(index, nullptr);
+    return true;
 }
 
 void ItemDelegate::setSearch(const QRegExp &re)
@@ -323,9 +358,9 @@ void ItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
     style->drawControl(QStyle::CE_ItemViewItem, &option, painter, m_view);
 
     const int row = index.row();
-    const auto w = m_cache[row];
+    const auto w = cacheOrNull(row);
     if (w == nullptr) {
-        m_view->itemWidget(index);
+        m_view->updateItemWidget(index);
         return;
     }
 
@@ -334,10 +369,10 @@ void ItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
     if (!colorExpr.isEmpty()) {
         const QColor color = m_sharedData->theme.evalColorExpression(colorExpr);
         if (color.isValid()) {
-            painter->save();
+            const auto oldMode = painter->compositionMode();
             painter->setCompositionMode(QPainter::CompositionMode_Multiply);
             painter->fillRect(option.rect, color);
-            painter->restore();
+            painter->setCompositionMode(oldMode);
         }
     }
 
@@ -350,7 +385,8 @@ void ItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
         painter->save();
         painter->setFont( m_sharedData->theme.rowNumberFont() );
         const auto rowNumberPalette = m_sharedData->theme.rowNumberPalette();
-        style->drawItemText(painter, rect.translated(margins.width() / 2, margins.height()), 0,
+        style->drawItemText(painter, rect.translated(margins.width(), margins.height()),
+                            Qt::AlignTop | Qt::AlignLeft,
                             rowNumberPalette, true, num,
                             role);
         painter->restore();
@@ -364,6 +400,10 @@ void ItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
     ww->move(offset);
     if ( ww->isHidden() ) {
         ww->show();
-        w->updateSize(m_maxSize, m_idealWidth);
+        if (m_idealWidth > 0)
+            w->updateSize(m_maxSize, m_idealWidth);
+
+        // Workaround: This fixes badly rendered items.
+        ww->update();
     }
 }
