@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -21,6 +21,7 @@
 
 #include "common/command.h"
 #include "common/common.h"
+#include "common/config.h"
 #include "common/contenttype.h"
 #include "common/log.h"
 #include "common/mimetypes.h"
@@ -36,6 +37,7 @@
 #include <QLabel>
 #include <QMetaObject>
 #include <QModelIndex>
+#include <QSettings>
 #include <QPluginLoader>
 
 #include <algorithm>
@@ -106,13 +108,12 @@ private:
 
 class DummyItem : public QLabel, public ItemWidget {
 public:
-    DummyItem(const QModelIndex &index, QWidget *parent, bool preview)
+    DummyItem(const QVariantMap &data, QWidget *parent, bool preview)
         : QLabel(parent)
         , ItemWidget(this)
-        , m_hasText(false)
-        , m_data(index.data(contentType::data).toMap())
+        , m_hasText( data.contains(mimeText) || data.contains(mimeUriList) )
+        , m_data(data)
     {
-        m_hasText = index.data(contentType::hasText).toBool();
         setMargin(0);
         setWordWrap(true);
         setTextFormat(Qt::PlainText);
@@ -122,7 +123,7 @@ public:
         if (!preview)
             setFixedHeight(sizeHint().height());
 
-        if ( !index.data(contentType::isHidden).toBool() ) {
+        if ( !data.value(mimeHidden).toBool() ) {
             const int height = preview ? -1 : contentsRect().height();
             trySetPixmap(this, m_data, height);
         }
@@ -181,9 +182,9 @@ public:
     QString author() const override { return QString(); }
     QString description() const override { return QString(); }
 
-    ItemWidget *create(const QModelIndex &index, QWidget *parent, bool preview) const override
+    ItemWidget *create(const QVariantMap &data, QWidget *parent, bool preview) const override
     {
-        return new DummyItem(index, parent, preview);
+        return new DummyItem(data, parent, preview);
     }
 
     bool canLoadItems(QIODevice *) const override { return true; }
@@ -272,10 +273,6 @@ ItemFactory::ItemFactory(QObject *parent)
     , m_disabledLoaders()
     , m_loaderChildren()
 {
-    loadPlugins();
-
-    if ( m_loaders.isEmpty() )
-        log( QObject::tr("No plugins loaded"), LogNote );
 }
 
 ItemFactory::~ItemFactory()
@@ -283,16 +280,17 @@ ItemFactory::~ItemFactory()
     // Plugins are unloaded at application exit.
 }
 
-ItemWidget *ItemFactory::createItem(const ItemLoaderPtr &loader, const QModelIndex &index,
+ItemWidget *ItemFactory::createItem(
+        const ItemLoaderPtr &loader, const QVariantMap &data,
         QWidget *parent, bool antialiasing, bool transform, bool preview)
 {
-    ItemWidget *item = loader->create(index, parent, preview);
+    ItemWidget *item = loader->create(data, parent, preview);
 
     if (item != nullptr) {
         if (transform)
-            item = transformItem(item, index);
+            item = transformItem(item, data);
         QWidget *w = item->widget();
-        QString notes = index.data(contentType::notes).toString();
+        const auto notes = getTextData(data, mimeItemNotes);
         if (!notes.isEmpty())
             w->setToolTip(notes);
 
@@ -313,10 +311,10 @@ ItemWidget *ItemFactory::createItem(const ItemLoaderPtr &loader, const QModelInd
 }
 
 ItemWidget *ItemFactory::createItem(
-        const QModelIndex &index, QWidget *parent, bool antialiasing, bool transform, bool preview)
+        const QVariantMap &data, QWidget *parent, bool antialiasing, bool transform, bool preview)
 {
     for ( auto &loader : enabledLoaders() ) {
-        ItemWidget *item = createItem(loader, index, parent, antialiasing, transform, preview);
+        ItemWidget *item = createItem(loader, data, parent, antialiasing, transform, preview);
         if (item != nullptr)
             return item;
     }
@@ -325,9 +323,9 @@ ItemWidget *ItemFactory::createItem(
 }
 
 ItemWidget *ItemFactory::createSimpleItem(
-        const QModelIndex &index, QWidget *parent, bool antialiasing)
+        const QVariantMap &data, QWidget *parent, bool antialiasing)
 {
-    return createItem(m_dummyLoader, index, parent, antialiasing);
+    return createItem(m_dummyLoader, data, parent, antialiasing);
 }
 
 QStringList ItemFactory::formatsToSave() const
@@ -388,6 +386,11 @@ ItemSaverPtr ItemFactory::loadItems(const QString &tabName, QAbstractItemModel *
         }
     }
 
+    const auto errorString =
+            QObject::tr("Tab %1 is corrupted or some CopyQ plugins are missing!")
+            .arg( quoteString(tabName) );
+    emitError(errorString);
+
     return nullptr;
 }
 
@@ -423,14 +426,14 @@ bool ItemFactory::matches(const QModelIndex &index, const QRegExp &re) const
     return false;
 }
 
-QList<ItemScriptable*> ItemFactory::scriptableObjects(QObject *parent) const
+QList<ItemScriptable*> ItemFactory::scriptableObjects() const
 {
     QList<ItemScriptable*> scriptables;
 
     for ( const auto &loader : enabledLoaders() ) {
-        auto scriptable = loader->scriptableObject(parent);
+        auto scriptable = loader->scriptableObject();
         if (scriptable) {
-            scriptable->setObjectName( loader->id() );
+            scriptable->setObjectName(loader->id());
             scriptables.append(scriptable);
         }
     }
@@ -438,9 +441,9 @@ QList<ItemScriptable*> ItemFactory::scriptableObjects(QObject *parent) const
     return scriptables;
 }
 
-QList<Command> ItemFactory::commands() const
+QVector<Command> ItemFactory::commands() const
 {
-    QList<Command> commands;
+    QVector<Command> commands;
 
     for ( const auto &loader : enabledLoaders() )
         commands << loader->commands();
@@ -460,7 +463,7 @@ void ItemFactory::loaderChildDestroyed(QObject *obj)
 }
 
 ItemWidget *ItemFactory::otherItemLoader(
-        const QModelIndex &index, ItemWidget *current, bool next, bool antialiasing)
+        const QVariantMap &data, ItemWidget *current, bool next, bool antialiasing)
 {
     Q_ASSERT(current->widget() != nullptr);
 
@@ -483,7 +486,7 @@ ItemWidget *ItemFactory::otherItemLoader(
         else if (i < 0)
             i = size - 1;
 
-        ItemWidget *item = createItem(loaders[i], index, w->parentWidget(), antialiasing);
+        ItemWidget *item = createItem(loaders[i], data, w->parentWidget(), antialiasing);
         if (item != nullptr)
             return item;
     }
@@ -508,7 +511,7 @@ bool ItemFactory::loadPlugins()
             const QString path = pluginsDir.absoluteFilePath(fileName);
             QPluginLoader pluginLoader(path);
             QObject *plugin = pluginLoader.instance();
-            log( QObject::tr("Loading plugin: %1").arg(path), LogNote );
+            COPYQ_LOG_VERBOSE( QString("Loading plugin: %1").arg(path) );
             if (plugin == nullptr) {
                 log( pluginLoader.errorString(), LogError );
             } else {
@@ -526,6 +529,30 @@ bool ItemFactory::loadPlugins()
     return true;
 }
 
+void ItemFactory::loadItemFactorySettings(QSettings *settings)
+{
+    // load settings for each plugin
+    settings->beginGroup("Plugins");
+    for ( auto &loader : loaders() ) {
+        settings->beginGroup(loader->id());
+
+        QVariantMap s;
+        for (const auto &name : settings->allKeys()) {
+            s[name] = settings->value(name);
+        }
+        loader->loadSettings(s);
+        setLoaderEnabled( loader, settings->value("enabled", true).toBool() );
+
+        settings->endGroup();
+    }
+    settings->endGroup();
+
+    // load plugin priority
+    const QStringList pluginPriority =
+            settings->value("plugin_priority", QStringList()).toStringList();
+    setPluginPriority(pluginPriority);
+}
+
 ItemLoaderList ItemFactory::enabledLoaders() const
 {
     ItemLoaderList enabledLoaders;
@@ -540,11 +567,11 @@ ItemLoaderList ItemFactory::enabledLoaders() const
     return enabledLoaders;
 }
 
-ItemWidget *ItemFactory::transformItem(ItemWidget *item, const QModelIndex &index)
+ItemWidget *ItemFactory::transformItem(ItemWidget *item, const QVariantMap &data)
 {
     for (auto &loader : m_loaders) {
         if ( isLoaderEnabled(loader) ) {
-            ItemWidget *newItem = loader->transform(item, index);
+            ItemWidget *newItem = loader->transform(item, data);
             if (newItem != nullptr)
                 item = newItem;
         }
@@ -563,7 +590,7 @@ void ItemFactory::addLoader(const ItemLoaderPtr &loader)
         if ( loaderMetaObject->indexOfSignal("error(QString)") != -1 )
             connect( signaler, SIGNAL(error(QString)), this, SIGNAL(error(QString)) );
 
-        if ( loaderMetaObject->indexOfSignal("addCommands(QList<Command>)") != -1 )
-            connect( signaler, SIGNAL(addCommands(QList<Command>)), this, SIGNAL(addCommands(QList<Command>)) );
+        if ( loaderMetaObject->indexOfSignal("addCommands(QVector<Command>)") != -1 )
+            connect( signaler, SIGNAL(addCommands(QVector<Command>)), this, SIGNAL(addCommands(QVector<Command>)) );
     }
 }
