@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -31,9 +31,11 @@
 
 class ItemLoaderInterface;
 class ItemWidget;
+class ScriptableProxy;
 class QAbstractItemModel;
 class QIODevice;
 class QModelIndex;
+class QSettings;
 class QWidget;
 struct Command;
 struct CommandMenu;
@@ -60,24 +62,21 @@ public:
      * Instantiate ItemWidget using given @a loader if possible.
      */
     ItemWidget *createItem(
-            const ItemLoaderPtr &loader, const QModelIndex &index, QWidget *parent,
+            const ItemLoaderPtr &loader, const QVariantMap &data, QWidget *parent,
             bool antialiasing, bool transform = true, bool preview = false);
 
     /**
      * Instantiate ItemWidget using appropriate loader or creates simple ItemWidget (DummyItem).
      */
-    ItemWidget *createItem(
-            const QModelIndex &index, QWidget *parent, bool antialiasing,
+    ItemWidget *createItem(const QVariantMap &data, QWidget *parent, bool antialiasing,
             bool transform = true, bool preview = false);
 
-    ItemWidget *createSimpleItem(
-            const QModelIndex &index, QWidget *parent, bool antialiasing);
+    ItemWidget *createSimpleItem(const QVariantMap &data, QWidget *parent, bool antialiasing);
 
     /**
      * Uses next/previous item loader to instantiate ItemWidget.
      */
-    ItemWidget *otherItemLoader(
-            const QModelIndex &index, ItemWidget *current, bool next, bool antialiasing);
+    ItemWidget *otherItemLoader(const QVariantMap &data, ItemWidget *current, bool next, bool antialiasing);
 
     /**
      * Formats to save in history, union of enabled ItemLoaderInterface objects.
@@ -131,31 +130,33 @@ public:
      */
     bool matches(const QModelIndex &index, const QRegExp &re) const;
 
-    QList<ItemScriptable *> scriptableObjects(QObject *parent) const;
+    QList<ItemScriptable*> scriptableObjects() const;
 
     /**
      * Adds commands from scripts for command dialog.
      */
-    QList<Command> commands() const;
+    QVector<Command> commands() const;
 
     void emitError(const QString &errorString);
 
+    bool loadPlugins();
+
+    void loadItemFactorySettings(QSettings *settings);
+
 signals:
     void error(const QString &errorString);
-    void addCommands(const QList<Command> &commands);
+    void addCommands(const QVector<Command> &commands);
 
 private slots:
     /** Called if child ItemWidget destroyed. **/
     void loaderChildDestroyed(QObject *obj);
 
 private:
-    bool loadPlugins();
-
     /** Return enabled plugins with dummy item loader. */
     ItemLoaderList enabledLoaders() const;
 
     /** Calls ItemLoaderInterface::transform() for all plugins in reverse order. */
-    ItemWidget *transformItem(ItemWidget *item, const QModelIndex &index);
+    ItemWidget *transformItem(ItemWidget *item, const QVariantMap &data);
 
     void addLoader(const ItemLoaderPtr &loader);
 
