@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -25,7 +25,6 @@
 
 #include <QAbstractItemModel>
 #include <QApplication>
-#include <QClipboard>
 #include <QDesktopServices>
 #include <QEvent>
 #include <QFont>
@@ -320,7 +319,7 @@ QVariantMap ItemSaverInterface::copyItem(const QAbstractItemModel &, const QVari
     return itemData;
 }
 
-ItemWidget *ItemLoaderInterface::create(const QModelIndex &, QWidget *, bool) const
+ItemWidget *ItemLoaderInterface::create(const QVariantMap &, QWidget *, bool) const
 {
     return nullptr;
 }
@@ -345,7 +344,7 @@ ItemSaverPtr ItemLoaderInterface::initializeTab(const QString &, QAbstractItemMo
     return nullptr;
 }
 
-ItemWidget *ItemLoaderInterface::transform(ItemWidget *, const QModelIndex &)
+ItemWidget *ItemLoaderInterface::transform(ItemWidget *, const QVariantMap &)
 {
     return nullptr;
 }
@@ -370,12 +369,12 @@ const QObject *ItemLoaderInterface::signaler() const
     return nullptr;
 }
 
-ItemScriptable *ItemLoaderInterface::scriptableObject(QObject *)
+ItemScriptable *ItemLoaderInterface::scriptableObject()
 {
     return nullptr;
 }
 
-QList<Command> ItemLoaderInterface::commands() const
+QVector<Command> ItemLoaderInterface::commands() const
 {
-    return QList<Command>();
+    return QVector<Command>();
 }
