@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -26,6 +26,7 @@
 #include <QStringList>
 #include <QtPlugin>
 #include <QVariantMap>
+#include <QVector>
 
 #include <memory>
 
@@ -45,6 +46,9 @@ using ItemLoaderPtr = std::shared_ptr<ItemLoaderInterface>;
 class ItemSaverInterface;
 using ItemSaverPtr = std::shared_ptr<ItemSaverInterface>;
 
+class ItemScriptableFactoryInterface;
+using ItemScriptableFactoryPtr = std::shared_ptr<ItemScriptableFactoryInterface>;
+
 #define COPYQ_PLUGIN_ITEM_LOADER_ID "org.CopyQ.ItemPlugin.ItemLoader/1.0"
 
 #if QT_VERSION < 0x050000
@@ -178,8 +182,6 @@ class ItemScriptable : public QObject
 {
     Q_OBJECT
 public:
-    explicit ItemScriptable(QObject *parent) : QObject(parent) {}
-
     /** Return scriptable object (nullptr before start() is called). */
     QObject *scriptable() const { return m_scriptable; }
 
@@ -215,6 +217,34 @@ private:
     QObject *m_scriptable = nullptr;
 };
 
+/**
+ * Factory for creating script objects.
+ */
+class ItemScriptableFactoryInterface
+{
+public:
+    virtual ~ItemScriptableFactoryInterface() = default;
+
+    /**
+     * Create script object.
+     *
+     * This method must be thread-safe.
+     */
+    virtual ItemScriptable *create() const = 0;
+
+    /**
+     * Variable name for scriptable object in script.
+     *
+     * This method is thread-safe.
+     */
+    QString name() const { return m_name; }
+
+    void setName(const QString &name) { m_name = name; }
+
+private:
+    QString m_name;
+};
+
 /**
  * Handles saving and modifying item models.
  */
@@ -287,7 +317,7 @@ public:
      *
      * @return nullptr if index hasn't appropriate data
      */
-    virtual ItemWidget *create(const QModelIndex &index, QWidget *parent, bool preview) const;
+    virtual ItemWidget *create(const QVariantMap &data, QWidget *parent, bool preview) const;
 
     /**
      * Simple ID of plugin.
@@ -372,7 +402,7 @@ public:
      * By default returns nullptr not to wrap the widget.
      * New ItemWidget must take care of deleting the old one!
      */
-    virtual ItemWidget *transform(ItemWidget *itemWidget, const QModelIndex &index);
+    virtual ItemWidget *transform(ItemWidget *itemWidget, const QVariantMap &data);
 
     /**
      * Transform loader.
@@ -398,22 +428,19 @@ public:
     /**
      * Return QObject instance with signals (by default null pointer).
      *
-     * Returned QObject can have signal error(QString) for signaling errors
-     * and addCommands(QList<Command>) to open and add commands to Command dialog.
+     * Returned QObject can have signal error(QString) for signaling errors.
      */
     virtual const QObject *signaler() const;
 
     /**
-     * Return scriptable object that can be used in scripts.
-     *
-     * Object will be available as `plugins.<PLUGIN_ID>`.
+     * Return scriptable object.
      */
-    virtual ItemScriptable *scriptableObject(QObject *parent);
+    virtual ItemScriptable *scriptableObject();
 
     /**
      * Adds commands from scripts for command dialog.
      */
-    virtual QList<Command> commands() const;
+    virtual QVector<Command> commands() const;
 
     ItemLoaderInterface(const ItemLoaderInterface &) = delete;
     ItemLoaderInterface &operator=(const ItemLoaderInterface &) = delete;
