@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,7 +20,6 @@
 #include "app/app.h"
 #include "app/applicationexceptionhandler.h"
 #include "app/clipboardclient.h"
-#include "app/clipboardmonitor.h"
 #include "app/clipboardserver.h"
 #include "common/commandstatus.h"
 #include "common/log.h"
@@ -110,6 +109,13 @@ QString restoreSessionName(const QString &sessionId)
 
 int startServer(int argc, char *argv[], QString sessionName)
 {
+#if QT_VERSION >= 0x050000
+    // By default, enable automatic screen scaling in Qt for high-DPI displays
+    // (this works better at least in Windows).
+    if ( qgetenv("QT_AUTO_SCREEN_SCALE_FACTOR").isEmpty() )
+        qputenv("QT_AUTO_SCREEN_SCALE_FACTOR", "1");
+#endif
+
     auto qapp = createPlatformNativeInterface()->createServerApplication(argc, argv);
     if ( qapp->isSessionRestored() ) {
         const auto sessionId = qapp->sessionId();
@@ -127,15 +133,9 @@ int startServer(int argc, char *argv[], QString sessionName)
     return app.exec();
 }
 
-int startMonitor(int argc, char *argv[], const QString &serverName, const QString &sessionName)
+int startClient(int argc, char *argv[], const QStringList &arguments, const QString &sessionName)
 {
-    ClipboardMonitor app(argc, argv, serverName, sessionName);
-    return app.exec();
-}
-
-int startClient(int argc, char *argv[], int skipArguments, const QString &sessionName)
-{
-    ClipboardClient app(argc, argv, skipArguments, sessionName);
+    ClipboardClient app(argc, argv, arguments, sessionName);
     return app.exec();
 }
 
@@ -232,14 +232,9 @@ int startApplication(int argc, char **argv)
     if ( arguments.size() - skipArguments == 0 )
         return startServer(argc, argv, sessionName);
 
-    // If first argument is "monitor" (second is monitor server name/ID)
-    // then run this process as clipboard monitor.
-    if ( arguments.size() == 4 && arguments[2] == "monitor" )
-        return startMonitor(argc, argv, arguments[3], sessionName);
-
     // If argument was specified and server is running
     // then run this process as client.
-    return startClient(argc, argv, skipArguments, sessionName);
+    return startClient(argc, argv, arguments.mid(skipArguments), sessionName);
 }
 
 } // namespace
