@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -22,8 +22,6 @@
 
 #include "platform/dummy/dummyclipboard.h"
 
-#include <QClipboard>
-
 class MacTimer;
 
 class MacClipboard : public DummyClipboard {
@@ -31,12 +29,12 @@ class MacClipboard : public DummyClipboard {
 public:
     explicit MacClipboard();
 
-    QVariantMap data(Mode mode, const QStringList &formats) const override;
+    QVariantMap data(ClipboardMode mode, const QStringList &formats) const override;
 
-    void setData(Mode mode, const QVariantMap &dataMap) override;
+    void setData(ClipboardMode mode, const QVariantMap &dataMap) override;
 
 signals:
-    void changed(PlatformClipboard::Mode mode);
+    void changed(ClipboardMode mode);
 
 private:
     long int m_prevChangeCount;
