@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,43 +20,34 @@
 #ifndef PLATFORMCLIPBOARD_H
 #define PLATFORMCLIPBOARD_H
 
+#include "common/clipboardmode.h"
+
 #include <QObject>
 #include <QVariantMap>
 
 /**
  * Interface for clipboard.
  *
- * Define signal changed(PlatformClipboard::Mode) in derived class.
+ * Define signal changed(ClipboardMode) in derived class.
  * This signal notifies about clipboard changes.
  */
 class PlatformClipboard : public QObject
 {
 public:
-    enum Mode {
-        Clipboard,
-        Selection,
-        FindBuffer
-    };
-
     /**
-     * Load settings from GUI.
-     *
-     * Settings keys are:
-     *   - "check_selection" (bool) - emit changed() when X11 selection changes
-     *   - "formats" (QStringList)  - string list with MIME formats to store
-     *     (formats are also passed to data() method call from ClipboardMonitor)
+     * Set MIME types to monitor.
      */
-    virtual void loadSettings(const QVariantMap &settings) = 0;
+    virtual void setFormats(const QStringList &formats) = 0;
 
     /**
      * Return clipboard data containing specified @a formats if available.
      */
-    virtual QVariantMap data(Mode mode, const QStringList &formats) const = 0;
+    virtual QVariantMap data(ClipboardMode mode, const QStringList &formats) const = 0;
 
     /**
      * Set data to clipboard.
      */
-    virtual void setData(Mode mode, const QVariantMap &dataMap) = 0;
+    virtual void setData(ClipboardMode mode, const QVariantMap &dataMap) = 0;
 };
 
 #endif // PLATFORMCLIPBOARD_H
