@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -22,7 +22,6 @@
 
 #include "platform/dummy/dummyclipboard.h"
 
-#include <QClipboard>
 #include <QStringList>
 #include <QTimer>
 
@@ -36,15 +35,16 @@ class X11PlatformClipboard : public DummyClipboard
 public:
     explicit X11PlatformClipboard(const std::shared_ptr<X11DisplayGuard> &d);
 
-    void loadSettings(const QVariantMap &settings) override;
+    void setFormats(const QStringList &formats) override;
 
-    QVariantMap data(Mode mode, const QStringList &formats) const override;
+    QVariantMap data(ClipboardMode mode, const QStringList &formats) const override;
 
-    void setData(Mode mode, const QVariantMap &dataMap) override;
+    void setData(ClipboardMode mode, const QVariantMap &dataMap) override;
+
+protected:
+    void onChanged(int mode) override;
 
 private slots:
-    void onChanged(QClipboard::Mode mode) override;
-
     void onClipboardChanged();
     void onSelectionChanged();
 
