@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -23,8 +23,9 @@
 
 COPYQ_DECLARE_SCRIPTABLE_CLASS(DirClass)
 
-DirClass::DirClass(QScriptEngine *engine)
-    : ScriptableClass(engine)
+DirClass::DirClass(QScriptEngine *engine, const QScriptValue &objectPrototype)
+    : ScriptableClass(engine, objectPrototype)
+    , m_currentPath(QDir::currentPath())
 {
 }
 
