@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -85,7 +85,7 @@ public slots:
     QScriptValue root() const;
     QScriptValue rootPath() const;
     QStringList searchPaths(const QScriptValue &prefix) const;
-    QChar separator() const;
+    QScriptValue separator() const;
     bool setCurrent(const QScriptValue &path) const;
     void setSearchPaths(const QScriptValue &prefix, const QStringList &searchPaths) const;
     QScriptValue temp() const;
