@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -23,8 +23,8 @@
 
 COPYQ_DECLARE_SCRIPTABLE_CLASS(FileClass)
 
-FileClass::FileClass(QScriptEngine *engine)
-    : ScriptableClass(engine)
+FileClass::FileClass(QScriptEngine *engine, const QScriptValue &objectPrototype)
+    : ScriptableClass(engine, objectPrototype)
 {
 }
 
