@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,19 +20,23 @@
 #ifndef SCRIPTABLE_H
 #define SCRIPTABLE_H
 
+#include "common/clipboardmode.h"
+#include "common/command.h"
 #include "common/mimetypes.h"
 
-#include <QClipboard>
 #include <QObject>
 #include <QString>
 #include <QScriptable>
 #include <QScriptValue>
 #include <QVariantMap>
+#include <QVector>
 
+class Action;
 class ByteArrayClass;
 class ClipboardBrowser;
 class DirClass;
 class FileClass;
+class ItemFactory;
 class ScriptableProxy;
 class TemporaryFileClass;
 
@@ -64,6 +68,8 @@ class Scriptable : public QObject, protected QScriptable
     Q_PROPERTY(QScriptValue mimeSyncToClipboard READ getMimeSyncToClipboard)
     Q_PROPERTY(QScriptValue mimeSyncToSelection READ getMimeSyncToSelection)
 
+    Q_PROPERTY(QScriptValue plugins READ getPlugins)
+
 public:
     explicit Scriptable(
             QScriptEngine *engine,
@@ -82,12 +88,13 @@ public:
     QVariant toVariant(const QScriptValue &value);
     bool toInt(const QScriptValue &value, int *number) const;
     QVariantMap toDataMap(const QScriptValue &value) const;
+    QScriptValue fromDataMap(const QVariantMap &dataMap) const;
 
     QByteArray makeByteArray(const QScriptValue &value) const;
 
     /**
      * Set data for item converted from @a value.
-     * Return true if data was successfuly converted and set.
+     * Return true if data was successfully converted and set.
      *
      * If mime starts with "text/" or isn't byte array the value is re-encoded
      * from local encoding to UTF8.
@@ -132,11 +139,21 @@ public:
     QScriptValue getMimeSyncToClipboard() const { return mimeSyncToClipboard; }
     QScriptValue getMimeSyncToSelection() const { return mimeSyncToSelection; }
 
+    QScriptValue getPlugins();
+
     ByteArrayClass *byteArrayClass() const { return m_baClass; }
     FileClass *fileClass() const { return m_fileClass; }
     TemporaryFileClass *temporaryFileClass() const { return m_temporaryFileClass; }
 
+    QScriptValue call(QScriptValue *fn, const QScriptValue &object, const QVariantList &arguments) const;
+
+    QScriptValue eval(const QString &script, const QString &fileName);
+
+    void executeArguments(const QStringList &args);
+
 public slots:
+    void setInput(const QByteArray &input);
+
     QScriptValue version();
     QScriptValue help();
 
@@ -230,7 +247,7 @@ public slots:
     void keys();
     QScriptValue testSelected();
     void resetTestSession();
-    void flush();
+    void serverLog();
 
     void setCurrentTab();
 
@@ -277,9 +294,6 @@ public slots:
 
     QScriptValue dateString();
 
-    void updateFirst();
-    void updateTitle();
-
     QScriptValue commands();
     void setCommands();
     void addCommands();
@@ -293,6 +307,7 @@ public slots:
     QScriptValue setEnv();
 
     void sleep();
+    void afterMilliseconds();
 
     // Call scriptable method.
     QVariant call(const QString &method, const QVariantList &arguments);
@@ -307,30 +322,71 @@ public slots:
 
     QScriptValue iconColor();
 
+    QScriptValue iconTag();
+
+    QScriptValue iconTagColor();
+
+    void onClipboardChanged();
+    void onOwnClipboardChanged();
+    void onHiddenClipboardChanged();
+
+    void setClipboardData();
+    void updateTitle();
+    void setTitle();
+    void synchronizeSelection();
+    void saveData();
+    void showDataNotification();
+    void hideDataNotification();
+    void updateClipboardData();
+    void clearClipboardData();
+    QScriptValue runAutomaticCommands();
+
+    void runDisplayCommands();
+
+    void runMenuCommandFilters();
+
+    void monitorClipboard();
+    void provideClipboard();
+    void provideSelection();
+
 public slots:
-    void onMessageReceived(const QByteArray &bytes, int messageCode);
     void onDisconnected();
 
 signals:
     void sendMessage(const QByteArray &message, int messageCode);
+    void dataReceived();
+    void finished();
+    void readInput();
 
 private slots:
-    void onExecuteOutput(const QStringList &lines);
+    void onExecuteOutput(const QByteArray &output);
+    void onMonitorRunScriptRequest(const QString &script, const QVariantMap &data);
+    void onProvidedClipboardChanged();
+    void onProvidedSelectionChanged();
 
 private:
-    void executeArguments(const QByteArray &bytes);
+    bool sourceScriptCommands();
+    void callDisplayFunctions(QScriptValueList displayFunctions);
     QString processUncaughtException(const QString &cmd);
     void showExceptionMessage(const QString &message);
-    QList<int> getRows() const;
-    QScriptValue copy(QClipboard::Mode mode);
-    bool setClipboard(QVariantMap *data, QClipboard::Mode mode);
+    QVector<int> getRows() const;
+    QScriptValue copy(ClipboardMode mode);
+    bool setClipboard(QVariantMap *data, ClipboardMode mode);
     void changeItem(bool create);
     void nextToClipboard(int where);
     QScriptValue screenshot(bool select);
     QByteArray serialize(const QScriptValue &value);
-    QScriptValue eval(const QString &script, const QString &fileName);
     QScriptValue eval(const QString &script);
     QTextCodec *codecFromNameOrThrow(const QScriptValue &codecName);
+    bool runAction(Action *action);
+    bool runCommands(CommandType::CommandType type);
+    bool canExecuteCommand(const Command &command);
+    bool canExecuteCommandFilter(const QString &matchCommand);
+    bool verifyClipboardAccess();
+    void provideClipboard(ClipboardMode mode);
+
+    void insert(int argumentsEnd);
+    void insert(int row, int argumentsBegin, int argumentsEnd);
 
     ScriptableProxy *m_proxy;
     QScriptEngine *m_engine;
@@ -341,11 +397,18 @@ private:
     QString m_inputSeparator;
     QScriptValue m_input;
     QVariantMap m_data;
-    QString m_actionName;
+    int m_actionId = -1;
     bool m_connected;
     int m_skipArguments = 0;
 
+    // FIXME: Parameters for execute() shouldn't be global.
+    QByteArray m_executeStdoutData;
+    QString m_executeStdoutLastLine;
     QScriptValue m_executeStdoutCallback;
+
+    bool m_displayFunctionsLock = false;
+
+    QScriptValue m_plugins;
 };
 
 class NetworkReply : public QObject {
