@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,22 +20,24 @@
 #ifndef SCRIPTABLEPROXY_H
 #define SCRIPTABLEPROXY_H
 
-#include "gui/clipboardbrowser.h"
+#include "common/clipboardmode.h"
+#include "common/command.h"
 #include "gui/notificationbutton.h"
 
-#include <QClipboard>
 #include <QList>
 #include <QMetaObject>
 #include <QObject>
+#include <QPoint>
+#include <QRect>
+#include <QVariant>
+#include <QVector>
 
 #include <memory>
 
+class ClipboardBrowser;
 class MainWindow;
 class QPersistentModelIndex;
 class QPixmap;
-class QPoint;
-
-struct Command;
 
 struct NamedValue {
     NamedValue() {}
@@ -46,18 +48,48 @@ struct NamedValue {
 
 using NamedValueList = QVector<NamedValue>;
 
+struct ScriptablePath {
+    QString path;
+};
+
+Q_DECLARE_METATYPE(NamedValueList)
+Q_DECLARE_METATYPE(ScriptablePath)
+Q_DECLARE_METATYPE(NotificationButtons)
+Q_DECLARE_METATYPE(QVector<QVariantMap>)
+Q_DECLARE_METATYPE(Qt::KeyboardModifiers)
+Q_DECLARE_METATYPE(Command)
+Q_DECLARE_METATYPE(ClipboardMode)
+
+#if QT_VERSION < 0x050000
+Q_DECLARE_METATYPE(QVector<int>)
+Q_DECLARE_METATYPE(QVector<Command>)
+#endif
+
+QDataStream &operator<<(QDataStream &out, const NotificationButton &button);
+QDataStream &operator>>(QDataStream &in, NotificationButton &button);
+QDataStream &operator<<(QDataStream &out, const NamedValueList &list);
+QDataStream &operator>>(QDataStream &in, NamedValueList &list);
+QDataStream &operator<<(QDataStream &out, const Command &command);
+QDataStream &operator>>(QDataStream &in, Command &command);
+QDataStream &operator<<(QDataStream &out, ClipboardMode mode);
+QDataStream &operator>>(QDataStream &in, ClipboardMode &mode);
+QDataStream &operator<<(QDataStream &out, const ScriptablePath &path);
+QDataStream &operator>>(QDataStream &in, ScriptablePath &path);
+
 class ScriptableProxy : public QObject
 {
     Q_OBJECT
+
 public:
     /** Create proxy object and move it to same thread as @a mainWindow. */
-    explicit ScriptableProxy(MainWindow* mainWindow);
+    ScriptableProxy(MainWindow* mainWindow, QObject *parent);
 
-    const QVariant &value() const;
+    QByteArray callFunction(const QByteArray &serializedFunctionCall);
 
-    void unsetValue();
+    int actionId() const { return m_actionId; }
 
-    bool isValueUnset();
+public slots:
+    void setReturnValue(const QByteArray &returnValue);
 
     QVariantMap getActionData(int id);
     void setActionData(int id, const QVariantMap &data);
@@ -70,13 +102,11 @@ public:
     bool pasteToCurrentWindow();
     bool copyFromCurrentWindow();
 
-    void abortAutomaticCommands();
-
     bool isMonitoringEnabled();
     bool isMainWindowVisible();
     bool isMainWindowFocused();
     void disableMonitoring(bool arg1);
-    void setClipboard(const QVariantMap &data, QClipboard::Mode mode);
+    void setClipboard(const QVariantMap &data, ClipboardMode mode);
 
     QString renameTab(const QString &arg1, const QString &arg2);
 
@@ -88,7 +118,7 @@ public:
     bool showBrowser(const QString &tabName);
     bool showBrowserAt(const QString &tabName, QRect rect);
 
-    bool showBrowser();
+    bool showCurrentBrowser();
 
     void action(const QVariantMap &arg1, const Command &arg2);
 
@@ -102,7 +132,7 @@ public:
     QVariantMap nextItem(int where);
     void browserMoveToClipboard(int arg1);
     void browserSetCurrent(int arg1);
-    QString browserRemoveRows(QList<int> rows);
+    QString browserRemoveRows(QVector<int> rows);
 
     void browserEditRow(int arg1);
     void browserEditNew(const QString &arg1, bool changeClipboard);
@@ -110,7 +140,7 @@ public:
     QStringList tabs();
     bool toggleVisible();
     bool toggleMenu(const QString &tabName, int maxItemCount, QPoint position);
-    bool toggleMenu();
+    bool toggleCurrentMenu();
     int findTabIndex(const QString &arg1);
 
     void openActionDialog(const QVariantMap &arg1);
@@ -124,14 +154,13 @@ public:
     QVariant config(const QStringList &nameValue);
     QVariant toggleConfig(const QString &optionName);
 
-    QByteArray getClipboardData(const QString &mime, QClipboard::Mode mode = QClipboard::Clipboard);
-    bool hasClipboardFormat(const QString &mime, QClipboard::Mode mode = QClipboard::Clipboard);
+    QByteArray getClipboardData(const QString &mime, ClipboardMode mode = ClipboardMode::Clipboard);
+    bool hasClipboardFormat(const QString &mime, ClipboardMode mode = ClipboardMode::Clipboard);
 
     int browserLength();
     bool browserOpenEditor(const QByteArray &arg1, bool changeClipboard);
 
-    QString browserAdd(const QStringList &texts);
-    QString browserAdd(const QVariantMap &arg1, int arg2);
+    QString browserInsert(int row, const QVector<QVariantMap> &items);
     bool browserChange(const QVariantMap &data, int row);
 
     QByteArray browserItemData(int arg1, const QString &arg2);
@@ -144,16 +173,16 @@ public:
     QString tab();
 
     int currentItem();
-    bool selectItems(const QList<int> &items);
+    bool selectItems(const QVector<int> &rows);
 
-    QList<int> selectedItems();
+    QVector<int> selectedItems();
 
     int selectedItemsDataCount();
     QVariantMap selectedItemData(int selectedIndex);
     bool setSelectedItemData(int selectedIndex, const QVariantMap &data);
 
-    QList<QVariantMap> selectedItemsData();
-    void setSelectedItemsData(const QList<QVariantMap> &dataList);
+    QVector<QVariantMap> selectedItemsData();
+    void setSelectedItemsData(const QVector<QVariantMap> &dataList);
 
 #ifdef HAS_TESTS
     void sendKeys(const QString &keys, int delay);
@@ -162,21 +191,21 @@ public:
     void resetTestSession(const QString &clipboardTabName);
 #endif // HAS_TESTS
 
+    void serverLog(const QString &text);
+
     QString currentWindowTitle();
 
     NamedValueList inputDialog(const NamedValueList &values);
 
     void setUserValue(const QString &key, const QVariant &value);
 
-    void updateFirstItem(const QVariantMap &data);
-    void updateTitle(const QVariantMap &data);
     void setSelectedItemsData(const QString &mime, const QVariant &value);
 
     void filter(const QString &text);
 
-    QList<Command> commands();
-    void setCommands(const QList<Command> &commands);
-    void addCommands(const QList<Command> &commands);
+    QVector<Command> commands();
+    void setCommands(const QVector<Command> &commands);
+    void addCommands(const QVector<Command> &commands);
 
     QByteArray screenshot(const QString &format, const QString &screenName, bool select);
 
@@ -191,8 +220,29 @@ public:
     QString iconColor();
     bool setIconColor(const QString &name);
 
+    QString iconTag();
+    void setIconTag(const QString &tag);
+
+    QString iconTagColor();
+    bool setIconTagColor(const QString &name);
+
+    void setClipboardData(const QVariantMap &data);
+    void setTitle(const QString &title);
+    void setTitleForData(const QVariantMap &data);
+    void saveData(const QString &tab, const QVariantMap &data);
+    void showDataNotification(const QVariantMap &data);
+
+    QStringList menuItemMatchCommands(int actionId);
+    bool enableMenuItem(int actionId, int menuItemMatchCommandIndex, bool enabled);
+
+    QVariantMap setDisplayData(int actionId, const QVariantMap &displayData);
+
+    QVector<Command> automaticCommands();
+    QVector<Command> displayCommands();
+    QVector<Command> scriptCommands();
+
 signals:
-    void sendMessage(const QByteArray &message, int messageCode);
+    void sendFunctionCall(const QByteArray &bytes);
 
 private:
     ClipboardBrowser *fetchBrowser(const QString &tabName);
@@ -207,9 +257,11 @@ private:
     MainWindow* m_wnd;
     QString m_tabName;
     QVariantMap m_actionData;
-    bool m_invoked;
+    int m_actionId = -1;
 
     uint m_sentKeyClicks = 0;
+
+    QVariant m_returnValue;
 };
 
 QString pluginsPath();
