@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -33,7 +33,7 @@ class TemporaryFileClass : public ScriptableClass<QTemporaryFile, TemporaryFileP
 {
     Q_OBJECT
 public:
-    explicit TemporaryFileClass(QScriptEngine *engine);
+    TemporaryFileClass(QScriptEngine *engine, const QScriptValue &objectPrototype);
 
     QScriptValue newInstance(const QString &templateName);
     QScriptValue newInstance();
