@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -85,12 +85,15 @@ do { \
     do { \
         TEST( m_test->getClientOutput((Args() << ARGUMENTS), &out_) ); \
     } while (out_ != expected_ && t_.sleep()); \
-    QCOMPARE(out_, expected_); \
+    QCOMPARE(QString(out_), QString(expected_)); \
 } while(false)
 
 /// Interval to wait (in ms) until window is shown and focused.
 const int waitMsShow = 1000;
 
+/// Interval to wait (in ms) until tray menu is updated.
+const int waitTrayUpdate = 3000;
+
 /// Interval to wait (in ms) before and after setting clipboard.
 const int waitMsSetClipboard = 1000;
 
