@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,11 +20,9 @@
 #include "tests.h"
 #include "test_utils.h"
 
-#include "app/remoteprocess.h"
 #include "common/client_server.h"
 #include "common/common.h"
 #include "common/mimetypes.h"
-#include "common/monitormessagecode.h"
 #include "common/shortcuts.h"
 #include "common/textdata.h"
 #include "common/version.h"
@@ -53,6 +51,7 @@ namespace {
 
 const auto clipboardTabName = "CLIPBOARD";
 const auto defaultSessionColor = "#ff8800";
+const auto defaultTagColor = "#000000";
 
 /**
  * Run a process after a delay.
@@ -169,7 +168,6 @@ class TestInterfaceImpl : public TestInterface {
 public:
     TestInterfaceImpl()
         : m_server(nullptr)
-        , m_monitor(nullptr)
         , m_env(QProcessEnvironment::systemEnvironment())
     {
         m_env.insert("COPYQ_LOG_LEVEL", "DEBUG");
@@ -365,22 +363,11 @@ public:
 
     QByteArray setClipboard(const QByteArray &bytes, const QString &mime) override
     {
-        if (m_monitor == nullptr) {
-            m_monitor.reset(new RemoteProcess);
-            const QString name = "copyq_TEST";
-            m_monitor->start( name, QStringList() << "-s" << "" << "monitor" << name );
-
-            SleepTimer t(4000);
-            while( !m_monitor->isConnected() && t.sleep() ) {}
-        }
-
-        if ( !m_monitor->isConnected() )
-            return "Failed to start clipboard monitor!";
-
         waitFor(waitMsSetClipboard);
 
-        const QVariantMap data = createDataMap(mime, bytes);
-        m_monitor->writeMessage( serializeData(data), MonitorChangeClipboard );
+        auto mimeData = new QMimeData();
+        mimeData->setData(mime, bytes);
+        QApplication::clipboard()->setMimeData(mimeData);
 
         waitUntilClipboardSet(bytes, mime);
         QByteArray error = testClipboard(bytes, mime);
@@ -401,13 +388,13 @@ public:
 
             // Flush server output.
             if (m_server->state() != QProcess::NotRunning) {
-                const QByteArray data = generateData();
-                if ( run(Args("flush") << data) == 0 ) {
+                const QByteArray data = "flush ID: " + generateData();
+                if ( run(Args("serverLog") << data) == 0 ) {
                     SleepTimer t(5000);
                     bool isFlushed = false;
                     do {
                         output.append(m_server->readAllStandardError());
-                        isFlushed = output.contains("ID: " + data);
+                        isFlushed = output.contains(data);
                     } while (!isFlushed && t.sleep());
 
                     if (!isFlushed)
@@ -517,9 +504,6 @@ private:
 
     QByteArray testClipboard(const QByteArray &bytes, const QString &mime)
     {
-        if ( !m_monitor || !m_monitor->isConnected() )
-            return "Clipboard monitor is not running!";
-
         const QByteArray actualBytes = getClipboard(mime);
         if (actualBytes != bytes) {
             return QString("Test failed (clipboard data for MIME \"%1\"): ")
@@ -532,7 +516,6 @@ private:
     }
 
     std::unique_ptr<QProcess> m_server;
-    std::unique_ptr<RemoteProcess> m_monitor; /// Process to provide clipboard set by tests.
     QProcessEnvironment m_env;
     QVariantMap m_settings;
 };
@@ -650,6 +633,9 @@ void Tests::commandEval()
     RUN("eval" << "[1,2,3]", "1\n2\n3\n");
     RUN("eval" << "'123'", "123\n");
     RUN("eval" << "'123'", "123\n");
+
+    RUN("-e" << "", "");
+    RUN("-e" << "1", "1\n");
 }
 
 void Tests::commandEvalThrows()
@@ -1267,6 +1253,31 @@ void Tests::commandIcon()
     RUN("iconColor", QByteArray(defaultSessionColor) + "\n");
 }
 
+void Tests::commandIconTag()
+{
+    RUN("iconTag", "\n");
+
+    RUN("iconTag" << "TEST", "");
+    RUN("iconTag", "TEST\n");
+
+    RUN("iconTag" << "", "");
+    RUN("iconTag", "\n");
+}
+
+void Tests::commandIconTagColor()
+{
+    RUN("iconTagColor", QByteArray(defaultTagColor) + "\n");
+
+    RUN("iconTagColor" << "red", "");
+    RUN("iconTagColor", "#ff0000\n");
+
+    RUN_EXPECT_ERROR("iconTagColor" << "BAD_COLOR_NAME", CommandException);
+    RUN("iconTagColor", "#ff0000\n");
+
+    RUN("iconTagColor" << defaultTagColor, "");
+    RUN("iconTagColor", QByteArray(defaultTagColor) + "\n");
+}
+
 void Tests::classFile()
 {
     RUN("eval" <<
@@ -1995,6 +2006,7 @@ void Tests::pasteFromMainWindow()
 void Tests::tray()
 {
     RUN("add" << "A", "");
+    waitFor(waitTrayUpdate);
     RUN("menu", "");
     waitFor(waitMsShow);
     RUN("keys" << "ENTER", "");
@@ -2007,12 +2019,14 @@ void Tests::menu()
 
     RUN("tab" << tab << "add" << "D" << "C" << "B" << "A", "");
 
+    waitFor(waitTrayUpdate);
     RUN("menu" << tab, "");
     waitFor(waitMsShow);
     RUN("keys" << "ENTER", "");
     WAIT_FOR_CLIPBOARD("A");
 
     // Show menu with 2 items from the tab and select last one.
+    waitFor(waitTrayUpdate);
     RUN("menu" << tab << "2", "");
     waitFor(waitMsShow);
     RUN("keys" << "END" << "ENTER", "");
@@ -2023,6 +2037,7 @@ void Tests::traySearch()
 {
     RUN("add" << "C" << "B" << "A", "");
 
+    waitFor(waitTrayUpdate);
     RUN("menu", "");
     waitFor(waitMsShow);
     RUN("keys" << "B" << "ENTER", "");
@@ -2038,6 +2053,7 @@ void Tests::trayPaste()
     RUN("keys" << "CTRL+N" << ":NEW ", "");
 
     RUN("add" << "TEST", "");
+    waitFor(waitTrayUpdate);
     RUN("menu", "");
     waitFor(waitMsShow);
     RUN("keys" << "ENTER", "");
@@ -2050,6 +2066,7 @@ void Tests::trayPaste()
 
     RUN("config" << "tray_item_paste" << "false", "false\n");
     RUN("add" << "TEST2", "");
+    waitFor(waitTrayUpdate);
     RUN("menu", "");
     waitFor(waitMsShow);
     RUN("keys" << "ENTER", "");
@@ -2071,6 +2088,7 @@ void Tests::configTrayTab()
 
     RUN("config" << "tray_tab" << tab1, tab1 + "\n");
 
+    waitFor(waitTrayUpdate);
     RUN("menu", "");
     waitFor(waitMsShow);
     RUN("keys" << "ENTER", "");
@@ -2078,6 +2096,7 @@ void Tests::configTrayTab()
 
     RUN("config" << "tray_tab" << tab2, tab2 + "\n");
 
+    waitFor(waitTrayUpdate);
     RUN("menu", "");
     waitFor(waitMsShow);
     RUN("keys" << "ENTER", "");
@@ -2090,6 +2109,7 @@ void Tests::configMove()
 
     RUN("config" << "move" << "true", "true\n");
 
+    waitFor(waitTrayUpdate);
     RUN("menu", "");
     waitFor(waitMsShow);
     RUN("keys" << "DOWN" << "ENTER", "");
@@ -2097,6 +2117,7 @@ void Tests::configMove()
 
     RUN("config" << "move" << "false", "false\n");
 
+    waitFor(waitTrayUpdate);
     RUN("menu", "");
     waitFor(waitMsShow);
     RUN("keys" << "DOWN" << "ENTER", "");
@@ -2114,12 +2135,14 @@ void Tests::configTrayTabIsCurrent()
     RUN("config" << "tray_tab_is_current" << "true", "true\n");
 
     RUN("setCurrentTab" << tab1, "");
+    waitFor(waitTrayUpdate);
     RUN("menu", "");
     waitFor(waitMsShow);
     RUN("keys" << "ENTER", "");
     WAIT_FOR_CLIPBOARD("A");
 
     RUN("setCurrentTab" << tab2, "");
+    waitFor(waitTrayUpdate);
     RUN("menu", "");
     waitFor(waitMsShow);
     RUN("keys" << "ENTER", "");
@@ -2282,9 +2305,40 @@ void Tests::shortcutCommandSetSelectedItemsData()
 
 void Tests::automaticCommandIgnore()
 {
-    RUN("setCommands([{automatic: true, cmd: 'copyq ignore; copyq add OK'}])", "");
-    TEST( m_test->setClipboard("SHOULD BE IGNORED") );
+    const auto script = R"(
+        setCommands([
+            { automatic: true, cmd: 'copyq ignore; copyq add OK' },
+            { automatic: true, cmd: 'copyq add "SHOULDN NOT BE EXECUTED"' }
+        ])
+        )";
+    RUN(script, "");
+
+    TEST( m_test->setClipboard("SHOULD BE IGNORED 1") );
     WAIT_ON_OUTPUT("read" << "0", "OK");
+
+    TEST( m_test->setClipboard("SHOULD BE IGNORED 2") );
+    WAIT_ON_OUTPUT("size", "2\n");
+
+    RUN("separator" << "," << "read" << "0" << "1" << "2", "OK,OK,");
+}
+
+void Tests::automaticCommandRemove()
+{
+    const auto script = R"(
+        setCommands([
+            { automatic: true, remove: true, cmd: 'copyq add OK' },
+            { automatic: true, cmd: 'copyq add "SHOULDN NOT BE EXECUTED"' }
+        ])
+        )";
+    RUN(script, "");
+
+    TEST( m_test->setClipboard("SHOULD BE IGNORED 1") );
+    WAIT_ON_OUTPUT("read" << "0", "OK");
+
+    TEST( m_test->setClipboard("SHOULD BE IGNORED 2") );
+    WAIT_ON_OUTPUT("size", "2\n");
+
+    RUN("separator" << "," << "read" << "0" << "1" << "2", "OK,OK,");
 }
 
 void Tests::automaticCommandInput()
@@ -2356,6 +2410,89 @@ void Tests::automaticCommandNoOutputTab()
     RUN("tab" << QString(clipboardTabName) << "size", "0\n");
 }
 
+void Tests::automaticCommandChaining()
+{
+    const auto script = R"(
+        setCommands([
+            {automatic: true, cmd: 'copyq: setData(mimeText, 1)'},
+            {automatic: true, cmd: 'copyq: setData(mimeText, str(data(mimeText)) + 2)'},
+            {automatic: true, cmd: 'copyq: setData(mimeText, str(data(mimeText)) + 3)'}
+        ])
+        )";
+    RUN(script, "");
+    TEST( m_test->setClipboard("TEST") );
+    WAIT_ON_OUTPUT("read" << "0", "123");
+}
+
+void Tests::automaticCommandCopyToTab()
+{
+    const auto tab1 = testTab(1);
+    const auto script = R"(
+        setCommands([{automatic: true, tab: ')" + tab1 + R"('}])
+        )";
+    RUN(script, "");
+    TEST( m_test->setClipboard("TEST") );
+    WAIT_ON_OUTPUT("tab" << QString(clipboardTabName) << "read" << "0", "TEST");
+    RUN("tab" << tab1 << "read" << "0", "TEST");
+}
+
+void Tests::scriptCommandLoaded()
+{
+    const auto script = R"(
+        setCommands([{
+            isScript: true,
+            cmd: 'add("LOADED")'
+        }])
+        )";
+    RUN(script, "");
+    RUN("read(0)", "LOADED");
+}
+
+void Tests::scriptCommandAddFunction()
+{
+    const auto script = R"(
+        setCommands([{
+            isScript: true,
+            cmd: 'test = function() { return "TEST"; }'
+        }])
+        )";
+    RUN(script, "");
+    RUN("test", "TEST\n");
+}
+
+void Tests::scriptCommandOverrideFunction()
+{
+    const auto script = R"(
+        setCommands([{
+            isScript: true,
+            cmd: 'popup = function(msg) { return msg; }'
+        }])
+        )";
+    RUN(script, "");
+    RUN("popup" << "test" << "xxx", "test");
+}
+
+void Tests::displayCommand()
+{
+    const auto script = QString(R"(
+        setCommands([{
+            display: true,
+            input: '%1',
+            cmd: 'copyq:'
+               + 'var data = unpack(input());'
+               + 'text = str(data[mimeText]);'
+               + 'if (text !== "a" && text != "b") abort();'
+               + 'add("CALLED in " + str(data[mimeCurrentTab]));'
+        }])
+        )").arg(mimeItems);
+    RUN("add('b', 'a'); " + QString(script), "");
+    WAIT_ON_OUTPUT(
+                "read(0,1,2,3,4)",
+                QString("CALLED in %1\nCALLED in %1\na\nb\n")
+                .arg(clipboardTabName)
+                .toUtf8() );
+}
+
 int Tests::run(const QStringList &arguments, QByteArray *stdoutData, QByteArray *stderrData, const QByteArray &in)
 {
     return m_test->run(arguments, stdoutData, stderrData, in);
@@ -2400,6 +2537,7 @@ int runTests(int argc, char *argv[])
 
     if (runPluginTests) {
         ItemFactory itemFactory;
+        itemFactory.loadPlugins();
         for ( const auto &loader : itemFactory.loaders() ) {
             if ( loader->id().contains(onlyPlugins) ) {
                 std::unique_ptr<QObject> pluginTests( loader->tests(test) );
