@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -25,7 +25,6 @@
 #include <QObject>
 #include <QStringList>
 
-class RemoteProcess;
 class QProcess;
 class QByteArray;
 
@@ -116,6 +115,8 @@ private slots:
     void commandScreenshot();
 
     void commandIcon();
+    void commandIconTag();
+    void commandIconTagColor();
 
     void classFile();
     void classDir();
@@ -187,11 +188,19 @@ private slots:
     void shortcutCommandSetSelectedItemsData();
 
     void automaticCommandIgnore();
+    void automaticCommandRemove();
     void automaticCommandInput();
     void automaticCommandRegExp();
     void automaticCommandSetData();
     void automaticCommandOutputTab();
     void automaticCommandNoOutputTab();
+    void automaticCommandChaining();
+    void automaticCommandCopyToTab();
+
+    void scriptCommandLoaded();
+    void scriptCommandAddFunction();
+    void scriptCommandOverrideFunction();
+    void displayCommand();
 
 private:
     void clearServerErrors();
